@@ -21,7 +21,7 @@
 #include "Python.h"
 #include <math.h>
 
-#define PYO_VERSION "0.7.9"
+#define PYO_VERSION "0.8.2"
 
 #ifndef __MYFLT_DEF
 #define __MYFLT_DEF
@@ -73,6 +73,8 @@
 #define TYPE_F_OF "f|Of"
 #define TYPE__OFFI "|Offi"
 #define TYPE__OFII "|Ofii"
+#define TYPE__OFIOO "|OfiOO"
+#define TYPE__OOFOO "|OOfOO"
 #define TYPE__FIIOO "|fiiOO"
 #define TYPE_O_OFOO "O|OfOO"
 #define TYPE_O_OOOOFF "O|OOOOff"
@@ -87,6 +89,7 @@
 #define TYPE_O_OOFFOO "O|OOffOO"
 #define TYPE_O_OOOFOO "O|OOOfOO"
 #define TYPE_OO_OOOIFOO "OO|OOOifOO"
+#define TYPE__FFFFIFI "|ffffifi"
 
 #define SF_WRITE sf_write_float
 #define SF_READ sf_read_float
@@ -156,6 +159,8 @@
 #define TYPE_F_OF "d|Od"
 #define TYPE__OFFI "|Oddi"
 #define TYPE__OFII "|Odii"
+#define TYPE__OFIOO "|OdiOO"
+#define TYPE__OOFOO "|OOdOO"
 #define TYPE__FIIOO "|diiOO"
 #define TYPE_O_OFOO "O|OdOO"
 #define TYPE_O_OOOOFF "O|OOOOdd"
@@ -170,6 +175,7 @@
 #define TYPE_O_OOFFOO "O|OOddOO"
 #define TYPE_O_OOOFOO "O|OOOdOO"
 #define TYPE_OO_OOOIFOO "OO|OOOidOO"
+#define TYPE__FFFFIFI "|ddddidi"
 
 #define SF_WRITE sf_write_double
 #define SF_READ sf_read_double
@@ -199,9 +205,21 @@
 #include "externalmodule.h"
 #endif
 
+#ifdef USE_PORTMIDI
 extern PyTypeObject MidiListenerType;
+#endif
+#ifdef USE_OSC
 extern PyTypeObject OscListenerType;
+extern PyTypeObject OscSendType;
+extern PyTypeObject OscReceiveType;
+extern PyTypeObject OscReceiverType;
+extern PyTypeObject OscDataSendType;
+extern PyTypeObject OscDataReceiveType;
+extern PyTypeObject OscListReceiveType;
+extern PyTypeObject OscListReceiverType;
+#endif
 extern PyTypeObject SineType;
+extern PyTypeObject FastSineType;
 extern PyTypeObject SineLoopType;
 extern PyTypeObject FmType;
 extern PyTypeObject CrossFmType;
@@ -211,6 +229,8 @@ extern PyTypeObject RosslerType;
 extern PyTypeObject RosslerAltType;
 extern PyTypeObject LorenzType;
 extern PyTypeObject LorenzAltType;
+extern PyTypeObject ChenLeeType;
+extern PyTypeObject ChenLeeAltType;
 extern PyTypeObject PhasorType;
 extern PyTypeObject SuperSawType;
 extern PyTypeObject PointerType;
@@ -303,6 +323,7 @@ extern PyTypeObject IRAverageType;
 extern PyTypeObject IRFMType;
 extern PyTypeObject GranulatorType;
 extern PyTypeObject LooperType;
+extern PyTypeObject LooperTimeStreamType;
 extern PyTypeObject HarmonizerType;
 extern PyTypeObject MidictlType;
 extern PyTypeObject CtlScanType;
@@ -355,13 +376,6 @@ extern PyTypeObject MatrixPointerType;
 extern PyTypeObject MatrixRecType;
 extern PyTypeObject MatrixRecLoopType;
 extern PyTypeObject MatrixMorphType;
-extern PyTypeObject OscSendType;
-extern PyTypeObject OscReceiveType;
-extern PyTypeObject OscReceiverType;
-extern PyTypeObject OscDataSendType;
-extern PyTypeObject OscDataReceiveType;
-extern PyTypeObject OscListReceiveType;
-extern PyTypeObject OscListReceiverType;
 extern PyTypeObject TrigRandIntType;
 extern PyTypeObject TrigValType;
 extern PyTypeObject TrigRandType;
@@ -420,6 +434,7 @@ extern PyTypeObject M_FloorType;
 extern PyTypeObject M_CeilType;
 extern PyTypeObject M_RoundType;
 extern PyTypeObject M_TanhType;
+extern PyTypeObject M_ExpType;
 extern PyTypeObject FFTMainType;
 extern PyTypeObject FFTType;
 extern PyTypeObject IFFTType;
@@ -446,6 +461,7 @@ extern PyTypeObject ButLPType;
 extern PyTypeObject ButHPType;
 extern PyTypeObject ButBPType;
 extern PyTypeObject ButBRType;
+extern PyTypeObject MoogLPType;
 extern PyTypeObject PVAnalType;
 extern PyTypeObject PVSynthType;
 extern PyTypeObject PVTransposeType;
@@ -488,13 +504,15 @@ extern PyTypeObject AtanTableType;
 extern PyTypeObject RawMidiType;
 extern PyTypeObject ResampleType;
 extern PyTypeObject ExprType;
+extern PyTypeObject PadSynthTableType;
+extern PyTypeObject LogiMapType;
 
 /* Constants */
 #define E M_E
 #define PI M_PI
 #define TWOPI (2 * M_PI)
 
-#define PYO_RAND_MAX 4294967295
+#define PYO_RAND_MAX 4294967295u
 
 /* random uniform (0.0 -> 1.0) */
 #define RANDOM_UNIFORM (pyorand()/((MYFLT)(PYO_RAND_MAX)+1))
@@ -975,6 +993,71 @@ extern PyTypeObject ExprType;
     Py_INCREF(Py_None); \
     return Py_None; \
 
+/* Table rotation */
+#define TABLE_ROTATE \
+    int i, j, pos; \
+    MYFLT tmp; \
+    static char *kwlist[] = {"pos", NULL}; \
+ \
+    if (! PyArg_ParseTupleAndKeywords(args, kwds, "i", kwlist, &pos)) \
+        return PyInt_FromLong(-1); \
+ \
+    pos = -pos; \
+    while (pos > self->size) pos -= self->size; \
+    while (pos < 0) pos += self->size; \
+ \
+    j = self->size; \
+    for (i=0; i<--j; i++) { \
+        tmp = self->data[i]; \
+        self->data[i] = self->data[j]; \
+        self->data[j] = tmp; \
+    } \
+    j = pos; \
+    for (i=0; i<--j; i++) { \
+        tmp = self->data[i]; \
+        self->data[i] = self->data[j]; \
+        self->data[j] = tmp; \
+    } \
+    j = self->size; \
+    for (i=pos; i<--j; i++) { \
+        tmp = self->data[i]; \
+        self->data[i] = self->data[j]; \
+        self->data[j] = tmp; \
+    } \
+ \
+    self->data[self->size] = self->data[0]; \
+ \
+    Py_RETURN_NONE;
+
+/* Table copy from table */
+#define TABLE_COPYDATA \
+    PyObject *tabletmp; \
+    int i, tabsize, srcpos=0, destpos=0, length=-1; \
+    PyObject *table = NULL; \
+    MYFLT *list = NULL; \
+    static char *kwlist[] = {"table", "srcpos", "destpos", "length", NULL}; \
+ \
+    if (! PyArg_ParseTupleAndKeywords(args, kwds, "O|iii", kwlist, &tabletmp, &srcpos, &destpos, &length)) \
+        return PyInt_FromLong(-1); \
+ \
+    if ( PyObject_HasAttrString((PyObject *)tabletmp, "getTableStream") == 1 ) { \
+        Py_XDECREF(table); \
+        table = PyObject_CallMethod((PyObject *)tabletmp, "getTableStream", ""); \
+        tabsize = TableStream_getSize((TableStream *)table); \
+        if (length < 0) \
+            length = tabsize < self->size ? tabsize : self->size; \
+        if ((srcpos + length) > tabsize) \
+            length = tabsize - srcpos; \
+        if ((destpos + length) > self->size) \
+            length = self->size - destpos; \
+        list = TableStream_getData((TableStream *)table); \
+        for (i=0; i<length; i++) { \
+            self->data[destpos+i] = list[srcpos+i]; \
+        } \
+    } \
+ \
+    Py_RETURN_NONE;
+
 /* Table bipolar gain */
 #define TABLE_BIPOLAR_GAIN \
     MYFLT gpos = 1.0, gneg = 1.0; \
@@ -1412,7 +1495,7 @@ extern PyTypeObject ExprType;
     if (isNumber == 1) { \
         if (PyFloat_AsDouble(tmp) != 0.) { \
             Py_DECREF(self->mul); \
-            self->mul = PyNumber_Divide(PyFloat_FromDouble(1.), PyNumber_Float(tmp)); \
+            self->mul = PyNumber_TrueDivide(PyFloat_FromDouble(1.), PyNumber_Float(tmp)); \
             self->modebuffer[0] = 0; \
         } \
     } \
@@ -1674,3 +1757,39 @@ extern PyTypeObject ExprType;
         val = old / tmp - add[i]; \
         self->data[i] = val; \
     }
+
+/* Tables buffer protocol. */
+#if PY_MAJOR_VERSION < 3
+#define TABLESTREAM_READ_WRITE_BUFFER \
+    if ( index != 0 ) { \
+        PySys_WriteStdout("Accessing non-existent bytes segment..."); \
+        return -1; \
+    } \
+    *ptr = (void *)self->data; \
+    return (Py_ssize_t)self->size * sizeof(MYFLT);
+
+#define TABLESTREAM_SEG_COUNT \
+    if ( lenp ) \
+        *lenp = (Py_ssize_t)self->size * sizeof(MYFLT); \
+    return 1;
+#endif
+
+#define TABLESTREAM_GET_BUFFER \
+    if (view == NULL) { \
+        PySys_WriteStdout("NULL view in getBuffer."); \
+        return -1; \
+    } \
+    self->shape[0] = self->size; \
+    view->obj = (PyObject *)self; \
+    view->buf = (void *)self->data; \
+    view->len = self->size * sizeof(MYFLT); \
+    view->readonly = 0; \
+    view->itemsize = sizeof(MYFLT); \
+    view->format = TYPE_F; \
+    view->ndim = 1; \
+    view->shape = self->shape; \
+    view->strides = NULL; \
+    view->suboffsets = NULL; \
+    view->internal = NULL; \
+    Py_INCREF(self); \
+    return 0;
