@@ -17,9 +17,9 @@
  * You should have received a copy of the GNU Lesser General Public       *
  * License along with pyo.  If not, see <http://www.gnu.org/licenses/>.   *
  *                                                                        *
- * Octobre 2013 : Multiple servers facility and embeded backend added by  *
- * Guillaume Barrette. See embeded possibilities at:                      *
- * https://github.com/guibarrette/PyoPlug                                 *
+ * Octobre 2013 : Multiple servers facility and embedded backend added    *
+ * by Guillaume Barrette.                                                 *
+ * 2014-2016 : Several improvements by Olivier Belanger.                  *
  *************************************************************************/
 
 #ifndef Py_SERVERMODULE_H
@@ -28,19 +28,9 @@
 extern "C" {
 #endif
 
-#include "portaudio.h"
-#include "portmidi.h"
 #include "sndfile.h"
 #include "pyomodule.h"
 
-#ifdef USE_JACK
-#include <jack/jack.h>
-#endif
-
-#ifdef USE_COREAUDIO
-# include <CoreAudio/AudioHardware.h>
-#endif
-
 typedef enum {
     PyoPortaudio = 0,
     PyoCoreaudio = 1,
@@ -50,33 +40,45 @@ typedef enum {
     PyoEmbedded
 } PyoAudioBackendType;
 
-typedef struct {
-    PaStream *stream;
-} PyoPaBackendData;
+typedef enum {
+    PyoPortmidi = 0
+} PyoMidiBackendType;
 
+/* Portmidi midi message and event type clones. */
+
+typedef long PyoMidiTimestamp;
+
+#define PyoMidi_Message(status, data1, data2) \
+         ((((data2) << 16) & 0xFF0000) | \
+          (((data1) << 8) & 0xFF00) | \
+          ((status) & 0xFF))
+#define PyoMidi_MessageStatus(msg) ((msg) & 0xFF)
+#define PyoMidi_MessageData1(msg) (((msg) >> 8) & 0xFF)
+#define PyoMidi_MessageData2(msg) (((msg) >> 16) & 0xFF)
+
+typedef long PyoMidiMessage; 
 typedef struct {
-#ifdef USE_JACK
-    jack_client_t *jack_client;
-    jack_port_t **jack_in_ports;
-    jack_port_t **jack_out_ports;
-#endif
-} PyoJackBackendData;
+    PyoMidiMessage      message;
+    PyoMidiTimestamp    timestamp;
+} PyoMidiEvent;
+
+/************************************************/
 
 typedef struct {
     PyObject_HEAD
     PyObject *streams;
     PyoAudioBackendType audio_be_type;
+    PyoMidiBackendType midi_be_type;
     void *audio_be_data;
+    void *midi_be_data;
     char *serverName; /* Only used for jack client name */
     int jackautoin; /* jack port auto-connection (on by default) */
     int jackautoout; /* jack port auto-connection (on by default) */
-    PyObject *jackAutoConnectInputPorts; /* list of regex to match for jack auto-connection */
-    PyObject *jackAutoConnectOutputPorts; /* list of regex to match for jack auto-connection */
-    PmStream *midiin[64];
-    PmStream *midiout[64];
+    PyObject *jackAutoConnectInputPorts; /* list of lists of jack auto-connection ports to pyo inputs */
+    PyObject *jackAutoConnectOutputPorts; /* list of lists of jack auto-connection ports from pyo outputs */
+    PyoMidiEvent midiEvents[200];
     int midiin_count;
     int midiout_count;
-    PmEvent midiEvents[200];
     int midi_count;
     double samplingRate;
     int nchnls;
@@ -121,6 +123,7 @@ typedef struct {
     char *recpath;
     int recformat;
     int rectype;
+    double recquality;
     SNDFILE *recfile;
     SF_INFO recinfo;
 
@@ -138,6 +141,9 @@ typedef struct {
     int tcount;
     PyObject *TIME;
 
+    /* custom callback */
+    PyObject *CALLBACK;
+
     /* Properties */
     int verbosity; /* a sum of values to display different levels: 1 = error */
                    /* 2 = message, 4 = warning , 8 = debug. Default 7.*/
@@ -148,15 +154,25 @@ PyObject * PyServer_get_server();
 extern unsigned int pyorand();
 extern PyObject * Server_removeStream(Server *self, int sid);
 extern MYFLT * Server_getInputBuffer(Server *self);
-extern PmEvent * Server_getMidiEventBuffer(Server *self);
+extern PyoMidiEvent * Server_getMidiEventBuffer(Server *self);
 extern int Server_getMidiEventCount(Server *self);
 extern int Server_getCurrentResamplingFactor(Server *self);
 extern int Server_getLastResamplingFactor(Server *self);
 extern int Server_generateSeed(Server *self, int oid);
 extern PyTypeObject ServerType;
+void pyoGetMidiEvents(Server *self);
+void Server_process_buffers(Server *server);
+void Server_error(Server *self, char * format, ...);
+void Server_message(Server *self, char * format, ...);
+void Server_warning(Server *self, char * format, ...);
+void Server_debug(Server *self, char * format, ...);
+PyObject * Server_shut_down(Server *self);
+PyObject *Server_stop(Server *self);
+void Server_process_gui(Server *server);
+void Server_process_time(Server *server);
+int Server_start_rec_internal(Server *self, char *filename);
 
 #ifdef __cplusplus
 }
-#endif
-
-#endif /* !defined(Py_SERVERMODULE_H) */
+#endif /* __cplusplus */
+#endif /* Py_SERVERMODULE_H */
