@@ -1,3 +1,5 @@
+from __future__ import division
+from __future__ import absolute_import
 """
 Tools to analyze audio signals.
 
@@ -27,10 +29,10 @@ You should have received a copy of the GNU Lesser General Public
 License along with pyo.  If not, see <http://www.gnu.org/licenses/>.
 """
 
-from _core import *
-from _maps import *
-from _widgets import createSpectrumWindow, createScopeWindow
-from pattern import Pattern
+from ._core import *
+from ._maps import *
+from ._widgets import createSpectrumWindow, createScopeWindow
+from .pattern import Pattern
 
 class Follower(PyoObject):
     """
@@ -42,9 +44,9 @@ class Follower(PyoObject):
 
     :Args:
 
-        input : PyoObject
+        input: PyoObject
             Input signal to process.
-        freq : float or PyoObject, optional
+        freq: float or PyoObject, optional
             Cutoff frequency of the filter in hertz. Default to 20.
 
     .. note::
@@ -52,6 +54,10 @@ class Follower(PyoObject):
         The out() method is bypassed. Follower's signal can not be sent to
         audio outs.
 
+    .. seealso::
+
+        :py:class:`Follower2`, :py:class: `Balance`
+
     >>> s = Server().boot()
     >>> s.start()
     >>> sf = SfPlayer(SNDS_PATH + "/transparent.aif", loop=True, mul=.4).out()
@@ -74,9 +80,9 @@ class Follower(PyoObject):
 
         :Args:
 
-            x : PyoObject
+            x: PyoObject
                 New signal to process.
-            fadetime : float, optional
+            fadetime: float, optional
                 Crossfade time between old and new input. Default to 0.05.
 
         """
@@ -90,7 +96,7 @@ class Follower(PyoObject):
 
         :Args:
 
-            x : float or PyoObject
+            x: float or PyoObject
                 New `freq` attribute.
 
         """
@@ -130,11 +136,11 @@ class Follower2(PyoObject):
 
     :Args:
 
-        input : PyoObject
+        input: PyoObject
             Input signal to process.
-        risetime : float or PyoObject, optional
+        risetime: float or PyoObject, optional
             Time to reach upward value in seconds. Default to 0.01.
-        falltime : float or PyoObject, optional
+        falltime: float or PyoObject, optional
             Time to reach downward value in seconds. Default to 0.1.
 
     .. note::
@@ -142,6 +148,10 @@ class Follower2(PyoObject):
         The out() method is bypassed. Follower's signal can not be sent to
         audio outs.
 
+    .. seealso::
+
+        :py:class:`Follower`, :py:class: `Balance`
+
     >>> s = Server().boot()
     >>> s.start()
     >>> sf = SfPlayer(SNDS_PATH + "/transparent.aif", loop=True, mul=.4).out()
@@ -165,9 +175,9 @@ class Follower2(PyoObject):
 
         :Args:
 
-            x : PyoObject
+            x: PyoObject
                 New signal to process.
-            fadetime : float, optional
+            fadetime: float, optional
                 Crossfade time between old and new input. Default to 0.05.
 
         """
@@ -181,7 +191,7 @@ class Follower2(PyoObject):
 
         :Args:
 
-            x : float or PyoObject
+            x: float or PyoObject
                 New `risetime` attribute.
 
         """
@@ -196,7 +206,7 @@ class Follower2(PyoObject):
 
         :Args:
 
-            x : float or PyoObject
+            x: float or PyoObject
                 New `falltime` attribute.
 
         """
@@ -244,9 +254,9 @@ class ZCross(PyoObject):
 
     :Args:
 
-        input : PyoObject
+        input: PyoObject
             Input signal to process.
-        thresh : float, optional
+        thresh: float, optional
             Minimum amplitude difference allowed between adjacent samples
             to be included in the zeros count.
 
@@ -277,9 +287,9 @@ class ZCross(PyoObject):
 
         :Args:
 
-            x : PyoObject
+            x: PyoObject
                 New signal to process.
-            fadetime : float, optional
+            fadetime: float, optional
                 Crossfade time between old and new input. Default to 0.05.
 
         """
@@ -293,7 +303,7 @@ class ZCross(PyoObject):
 
         :Args:
 
-            x : float
+            x: float
                 New amplitude difference threshold.
 
         """
@@ -339,23 +349,23 @@ class Yin(PyoObject):
 
     :Args:
 
-        input : PyoObject
+        input: PyoObject
             Input signal to process.
-        tolerance : float, optional
+        tolerance: float, optional
             Parameter for minima selection, between 0 and 1. Defaults to 0.2.
-        minfreq : float, optional
+        minfreq: float, optional
             Minimum estimated frequency in Hz. Frequency below this threshold will
             be ignored. Defaults to 40.
-        maxfreq : float, optional
+        maxfreq: float, optional
             Maximum estimated frequency in Hz. Frequency above this threshold will
             be ignored. Defaults to 1000.
-        cutoff : float, optional
+        cutoff: float, optional
             Cutoff frequency, in Hz, of the lowpass filter applied on the input sound.
             Defaults to 1000.
 
             The lowpass filter helps the algorithm to detect the fundamental frequency by filtering
             higher harmonics.
-        winsize : int, optional
+        winsize: int, optional
             Size, in samples, of the analysis window. Must be higher that two period
             of the lowest desired frequency.
 
@@ -390,9 +400,9 @@ class Yin(PyoObject):
 
         :Args:
 
-            x : PyoObject
+            x: PyoObject
                 New signal to process.
-            fadetime : float, optional
+            fadetime: float, optional
                 Crossfade time between old and new input. Default to 0.05.
 
         """
@@ -406,7 +416,7 @@ class Yin(PyoObject):
 
         :Args:
 
-            x : float
+            x: float
                 New parameter for minima selection, between 0 and 1.
 
         """
@@ -421,7 +431,7 @@ class Yin(PyoObject):
 
         :Args:
 
-            x : float
+            x: float
                 New minimum frequency detected.
 
         """
@@ -436,7 +446,7 @@ class Yin(PyoObject):
 
         :Args:
 
-            x : float
+            x: float
                 New maximum frequency detected.
 
         """
@@ -451,7 +461,7 @@ class Yin(PyoObject):
 
         :Args:
 
-            x : float
+            x: float
                 New input lowpass filter cutoff frequency.
 
         """
@@ -520,9 +530,9 @@ class Centroid(PyoObject):
 
     :Args:
 
-        input : PyoObject
+        input: PyoObject
             Input signal to process.
-        size : int, optional
+        size: int, optional
             Size, as a power-of-two, of the FFT used to compute the centroid.
 
             Available at initialization time only.  Defaults to 1024.
@@ -556,9 +566,9 @@ class Centroid(PyoObject):
 
         :Args:
 
-            x : PyoObject
+            x: PyoObject
                 New signal to process.
-            fadetime : float, optional
+            fadetime: float, optional
                 Crossfade time between old and new input. Default to 0.05.
 
         """
@@ -594,23 +604,23 @@ class AttackDetector(PyoObject):
 
     :Args:
 
-        input : PyoObject
+        input: PyoObject
             Input signal to process.
-        deltime : float, optional
+        deltime: float, optional
             Delay time, in seconds, between previous and current rms analysis to compare.
             Defaults to 0.005.
-        cutoff : float, optional
+        cutoff: float, optional
             Cutoff frequency, in Hz, of the amplitude follower's lowpass filter.
             Defaults to 10.
 
             Higher values are more responsive and also more likely to give false onsets.
-        maxthresh : float, optional
+        maxthresh: float, optional
             Attack threshold in positive dB (current rms must be higher than previous
             rms + maxthresh to be reported as an attack). Defaults to 3.0.
-        minthresh : float, optional
+        minthresh: float, optional
             Minimum threshold in dB (signal must fall below this threshold to allow
             a new attack to be detected). Defaults to -30.0.
-        reltime : float, optional
+        reltime: float, optional
             Time, in seconds, to wait before reporting a new attack. Defaults to 0.1.
 
 
@@ -641,9 +651,9 @@ class AttackDetector(PyoObject):
 
         :Args:
 
-            x : PyoObject
+            x: PyoObject
                 New signal to process.
-            fadetime : float, optional
+            fadetime: float, optional
                 Crossfade time between old and new input. Default to 0.05.
 
         """
@@ -657,7 +667,7 @@ class AttackDetector(PyoObject):
 
         :Args:
 
-            x : float
+            x: float
                 New delay between rms analysis.
 
         """
@@ -672,7 +682,7 @@ class AttackDetector(PyoObject):
 
         :Args:
 
-            x : float
+            x: float
                 New cutoff for the follower lowpass filter.
 
         """
@@ -687,7 +697,7 @@ class AttackDetector(PyoObject):
 
         :Args:
 
-            x : float
+            x: float
                 New attack threshold in dB.
 
         """
@@ -702,7 +712,7 @@ class AttackDetector(PyoObject):
 
         :Args:
 
-            x : float
+            x: float
                 New minimum threshold in dB.
 
         """
@@ -717,7 +727,7 @@ class AttackDetector(PyoObject):
 
         :Args:
 
-            x : float
+            x: float
                 Time, in seconds, to wait before reporting a new attack.
 
         """
@@ -732,7 +742,7 @@ class AttackDetector(PyoObject):
 
         """
         [obj.readyToDetect() for obj in self._base_objs]
-        
+
     def out(self, chnl=0, inc=1, dur=0, delay=0):
         return self.play(dur, delay)
 
@@ -798,13 +808,13 @@ class Spectrum(PyoObject):
 
     :Args:
 
-        input : PyoObject
+        input: PyoObject
             Input signal to process.
-        size : int {pow-of-two > 4}, optional
+        size: int {pow-of-two > 4}, optional
             FFT size. Must be a power of two greater than 4.
             The FFT size is the number of samples used in each
             analysis frame. Defaults to 1024.
-        wintype : int, optional
+        wintype: int, optional
             Shape of the envelope used to filter each input frame.
             Possible shapes are :
                 0. rectangular (no windowing)
@@ -816,7 +826,7 @@ class Spectrum(PyoObject):
                 6. Blackman-Harris 7-term
                 7. Tuckey (alpha = 0.66)
                 8. Sine (half-sine window)
-        function : python callable, optional
+        function: python callable, optional
             If set, this function will be called with magnitudes (as
             list of lists, one list per channel). Useful if someone
             wants to save the analysis data into a text file.
@@ -854,7 +864,7 @@ class Spectrum(PyoObject):
         in_fader, size, wintype, lmax = convertArgsToLists(self._in_fader, size, wintype)
         self._base_objs = [Spectrum_base(wrap(in_fader,i), wrap(size,i), wrap(wintype,i)) for i in range(lmax)]
         self._timer = Pattern(self.refreshView, 0.05).play()
-        if function == None:
+        if function is None:
             self.view()
 
     def setInput(self, x, fadetime=0.05):
@@ -863,9 +873,9 @@ class Spectrum(PyoObject):
 
         :Args:
 
-            x : PyoObject
+            x: PyoObject
                 New signal to process.
-            fadetime : float, optional
+            fadetime: float, optional
                 Crossfade time between old and new input. Default to 0.05.
 
         """
@@ -879,7 +889,7 @@ class Spectrum(PyoObject):
 
         :Args:
 
-            x : int
+            x: int
                 new `size` attribute.
 
         """
@@ -894,7 +904,7 @@ class Spectrum(PyoObject):
 
         :Args:
 
-            x : int
+            x: int
                 new `wintype` attribute.
 
         """
@@ -909,7 +919,7 @@ class Spectrum(PyoObject):
 
         :Args:
 
-            function : python callable
+            function: python callable
                 The function called by the internal timer to retrieve the
                 analysis data. The function must be created with one argument
                 and will receive the data as a list of lists (one list per channel).
@@ -924,7 +934,7 @@ class Spectrum(PyoObject):
 
         :Args:
 
-            active : boolean
+            active: boolean
                 If True, starts the analysis polling, False to stop it.
                 defaults to True.
 
@@ -941,7 +951,7 @@ class Spectrum(PyoObject):
 
         :Args:
 
-            time : float
+            time: float
                 Adjusts the frequency of the internal timer used to
                 retrieve the current analysis frame. defaults to 0.05.
 
@@ -955,7 +965,7 @@ class Spectrum(PyoObject):
 
         :Args:
 
-            x : float
+            x: float
                 New low frequency in Hz. Adjusts the `lowbound` attribute, as `x / sr`.
 
         """
@@ -971,7 +981,7 @@ class Spectrum(PyoObject):
 
         :Args:
 
-            x : float
+            x: float
                 New high frequency in Hz. Adjusts the `highbound` attribute, as `x / sr`.
 
         """
@@ -989,7 +999,7 @@ class Spectrum(PyoObject):
 
         :Args:
 
-            x : float {0 <= x <= 0.5}
+            x: float {0 <= x <= 0.5}
                 new `lowbound` attribute.
 
         """
@@ -1007,7 +1017,7 @@ class Spectrum(PyoObject):
 
         :Args:
 
-            x : float {0 <= x <= 0.5}
+            x: float {0 <= x <= 0.5}
                 new `highbound` attribute.
 
         """
@@ -1040,7 +1050,7 @@ class Spectrum(PyoObject):
 
         :Args:
 
-            x : int
+            x: int
                 new `width` attribute.
 
         """
@@ -1057,7 +1067,7 @@ class Spectrum(PyoObject):
 
         :Args:
 
-            x : int
+            x: int
                 new `height` attribute.
 
         """
@@ -1072,7 +1082,7 @@ class Spectrum(PyoObject):
 
         :Args:
 
-            x : boolean
+            x: boolean
                 If True, the frequency display is logarithmic. False turns
                 it back to linear. Defaults to False.
 
@@ -1081,7 +1091,7 @@ class Spectrum(PyoObject):
         self._fscaling = x
         x, lmax = convertArgsToLists(x)
         [obj.setFscaling(wrap(x,i)) for i, obj in enumerate(self._base_objs)]
-        if self.viewFrame != None:
+        if self.viewFrame is not None:
             self.viewFrame.setFscaling(self._fscaling)
 
     def setMscaling(self, x):
@@ -1090,7 +1100,7 @@ class Spectrum(PyoObject):
 
         :Args:
 
-            x : boolean
+            x: boolean
                 If True, the magnitude display is logarithmic (which means in dB).
                 False turns it back to linear. Defaults to True.
 
@@ -1099,7 +1109,7 @@ class Spectrum(PyoObject):
         self._mscaling = x
         x, lmax = convertArgsToLists(x)
         [obj.setMscaling(wrap(x,i)) for i, obj in enumerate(self._base_objs)]
-        if self.viewFrame != None:
+        if self.viewFrame is not None:
             self.viewFrame.setMscaling(self._mscaling)
 
     def getFscaling(self):
@@ -1126,7 +1136,7 @@ class Spectrum(PyoObject):
 
         :Args:
 
-            x : float
+            x: float
                 new `gain` attribute, as linear values.
 
         """
@@ -1141,9 +1151,9 @@ class Spectrum(PyoObject):
 
         :Args:
 
-            title : string, optional
+            title: string, optional
                 Window title. Defaults to "Spectrum".
-            wxnoserver : boolean, optional
+            wxnoserver: boolean, optional
                 With wxPython graphical toolkit, if True, tells the
                 interpreter that there will be no server window.
 
@@ -1165,9 +1175,9 @@ class Spectrum(PyoObject):
 
         """
         self.points = [obj.display() for obj in self._base_objs]
-        if self._function != None:
+        if self._function is not None:
             self._function(self.points)
-        if self.viewFrame != None:
+        if self.viewFrame is not None:
             self.viewFrame.update(self.points)
 
     @property
@@ -1251,15 +1261,15 @@ class Scope(PyoObject):
 
     :Args:
 
-        input : PyoObject
+        input: PyoObject
             Input signal to process.
-        length : float, optional
+        length: float, optional
             Length, in seconds, of the displayed window. Can't be a list.
             Defaults to 0.05.
-        gain : float, optional
+        gain: float, optional
             Linear gain applied to the signal to be displayed.
             Can't be a list. Defaults to 0.67.
-        function : python callable, optional
+        function: python callable, optional
             If set, this function will be called with samples (as
             list of lists, one list per channel). Useful if someone
             wants to save the analysis data into a text file.
@@ -1293,7 +1303,7 @@ class Scope(PyoObject):
         in_fader, lmax = convertArgsToLists(self._in_fader)
         self._base_objs = [Scope_base(wrap(in_fader,i), length) for i in range(lmax)]
         self._timer = Pattern(self.refreshView, length).play()
-        if function == None:
+        if function is None:
             self.view()
 
     def setInput(self, x, fadetime=0.05):
@@ -1302,9 +1312,9 @@ class Scope(PyoObject):
 
         :Args:
 
-            x : PyoObject
+            x: PyoObject
                 New signal to process.
-            fadetime : float, optional
+            fadetime: float, optional
                 Crossfade time between old and new input. Default to 0.05.
 
         """
@@ -1318,7 +1328,7 @@ class Scope(PyoObject):
 
         :Args:
 
-            x : float
+            x: float
                 new `length` attribute.
 
         """
@@ -1333,7 +1343,7 @@ class Scope(PyoObject):
 
         :Args:
 
-            x : float
+            x: float
                 new `gain` attribute, as linear values.
 
         """
@@ -1348,7 +1358,7 @@ class Scope(PyoObject):
 
         :Args:
 
-            active : boolean
+            active: boolean
                 If True, starts the analysis polling, False to stop it.
                 defaults to True.
 
@@ -1368,7 +1378,7 @@ class Scope(PyoObject):
 
         :Args:
 
-            x : int
+            x: int
                 Width of the display in pixel value. The default
                 width is 500.
 
@@ -1386,7 +1396,7 @@ class Scope(PyoObject):
 
         :Args:
 
-            x : int
+            x: int
                 Height of the display in pixel value. The default
                 height is 400.
 
@@ -1401,9 +1411,9 @@ class Scope(PyoObject):
 
         :Args:
 
-            title : string, optional
+            title: string, optional
                 Window title. Defaults to "Spectrum".
-            wxnoserver : boolean, optional
+            wxnoserver: boolean, optional
                 With wxPython graphical toolkit, if True, tells the
                 interpreter that there will be no server window.
 
@@ -1420,7 +1430,7 @@ class Scope(PyoObject):
 
         :Args:
 
-            function : python callable
+            function: python callable
                 The function called by the internal timer to retrieve the
                 analysis data. The function must be created with one argument
                 and will receive the data as a list of lists (one list per channel).
@@ -1440,7 +1450,7 @@ class Scope(PyoObject):
 
         """
         self.points = [obj.display() for obj in self._base_objs]
-        if self.viewFrame != None:
+        if self.viewFrame is not None:
             self.viewFrame.update(self.points)
         if self._function is not None:
             self._function(self.points)
@@ -1483,9 +1493,9 @@ class PeakAmp(PyoObject):
 
     :Args:
 
-        input : PyoObject
+        input: PyoObject
             Input signal to process.
-        function : callable, optional
+        function: callable, optional
             Function that will be called with amplitude values in arguments.
             Default to None.
 
@@ -1522,9 +1532,9 @@ class PeakAmp(PyoObject):
 
         :Args:
 
-            x : PyoObject
+            x: PyoObject
                 New signal to process.
-            fadetime : float, optional
+            fadetime: float, optional
                 Crossfade time between old and new input. Default to 0.05.
 
         """
@@ -1538,7 +1548,7 @@ class PeakAmp(PyoObject):
 
         :Args:
 
-            x : callable
+            x: callable
                 New function to call with amplitude values in arguments.
 
         """
@@ -1552,7 +1562,7 @@ class PeakAmp(PyoObject):
 
         :Args:
 
-            x : float
+            x: float
                 New polling time in seconds.
 
         """
@@ -1563,7 +1573,7 @@ class PeakAmp(PyoObject):
         return self.play(dur, delay)
 
     def _buildList(self):
-        if self._function != None:
+        if self._function is not None:
             values = [obj.getValue() for obj in self._base_objs]
             self._function(*values)
 
@@ -1583,4 +1593,4 @@ class PeakAmp(PyoObject):
         """PyoObject. function signal to process."""
         return self._function
     @function.setter
-    def function(self, x): self.setFunction(x)
\ No newline at end of file
+    def function(self, x): self.setFunction(x)
