@@ -1,3 +1,4 @@
+from __future__ import absolute_import
 """
 Tools to perform arithmetic operations on audio signals.
 
@@ -22,8 +23,8 @@ You should have received a copy of the GNU Lesser General Public
 License along with pyo.  If not, see <http://www.gnu.org/licenses/>.
 """
 
-from _core import *
-from _maps import *
+from ._core import *
+from ._maps import *
 
 class Sin(PyoObject):
     """
@@ -35,7 +36,7 @@ class Sin(PyoObject):
 
     :Args:
 
-        input : PyoObject
+        input: PyoObject
             Input signal, angle in radians.
 
     >>> s = Server().boot()
@@ -47,6 +48,7 @@ class Sin(PyoObject):
     """
 
     def __init__(self, input, mul=1, add=0):
+        pyoArgsAssert(self, "oOO", input, mul, add)
         PyoObject.__init__(self, mul, add)
         self._input = input
         self._in_fader = InputFader(input)
@@ -59,12 +61,13 @@ class Sin(PyoObject):
 
         :Args:
 
-            x : PyoObject
+            x: PyoObject
                 New signal to process.
-            fadetime : float, optional
+            fadetime: float, optional
                 Crossfade time between old and new input. Default to 0.05.
 
         """
+        pyoArgsAssert(self, "oN", x, fadetime)
         self._input = x
         self._in_fader.setInput(x, fadetime)
 
@@ -85,7 +88,7 @@ class Cos(PyoObject):
 
     :Args:
 
-        input : PyoObject
+        input: PyoObject
             Input signal, angle in radians.
 
     >>> s = Server().boot()
@@ -96,6 +99,7 @@ class Cos(PyoObject):
 
     """
     def __init__(self, input, mul=1, add=0):
+        pyoArgsAssert(self, "oOO", input, mul, add)
         PyoObject.__init__(self, mul, add)
         self._input = input
         self._in_fader = InputFader(input)
@@ -108,12 +112,13 @@ class Cos(PyoObject):
 
         :Args:
 
-            x : PyoObject
+            x: PyoObject
                 New signal to process.
-            fadetime : float, optional
+            fadetime: float, optional
                 Crossfade time between old and new input. Default to 0.05.
 
         """
+        pyoArgsAssert(self, "oN", x, fadetime)
         self._input = x
         self._in_fader.setInput(x, fadetime)
 
@@ -134,7 +139,7 @@ class Tan(PyoObject):
 
     :Args:
 
-        input : PyoObject
+        input: PyoObject
             Input signal, angle in radians.
 
     >>> s = Server().boot()
@@ -152,6 +157,7 @@ class Tan(PyoObject):
 
     """
     def __init__(self, input, mul=1, add=0):
+        pyoArgsAssert(self, "oOO", input, mul, add)
         PyoObject.__init__(self, mul, add)
         self._input = input
         self._in_fader = InputFader(input)
@@ -164,12 +170,13 @@ class Tan(PyoObject):
 
         :Args:
 
-            x : PyoObject
+            x: PyoObject
                 New signal to process.
-            fadetime : float, optional
+            fadetime: float, optional
                 Crossfade time between old and new input. Default to 0.05.
 
         """
+        pyoArgsAssert(self, "oN", x, fadetime)
         self._input = x
         self._in_fader.setInput(x, fadetime)
 
@@ -190,7 +197,7 @@ class Abs(PyoObject):
 
     :Args:
 
-        input : PyoObject
+        input: PyoObject
             Input signal to process.
 
     >>> s = Server().boot()
@@ -203,6 +210,7 @@ class Abs(PyoObject):
     """
 
     def __init__(self, input, mul=1, add=0):
+        pyoArgsAssert(self, "oOO", input, mul, add)
         PyoObject.__init__(self, mul, add)
         self._input = input
         self._in_fader = InputFader(input)
@@ -215,12 +223,13 @@ class Abs(PyoObject):
 
         :Args:
 
-            x : PyoObject
+            x: PyoObject
                 New signal to process.
-            fadetime : float, optional
+            fadetime: float, optional
                 Crossfade time between old and new input. Default to 0.05.
 
         """
+        pyoArgsAssert(self, "oN", x, fadetime)
         self._input = x
         self._in_fader.setInput(x, fadetime)
 
@@ -241,7 +250,7 @@ class Sqrt(PyoObject):
 
     :Args:
 
-        input : PyoObject
+        input: PyoObject
             Input signal to process.
 
     >>> s = Server().boot()
@@ -259,6 +268,7 @@ class Sqrt(PyoObject):
     """
 
     def __init__(self, input, mul=1, add=0):
+        pyoArgsAssert(self, "oOO", input, mul, add)
         PyoObject.__init__(self, mul, add)
         self._input = input
         self._in_fader = InputFader(input)
@@ -271,12 +281,13 @@ class Sqrt(PyoObject):
 
         :Args:
 
-            x : PyoObject
+            x: PyoObject
                 New signal to process.
-            fadetime : float, optional
+            fadetime: float, optional
                 Crossfade time between old and new input. Default to 0.05.
 
         """
+        pyoArgsAssert(self, "oN", x, fadetime)
         self._input = x
         self._in_fader.setInput(x, fadetime)
 
@@ -298,7 +309,7 @@ class Log(PyoObject):
 
     :Args:
 
-        input : PyoObject
+        input: PyoObject
             Input signal to process.
 
     >>> s = Server().boot()
@@ -311,6 +322,7 @@ class Log(PyoObject):
     """
 
     def __init__(self, input, mul=1, add=0):
+        pyoArgsAssert(self, "oOO", input, mul, add)
         PyoObject.__init__(self, mul, add)
         self._input = input
         self._in_fader = InputFader(input)
@@ -323,12 +335,13 @@ class Log(PyoObject):
 
         :Args:
 
-            x : PyoObject
+            x: PyoObject
                 New signal to process.
-            fadetime : float, optional
+            fadetime: float, optional
                 Crossfade time between old and new input. Default to 0.05.
 
         """
+        pyoArgsAssert(self, "oN", x, fadetime)
         self._input = x
         self._in_fader.setInput(x, fadetime)
 
@@ -350,7 +363,7 @@ class Log2(PyoObject):
 
     :Args:
 
-        input : PyoObject
+        input: PyoObject
             Input signal to process.
 
     >>> s = Server().boot()
@@ -363,6 +376,7 @@ class Log2(PyoObject):
     """
 
     def __init__(self, input, mul=1, add=0):
+        pyoArgsAssert(self, "oOO", input, mul, add)
         PyoObject.__init__(self, mul, add)
         self._input = input
         self._in_fader = InputFader(input)
@@ -375,12 +389,13 @@ class Log2(PyoObject):
 
         :Args:
 
-            x : PyoObject
+            x: PyoObject
                 New signal to process.
-            fadetime : float, optional
+            fadetime: float, optional
                 Crossfade time between old and new input. Default to 0.05.
 
         """
+        pyoArgsAssert(self, "oN", x, fadetime)
         self._input = x
         self._in_fader.setInput(x, fadetime)
 
@@ -402,7 +417,7 @@ class Log10(PyoObject):
 
     :Args:
 
-        input : PyoObject
+        input: PyoObject
             Input signal to process.
 
     >>> s = Server().boot()
@@ -415,6 +430,7 @@ class Log10(PyoObject):
     """
 
     def __init__(self, input, mul=1, add=0):
+        pyoArgsAssert(self, "oOO", input, mul, add)
         PyoObject.__init__(self, mul, add)
         self._input = input
         self._in_fader = InputFader(input)
@@ -427,12 +443,13 @@ class Log10(PyoObject):
 
         :Args:
 
-            x : PyoObject
+            x: PyoObject
                 New signal to process.
-            fadetime : float, optional
+            fadetime: float, optional
                 Crossfade time between old and new input. Default to 0.05.
 
         """
+        pyoArgsAssert(self, "oN", x, fadetime)
         self._input = x
         self._in_fader.setInput(x, fadetime)
 
@@ -455,9 +472,9 @@ class Atan2(PyoObject):
 
     :Args:
 
-        b : float or PyoObject, optional
+        b: float or PyoObject, optional
             Numerator. Defaults to 1.
-        a : float or PyoObject, optional
+        a: float or PyoObject, optional
             Denominator. Defaults to 1.
 
     >>> s = Server().boot()
@@ -470,6 +487,7 @@ class Atan2(PyoObject):
 
     """
     def __init__(self, b=1, a=1, mul=1, add=0):
+        pyoArgsAssert(self, "OOOO", b, a, mul, add)
         PyoObject.__init__(self, mul, add)
         self._b = b
         self._a = a
@@ -482,10 +500,11 @@ class Atan2(PyoObject):
 
         :Args:
 
-            x : float or PyoObject
+            x: float or PyoObject
                 new `b` attribute.
 
         """
+        pyoArgsAssert(self, "O", x)
         self._b = x
         x, lmax = convertArgsToLists(x)
         [obj.setB(wrap(x,i)) for i, obj in enumerate(self._base_objs)]
@@ -496,10 +515,11 @@ class Atan2(PyoObject):
 
         :Args:
 
-            x : float or PyoObject
+            x: float or PyoObject
                 new `a` attribute.
 
         """
+        pyoArgsAssert(self, "O", x)
         self._a = x
         x, lmax = convertArgsToLists(x)
         [obj.setA(wrap(x,i)) for i, obj in enumerate(self._base_objs)]
@@ -529,7 +549,7 @@ class Floor(PyoObject):
 
     :Args:
 
-        input : PyoObject
+        input: PyoObject
             Input signal to process.
 
     >>> s = Server().boot()
@@ -542,6 +562,7 @@ class Floor(PyoObject):
     """
 
     def __init__(self, input, mul=1, add=0):
+        pyoArgsAssert(self, "oOO", input, mul, add)
         PyoObject.__init__(self, mul, add)
         self._input = input
         self._in_fader = InputFader(input)
@@ -554,12 +575,13 @@ class Floor(PyoObject):
 
         :Args:
 
-            x : PyoObject
+            x: PyoObject
                 New signal to process.
-            fadetime : float, optional
+            fadetime: float, optional
                 Crossfade time between old and new input. Default to 0.05.
 
         """
+        pyoArgsAssert(self, "oN", x, fadetime)
         self._input = x
         self._in_fader.setInput(x, fadetime)
 
@@ -581,7 +603,7 @@ class Ceil(PyoObject):
 
     :Args:
 
-        input : PyoObject
+        input: PyoObject
             Input signal to process.
 
     >>> s = Server().boot()
@@ -594,6 +616,7 @@ class Ceil(PyoObject):
     """
 
     def __init__(self, input, mul=1, add=0):
+        pyoArgsAssert(self, "oOO", input, mul, add)
         PyoObject.__init__(self, mul, add)
         self._input = input
         self._in_fader = InputFader(input)
@@ -606,12 +629,13 @@ class Ceil(PyoObject):
 
         :Args:
 
-            x : PyoObject
+            x: PyoObject
                 New signal to process.
-            fadetime : float, optional
+            fadetime: float, optional
                 Crossfade time between old and new input. Default to 0.05.
 
         """
+        pyoArgsAssert(self, "oN", x, fadetime)
         self._input = x
         self._in_fader.setInput(x, fadetime)
 
@@ -633,7 +657,7 @@ class Round(PyoObject):
 
     :Args:
 
-        input : PyoObject
+        input: PyoObject
             Input signal to process.
 
     >>> s = Server().boot()
@@ -646,6 +670,7 @@ class Round(PyoObject):
     """
 
     def __init__(self, input, mul=1, add=0):
+        pyoArgsAssert(self, "oOO", input, mul, add)
         PyoObject.__init__(self, mul, add)
         self._input = input
         self._in_fader = InputFader(input)
@@ -658,12 +683,13 @@ class Round(PyoObject):
 
         :Args:
 
-            x : PyoObject
+            x: PyoObject
                 New signal to process.
-            fadetime : float, optional
+            fadetime: float, optional
                 Crossfade time between old and new input. Default to 0.05.
 
         """
+        pyoArgsAssert(self, "oN", x, fadetime)
         self._input = x
         self._in_fader.setInput(x, fadetime)
 
@@ -684,7 +710,7 @@ class Tanh(PyoObject):
 
     :Args:
 
-        input : PyoObject
+        input: PyoObject
             Input signal, angle in radians.
 
     >>> s = Server().boot()
@@ -696,6 +722,7 @@ class Tanh(PyoObject):
     """
 
     def __init__(self, input, mul=1, add=0):
+        pyoArgsAssert(self, "oOO", input, mul, add)
         PyoObject.__init__(self, mul, add)
         self._input = input
         self._in_fader = InputFader(input)
@@ -708,12 +735,13 @@ class Tanh(PyoObject):
 
         :Args:
 
-            x : PyoObject
+            x: PyoObject
                 New signal to process.
-            fadetime : float, optional
+            fadetime: float, optional
                 Crossfade time between old and new input. Default to 0.05.
 
         """
+        pyoArgsAssert(self, "oN", x, fadetime)
         self._input = x
         self._in_fader.setInput(x, fadetime)
 
@@ -722,4 +750,60 @@ class Tanh(PyoObject):
         """PyoObject. Input signal to process."""
         return self._input
     @input.setter
-    def input(self, x): self.setInput(x)
\ No newline at end of file
+    def input(self, x): self.setInput(x)
+
+class Exp(PyoObject):
+    """
+    Calculates the value of e to the power of x.
+
+    Returns the value of e to the power of x, where e is the base of the
+    natural logarithm, 2.718281828...
+
+    :Parent: :py:class:`PyoObject`
+
+    :Args:
+
+        input: PyoObject
+            Input signal, the exponent.
+
+    >>> s = Server().boot()
+    >>> s.start()
+    >>> a = Sine(freq=200)
+    >>> lf = Sine(freq=.5, mul=5, add=6)
+    >>> # Tanh style distortion
+    >>> t = Exp(2 * a * lf)
+    >>> th = (t - 1) / (t + 1)
+    >>> out = (th * 0.3).out()
+
+    """
+
+    def __init__(self, input, mul=1, add=0):
+        pyoArgsAssert(self, "oOO", input, mul, add)
+        PyoObject.__init__(self, mul, add)
+        self._input = input
+        self._in_fader = InputFader(input)
+        in_fader, mul, add, lmax = convertArgsToLists(self._in_fader, mul, add)
+        self._base_objs = [M_Exp_base(wrap(in_fader,i), wrap(mul,i), wrap(add,i)) for i in range(lmax)]
+
+    def setInput(self, x, fadetime=0.05):
+        """
+        Replace the `input` attribute.
+
+        :Args:
+
+            x: PyoObject
+                New signal to process.
+            fadetime: float, optional
+                Crossfade time between old and new input. Default to 0.05.
+
+        """
+        pyoArgsAssert(self, "oN", x, fadetime)
+        self._input = x
+        self._in_fader.setInput(x, fadetime)
+
+    @property
+    def input(self):
+        """PyoObject. Input signal to process."""
+        return self._input
+    @input.setter
+    def input(self, x): self.setInput(x)
