@@ -9,6 +9,7 @@ soundcard.
 
 """
 
+from __future__ import absolute_import
 """
 Copyright 2009-2015 Olivier Belanger
 
@@ -28,11 +29,11 @@ GNU Lesser General Public License for more details.
 You should have received a copy of the GNU Lesser General Public
 License along with pyo.  If not, see <http://www.gnu.org/licenses/>.
 """
+
 import sys
-from _core import *
-from _maps import *
-from _widgets import createGraphWindow
-from types import ListType, TupleType
+from ._core import *
+from ._maps import *
+from ._widgets import createGraphWindow
 
 ######################################################################
 ### Controls
@@ -51,11 +52,11 @@ class Fader(PyoObject):
 
     :Args:
 
-        fadein : float, optional
+        fadein: float, optional
             Rising time of the envelope in seconds. Defaults to 0.01.
-        fadeout : float, optional
+        fadeout: float, optional
             Falling time of the envelope in seconds. Defaults to 0.1.
-        dur : float, optional
+        dur: float, optional
             Total duration of the envelope. Defaults to 0, which means wait
             for the stop() method to start the fadeout.
 
@@ -65,7 +66,10 @@ class Fader(PyoObject):
 
         The play() method starts the envelope.
 
-        The stop() calls the envelope's release phase if `dur` = 0.
+        The stop() method calls the envelope's release phase if `dur` = 0.
+
+        As of version 0.8.0, exponential or logarithmic envelopes can be created
+        with the exponent factor (see setExp() method).
 
     >>> s = Server().boot()
     >>> s.start()
@@ -82,6 +86,7 @@ class Fader(PyoObject):
         self._fadein = fadein
         self._fadeout = fadeout
         self._dur = dur
+        self._exp = 1.0
         fadein, fadeout, dur, mul, add, lmax = convertArgsToLists(fadein, fadeout, dur, mul, add)
         self._base_objs = [Fader_base(wrap(fadein,i), wrap(fadeout,i), wrap(dur,i), wrap(mul,i), wrap(add,i)) for i in range(lmax)]
 
@@ -94,7 +99,7 @@ class Fader(PyoObject):
 
         :Args:
 
-            x : float
+            x: float
                 new `fadein` attribute.
 
         """
@@ -109,7 +114,7 @@ class Fader(PyoObject):
 
         :Args:
 
-            x : float
+            x: float
                 new `fadeout` attribute.
 
         """
@@ -124,7 +129,7 @@ class Fader(PyoObject):
 
         :Args:
 
-            x : float
+            x: float
                 new `dur` attribute.
 
         """
@@ -133,6 +138,25 @@ class Fader(PyoObject):
         x, lmax = convertArgsToLists(x)
         [obj.setDur(wrap(x,i)) for i, obj in enumerate(self._base_objs)]
 
+    def setExp(self, x):
+        """
+        Sets an exponent factor to create exponential or logarithmic envelope.
+
+        The default value is 1.0, which means linear segments. A value
+        higher than 1.0 will produce exponential segments while a value
+        between 0 and 1 will produce logarithmic segments. Must be > 0.0.
+
+        :Args:
+
+            x: float
+                new `exp` attribute.
+
+        """
+        pyoArgsAssert(self, "n", x)
+        self._exp = x
+        x, lmax = convertArgsToLists(x)
+        [obj.setExp(wrap(x,i)) for i, obj in enumerate(self._base_objs)]
+
     def ctrl(self, map_list=None, title=None, wxnoserver=False):
         self._map_list = [SLMap(0, 10., 'lin', 'fadein', self._fadein, dataOnly=True),
                           SLMap(0, 10., 'lin', 'fadeout', self._fadeout, dataOnly=True),
@@ -160,6 +184,13 @@ class Fader(PyoObject):
     @dur.setter
     def dur(self, x): self.setDur(x)
 
+    @property
+    def exp(self):
+        """float. Exponent factor of the envelope."""
+        return self._exp
+    @exp.setter
+    def exp(self, x): self.setExp(x)
+
 
 class Adsr(PyoObject):
     """
@@ -176,15 +207,15 @@ class Adsr(PyoObject):
 
     :Args:
 
-        attack : float, optional
+        attack: float, optional
             Duration of the attack phase in seconds. Defaults to 0.01.
-        decay : float, optional
+        decay: float, optional
             Duration of the decay in seconds. Defaults to 0.05.
-        sustain : float, optional
+        sustain: float, optional
             Amplitude of the sustain phase. Defaults to 0.707.
-        release : float, optional
+        release: float, optional
             Duration of the release in seconds. Defaults to 0.1.
-        dur : float, optional
+        dur: float, optional
             Total duration of the envelope. Defaults to 0, which means wait
             for the stop() method to start the release phase.
 
@@ -195,7 +226,10 @@ class Adsr(PyoObject):
 
         The play() method starts the envelope.
 
-        The stop() calls the envelope's release phase if `dur` = 0.
+        The stop() method calls the envelope's release phase if `dur` = 0.
+
+        As of version 0.8.0, exponential or logarithmic envelopes can be created
+        with the exponent factor (see setExp() method).
 
     >>> s = Server().boot()
     >>> s.start()
@@ -214,6 +248,7 @@ class Adsr(PyoObject):
         self._sustain = sustain
         self._release = release
         self._dur = dur
+        self._exp = 1.0
         attack, decay, sustain, release, dur, mul, add, lmax = convertArgsToLists(attack, decay, sustain, release, dur, mul, add)
         self._base_objs = [Adsr_base(wrap(attack,i), wrap(decay,i), wrap(sustain,i), wrap(release,i), wrap(dur,i), wrap(mul,i), wrap(add,i)) for i in range(lmax)]
 
@@ -226,7 +261,7 @@ class Adsr(PyoObject):
 
         :Args:
 
-            x : float
+            x: float
                 new `attack` attribute.
 
         """
@@ -241,7 +276,7 @@ class Adsr(PyoObject):
 
         :Args:
 
-            x : float
+            x: float
                 new `decay` attribute.
 
         """
@@ -256,7 +291,7 @@ class Adsr(PyoObject):
 
         :Args:
 
-            x : float
+            x: float
                 new `sustain` attribute.
 
         """
@@ -271,7 +306,7 @@ class Adsr(PyoObject):
 
         :Args:
 
-            x : float
+            x: float
                 new `sustain` attribute.
 
         """
@@ -286,7 +321,7 @@ class Adsr(PyoObject):
 
         :Args:
 
-            x : float
+            x: float
                 new `dur` attribute.
 
         """
@@ -295,6 +330,25 @@ class Adsr(PyoObject):
         x, lmax = convertArgsToLists(x)
         [obj.setDur(wrap(x,i)) for i, obj in enumerate(self._base_objs)]
 
+    def setExp(self, x):
+        """
+        Sets an exponent factor to create exponential or logarithmic envelope.
+
+        The default value is 1.0, which means linear segments. A value
+        higher than 1.0 will produce exponential segments while a value
+        between 0 and 1 will produce logarithmic segments. Must be > 0.0.
+
+        :Args:
+
+            x: float
+                new `exp` attribute.
+
+        """
+        pyoArgsAssert(self, "n", x)
+        self._exp = x
+        x, lmax = convertArgsToLists(x)
+        [obj.setExp(wrap(x,i)) for i, obj in enumerate(self._base_objs)]
+
     def ctrl(self, map_list=None, title=None, wxnoserver=False):
         self._map_list = [SLMap(0, 5, 'lin', 'attack', self._attack, dataOnly=True),
                           SLMap(0, 5, 'lin', 'decay', self._decay, dataOnly=True),
@@ -338,9 +392,16 @@ class Adsr(PyoObject):
     @dur.setter
     def dur(self, x): self.setDur(x)
 
+    @property
+    def exp(self):
+        """float. Exponent factor of the envelope."""
+        return self._exp
+    @exp.setter
+    def exp(self, x): self.setExp(x)
+
 class Linseg(PyoObject):
     """
-    Trace a series of line segments between specified break-points.
+    Draw a series of line segments between specified break-points.
 
     The play() method starts the envelope and is not called at the
     object creation time.
@@ -349,14 +410,14 @@ class Linseg(PyoObject):
 
     :Args:
 
-        list : list of tuples
+        list: list of tuples
             Points used to construct the line segments. Each tuple is a
             new point in the form (time, value).
 
             Times are given in seconds and must be in increasing order.
-        loop : boolean, optional
+        loop: boolean, optional
             Looping mode. Defaults to False.
-        initToFirstVal : boolean, optional
+        initToFirstVal: boolean, optional
             If True, audio buffer will be filled at initialization with the
             first value of the line. Defaults to False.
 
@@ -378,7 +439,7 @@ class Linseg(PyoObject):
         self._list = list
         self._loop = loop
         initToFirstVal, loop, mul, add, lmax = convertArgsToLists(initToFirstVal, loop, mul, add)
-        if type(list[0]) != ListType:
+        if type(list[0]) != list:
             self._base_objs = [Linseg_base(list, wrap(loop,i), wrap(initToFirstVal,i), wrap(mul,i), wrap(add,i)) for i in range(lmax)]
         else:
             listlen = len(list)
@@ -394,13 +455,13 @@ class Linseg(PyoObject):
 
         :Args:
 
-            x : list of tuples
+            x: list of tuples
                 new `list` attribute.
 
         """
         pyoArgsAssert(self, "l", x)
         self._list = x
-        if type(x[0]) != ListType:
+        if type(x[0]) != list:
             [obj.setList(x) for i, obj in enumerate(self._base_objs)]
         else:
             [obj.setList(wrap(x,i)) for i, obj in enumerate(self._base_objs)]
@@ -411,7 +472,7 @@ class Linseg(PyoObject):
 
         :Args:
 
-            x : list of tuples
+            x: list of tuples
                 new `list` attribute.
 
         """
@@ -426,7 +487,7 @@ class Linseg(PyoObject):
 
         :Args:
 
-            x : boolean
+            x: boolean
                 new `loop` attribute.
 
         """
@@ -435,6 +496,13 @@ class Linseg(PyoObject):
         x, lmax = convertArgsToLists(x)
         [obj.setLoop(wrap(x,i)) for i, obj in enumerate(self._base_objs)]
 
+    def pause(self):
+        """
+        Toggles between play and stop mode without reset.
+
+        """
+        [obj.pause() for obj in self._base_objs]
+
     def graph(self, xlen=None, yrange=None, title=None, wxnoserver=False):
         """
         Opens a grapher window to control the shape of the envelope.
@@ -447,16 +515,16 @@ class Linseg(PyoObject):
 
         :Args:
 
-            xlen : float, optional
+            xlen: float, optional
                 Set the maximum value of the X axis of the graph. If None, the
                 maximum value is retrieve from the current list of points.
-            yrange : tuple, optional
+            yrange: tuple, optional
                 Set the min and max values of the Y axis of the graph. If
                 None, min and max are retrieve from the current list of points.
-            title : string, optional
+            title: string, optional
                 Title of the window. If none is provided, the name of the
                 class is used.
-            wxnoserver : boolean, optional
+            wxnoserver: boolean, optional
                 With wxPython graphical toolkit, if True, tells the
                 interpreter that there will be no server window.
 
@@ -464,11 +532,11 @@ class Linseg(PyoObject):
         server GUI before showing the controller window.
 
         """
-        if xlen == None:
+        if xlen is None:
             xlen = float(self._list[-1][0])
         else:
             xlen = float(xlen)
-        if yrange == None:
+        if yrange is None:
             ymin = float(min([x[1] for x in self._list]))
             ymax = float(max([x[1] for x in self._list]))
             if ymin == ymax:
@@ -493,7 +561,7 @@ class Linseg(PyoObject):
 
 class Expseg(PyoObject):
     """
-    Trace a series of exponential segments between specified break-points.
+    Draw a series of exponential segments between specified break-points.
 
     The play() method starts the envelope and is not called at the
     object creation time.
@@ -502,20 +570,20 @@ class Expseg(PyoObject):
 
     :Args:
 
-        list : list of tuples
+        list: list of tuples
             Points used to construct the line segments. Each tuple is a
             new point in the form (time, value).
 
             Times are given in seconds and must be in increasing order.
-        loop : boolean, optional
+        loop: boolean, optional
             Looping mode. Defaults to False.
-        exp : float, optional
+        exp: float, optional
             Exponent factor. Used to control the slope of the curves.
             Defaults to 10.
-        inverse : boolean, optional
+        inverse: boolean, optional
             If True, downward slope will be inversed. Useful to create
             biexponential curves. Defaults to True.
-        initToFirstVal : boolean, optional
+        initToFirstVal: boolean, optional
             If True, audio buffer will be filled at initialization with the
             first value of the line. Defaults to False.
 
@@ -539,7 +607,7 @@ class Expseg(PyoObject):
         self._exp = exp
         self._inverse = inverse
         loop, exp, inverse, initToFirstVal, mul, add, lmax = convertArgsToLists(loop, exp, inverse, initToFirstVal, mul, add)
-        if type(list[0]) != ListType:
+        if type(list[0]) != list:
             self._base_objs = [Expseg_base(list, wrap(loop,i), wrap(exp,i), wrap(inverse,i), wrap(initToFirstVal,i), wrap(mul,i), wrap(add,i)) for i in range(lmax)]
         else:
             listlen = len(list)
@@ -555,13 +623,13 @@ class Expseg(PyoObject):
 
         :Args:
 
-            x : list of tuples
+            x: list of tuples
                 new `list` attribute.
 
         """
         pyoArgsAssert(self, "l", x)
         self._list = x
-        if type(x[0]) != ListType:
+        if type(x[0]) != list:
             [obj.setList(x) for i, obj in enumerate(self._base_objs)]
         else:
             [obj.setList(wrap(x,i)) for i, obj in enumerate(self._base_objs)]
@@ -572,7 +640,7 @@ class Expseg(PyoObject):
 
         :Args:
 
-            x : boolean
+            x: boolean
                 new `loop` attribute.
 
         """
@@ -587,7 +655,7 @@ class Expseg(PyoObject):
 
         :Args:
 
-            x : float
+            x: float
                 new `exp` attribute.
 
         """
@@ -602,7 +670,7 @@ class Expseg(PyoObject):
 
         :Args:
 
-            x : boolean
+            x: boolean
                 new `inverse` attribute.
 
         """
@@ -617,12 +685,19 @@ class Expseg(PyoObject):
 
         :Args:
 
-            x : list of tuples
+            x: list of tuples
                 new `list` attribute.
 
         """
         self.setList(x)
 
+    def pause(self):
+        """
+        Toggles between play and stop mode without reset.
+
+        """
+        [obj.pause() for obj in self._base_objs]
+
     def getPoints(self):
         return self._list
 
@@ -638,18 +713,18 @@ class Expseg(PyoObject):
 
         :Args:
 
-            xlen : float, optional
+            xlen: float, optional
                 Set the maximum value of the X axis of the graph. If None, the
                 maximum value is retrieve from the current list of points.
                 Defaults to None.
-            yrange : tuple, optional
+            yrange: tuple, optional
                 Set the min and max values of the Y axis of the graph. If
                 None, min and max are retrieve from the current list of points.
                 Defaults to None.
-            title : string, optional
+            title: string, optional
                 Title of the window. If none is provided, the name of the
                 class is used.
-            wxnoserver : boolean, optional
+            wxnoserver: boolean, optional
                 With wxPython graphical toolkit, if True, tells the
                 interpreter that there will be no server window.
 
@@ -657,11 +732,11 @@ class Expseg(PyoObject):
         server GUI before showing the controller window.
 
         """
-        if xlen == None:
+        if xlen is None:
             xlen = float(self._list[-1][0])
         else:
             xlen = float(xlen)
-        if yrange == None:
+        if yrange is None:
             ymin = float(min([x[1] for x in self._list]))
             ymax = float(max([x[1] for x in self._list]))
             if ymin == ymax:
@@ -710,11 +785,11 @@ class SigTo(PyoObject):
 
     :Args:
 
-        value : float or PyoObject
+        value: float or PyoObject
             Numerical value to convert.
-        time : float, optional
+        time: float or PyoObject, optional
             Ramp time, in seconds, to reach the new value. Defaults to 0.025.
-        init : float, optional
+        init: float, optional
             Initial value of the internal memory. Defaults to 0.
 
     .. note::
@@ -733,7 +808,7 @@ class SigTo(PyoObject):
 
     """
     def __init__(self, value, time=0.025, init=0.0, mul=1, add=0):
-        pyoArgsAssert(self, "OnnOO", value, time, init, mul, add)
+        pyoArgsAssert(self, "OOnOO", value, time, init, mul, add)
         PyoObject.__init__(self, mul, add)
         self._value = value
         self._time = time
@@ -746,7 +821,7 @@ class SigTo(PyoObject):
 
         :Args:
 
-            x : float or PyoObject
+            x: float or PyoObject
                 Numerical value to convert.
 
         """
@@ -761,17 +836,17 @@ class SigTo(PyoObject):
 
         :Args:
 
-            x : float
-                New ramp time.
+            x: float or PyoObject
+                New ramp time in seconds.
 
         """
-        pyoArgsAssert(self, "n", x)
+        pyoArgsAssert(self, "O", x)
         self._time = x
         x, lmax = convertArgsToLists(x)
         [obj.setTime(wrap(x,i)) for i, obj in enumerate(self._base_objs)]
 
     def ctrl(self, map_list=None, title=None, wxnoserver=False):
-        self._map_list = [SLMap(0, 10, 'lin', 'time', self._time, dataOnly=True)]
+        self._map_list = [SLMap(0, 10, 'lin', 'time', self._time)]
         PyoObject.ctrl(self, map_list, title, wxnoserver)
 
     @property
@@ -783,7 +858,7 @@ class SigTo(PyoObject):
 
     @property
     def time(self):
-        """float. Ramp time."""
+        """floator PyoObject. Ramp time in seconds."""
         return self._time
     @time.setter
-    def time(self, x): self.setTime(x)
\ No newline at end of file
+    def time(self, x): self.setTime(x)
