@@ -3,6 +3,7 @@ Objects to modify the dynamic range and sample quality of audio signals.
 
 """
 
+from __future__ import absolute_import
 """
 Copyright 2009-2015 Olivier Belanger
 
@@ -22,8 +23,9 @@ GNU Lesser General Public License for more details.
 You should have received a copy of the GNU Lesser General Public
 License along with pyo.  If not, see <http://www.gnu.org/licenses/>.
 """
-from _core import *
-from _maps import *
+
+from ._core import *
+from ._maps import *
 
 class Clip(PyoObject):
     """
@@ -33,11 +35,11 @@ class Clip(PyoObject):
 
     :Args:
 
-        input : PyoObject
+        input: PyoObject
             Input signal to process.
-        min : float or PyoObject, optional
+        min: float or PyoObject, optional
             Minimum possible value. Defaults to -1.
-        max : float or PyoObject, optional
+        max: float or PyoObject, optional
             Maximum possible value. Defaults to 1.
 
     >>> s = Server().boot()
@@ -49,6 +51,7 @@ class Clip(PyoObject):
 
     """
     def __init__(self, input, min=-1.0, max=1.0, mul=1, add=0):
+        pyoArgsAssert(self, "oOOOO", input, min, max, mul, add)
         PyoObject.__init__(self, mul, add)
         self._input = input
         self._min = min
@@ -63,12 +66,13 @@ class Clip(PyoObject):
 
         :Args:
 
-            x : PyoObject
+            x: PyoObject
                 New signal to process.
-            fadetime : float, optional
+            fadetime: float, optional
                 Crossfade time between old and new input. Defaults to 0.05.
 
         """
+        pyoArgsAssert(self, "oN", x, fadetime)
         self._input = x
         self._in_fader.setInput(x, fadetime)
 
@@ -78,10 +82,11 @@ class Clip(PyoObject):
 
         :Args:
 
-            x : float or PyoObject
+            x: float or PyoObject
                 New `min` attribute.
 
         """
+        pyoArgsAssert(self, "O", x)
         self._min = x
         x, lmax = convertArgsToLists(x)
         [obj.setMin(wrap(x,i)) for i, obj in enumerate(self._base_objs)]
@@ -92,10 +97,11 @@ class Clip(PyoObject):
 
         :Args:
 
-            x : float or PyoObject
+            x: float or PyoObject
                 New `max` attribute.
 
         """
+        pyoArgsAssert(self, "O", x)
         self._max = x
         x, lmax = convertArgsToLists(x)
         [obj.setMax(wrap(x,i)) for i, obj in enumerate(self._base_objs)]
@@ -138,11 +144,11 @@ class Mirror(PyoObject):
 
     :Args:
 
-        input : PyoObject
+        input: PyoObject
             Input signal to process.
-        min : float or PyoObject, optional
+        min: float or PyoObject, optional
             Minimum possible value. Defaults to 0.
-        max : float or PyoObject, optional
+        max: float or PyoObject, optional
             Maximum possible value. Defaults to 1.
 
     .. note::
@@ -159,6 +165,7 @@ class Mirror(PyoObject):
 
     """
     def __init__(self, input, min=0.0, max=1.0, mul=1, add=0):
+        pyoArgsAssert(self, "oOOOO", input, min, max, mul, add)
         PyoObject.__init__(self, mul, add)
         self._input = input
         self._min = min
@@ -173,12 +180,13 @@ class Mirror(PyoObject):
 
         :Args:
 
-            x : PyoObject
+            x: PyoObject
                 New signal to process.
-            fadetime : float, optional
+            fadetime: float, optional
                 Crossfade time between old and new input. Defaults to 0.05.
 
         """
+        pyoArgsAssert(self, "oN", x, fadetime)
         self._input = x
         self._in_fader.setInput(x, fadetime)
 
@@ -188,10 +196,11 @@ class Mirror(PyoObject):
 
         :Args:
 
-            x : float or PyoObject
+            x: float or PyoObject
                 New `min` attribute.
 
         """
+        pyoArgsAssert(self, "O", x)
         self._min = x
         x, lmax = convertArgsToLists(x)
         [obj.setMin(wrap(x,i)) for i, obj in enumerate(self._base_objs)]
@@ -202,10 +211,11 @@ class Mirror(PyoObject):
 
         :Args:
 
-            x : float or PyoObject
+            x: float or PyoObject
                 New `max` attribute.
 
         """
+        pyoArgsAssert(self, "O", x)
         self._max = x
         x, lmax = convertArgsToLists(x)
         [obj.setMax(wrap(x,i)) for i, obj in enumerate(self._base_objs)]
@@ -248,12 +258,12 @@ class Degrade(PyoObject):
 
     :Args:
 
-        input : PyoObject
+        input: PyoObject
             Input signal to process.
-        bitdepth : float or PyoObject, optional
+        bitdepth: float or PyoObject, optional
             Signal quantization in bits. Must be in range 1 -> 32.
             Defaults to 16.
-        srscale : float or PyoObject, optional
+        srscale: float or PyoObject, optional
             Sampling rate multiplier. Must be in range 0.0009765625 -> 1.
             Defaults to 1.
 
@@ -267,6 +277,7 @@ class Degrade(PyoObject):
 
     """
     def __init__(self, input, bitdepth=16, srscale=1.0, mul=1, add=0):
+        pyoArgsAssert(self, "oOOOO", input, bitdepth, srscale, mul, add)
         PyoObject.__init__(self, mul, add)
         self._input = input
         self._bitdepth = bitdepth
@@ -281,12 +292,13 @@ class Degrade(PyoObject):
 
         :Args:
 
-            x : PyoObject
+            x: PyoObject
                 New signal to process.
-            fadetime : float, optional
+            fadetime: float, optional
                 Crossfade time between old and new input. Defaults to 0.05.
 
         """
+        pyoArgsAssert(self, "oN", x, fadetime)
         self._input = x
         self._in_fader.setInput(x, fadetime)
 
@@ -296,10 +308,11 @@ class Degrade(PyoObject):
 
         :Args:
 
-            x : float or PyoObject
+            x: float or PyoObject
                 New `bitdepth` attribute.
 
         """
+        pyoArgsAssert(self, "O", x)
         self._bitdepth = x
         x, lmax = convertArgsToLists(x)
         [obj.setBitdepth(wrap(x,i)) for i, obj in enumerate(self._base_objs)]
@@ -310,10 +323,11 @@ class Degrade(PyoObject):
 
         :Args:
 
-            x : float or PyoObject
+            x: float or PyoObject
                 New `srscale` attribute.
 
         """
+        pyoArgsAssert(self, "O", x)
         self._srscale = x
         x, lmax = convertArgsToLists(x)
         [obj.setSrscale(wrap(x,i)) for i, obj in enumerate(self._base_objs)]
@@ -356,30 +370,30 @@ class Compress(PyoObject):
 
     :Args:
 
-        input : PyoObject
+        input: PyoObject
             Input signal to process.
-        thresh : float or PyoObject, optional
+        thresh: float or PyoObject, optional
             Level, expressed in dB, above which the signal is reduced.
             Reference level is 0dB. Defaults to -20.
-        ratio : float or PyoObject, optional
+        ratio: float or PyoObject, optional
             Determines the input/output ratio for signals above the
             threshold. Defaults to 2.
-        risetime : float or PyoObject, optional
+        risetime: float or PyoObject, optional
             Used in amplitude follower, time to reach upward value in
             seconds. Defaults to 0.01.
-        falltime : float or PyoObject, optional
+        falltime: float or PyoObject, optional
             Used in amplitude follower, time to reach downward value in
             seconds. Defaults to 0.1.
-        lookahead : float, optional
+        lookahead: float, optional
             Delay length, in ms, for the "look-ahead" buffer. Range is
             0 -> 25 ms. Defaults to 5.0.
-        knee : float optional
+        knee: float optional
             Shape of the transfert function around the threshold, specified
             in the range 0 -> 1.
 
             A value of 0 means a hard knee and a value of 1.0 means a softer
             knee. Defaults to 0.
-        outputAmp : boolean, optional
+        outputAmp: boolean, optional
             If True, the object's output signal will be the compression level
             alone, not the compressed signal.
 
@@ -395,6 +409,7 @@ class Compress(PyoObject):
 
     """
     def __init__(self, input, thresh=-20, ratio=2, risetime=0.01, falltime=0.1, lookahead=5.0, knee=0, outputAmp=False, mul=1, add=0):
+        pyoArgsAssert(self, "oOOOOnnbOO", input, thresh, ratio, risetime, falltime, lookahead, knee, outputAmp, mul, add)
         PyoObject.__init__(self, mul, add)
         self._input = input
         self._thresh = thresh
@@ -413,12 +428,13 @@ class Compress(PyoObject):
 
         :Args:
 
-            x : PyoObject
+            x: PyoObject
                 New signal to process.
-            fadetime : float, optional
+            fadetime: float, optional
                 Crossfade time between old and new input. Defaults to 0.05.
 
         """
+        pyoArgsAssert(self, "oN", x, fadetime)
         self._input = x
         self._in_fader.setInput(x, fadetime)
 
@@ -428,10 +444,11 @@ class Compress(PyoObject):
 
         :Args:
 
-            x : float or PyoObject
+            x: float or PyoObject
                 New `thresh` attribute.
 
         """
+        pyoArgsAssert(self, "O", x)
         self._thresh = x
         x, lmax = convertArgsToLists(x)
         [obj.setThresh(wrap(x,i)) for i, obj in enumerate(self._base_objs)]
@@ -442,10 +459,11 @@ class Compress(PyoObject):
 
         :Args:
 
-            x : float or PyoObject
+            x: float or PyoObject
                 New `ratio` attribute.
 
         """
+        pyoArgsAssert(self, "O", x)
         self._ratio = x
         x, lmax = convertArgsToLists(x)
         [obj.setRatio(wrap(x,i)) for i, obj in enumerate(self._base_objs)]
@@ -456,10 +474,11 @@ class Compress(PyoObject):
 
         :Args:
 
-            x : float or PyoObject
+            x: float or PyoObject
                 New `risetime` attribute.
 
         """
+        pyoArgsAssert(self, "O", x)
         self._risetime = x
         x, lmax = convertArgsToLists(x)
         [obj.setRiseTime(wrap(x,i)) for i, obj in enumerate(self._base_objs)]
@@ -470,10 +489,11 @@ class Compress(PyoObject):
 
         :Args:
 
-            x : float or PyoObject
+            x: float or PyoObject
                 New `falltime` attribute.
 
         """
+        pyoArgsAssert(self, "O", x)
         self._falltime = x
         x, lmax = convertArgsToLists(x)
         [obj.setFallTime(wrap(x,i)) for i, obj in enumerate(self._base_objs)]
@@ -484,10 +504,11 @@ class Compress(PyoObject):
 
         :Args:
 
-            x : float
+            x: float
                 New `lookahead` attribute.
 
         """
+        pyoArgsAssert(self, "n", x)
         self._lookahead = x
         x, lmax = convertArgsToLists(x)
         [obj.setLookAhead(wrap(x,i)) for i, obj in enumerate(self._base_objs)]
@@ -498,21 +519,22 @@ class Compress(PyoObject):
 
         :Args:
 
-            x : float
+            x: float
                 New `knee` attribute.
 
         """
+        pyoArgsAssert(self, "n", x)
         self._knee = x
         x, lmax = convertArgsToLists(x)
         [obj.setKnee(wrap(x,i)) for i, obj in enumerate(self._base_objs)]
 
     def ctrl(self, map_list=None, title=None, wxnoserver=False):
-        self._map_list = [SLMap(-60., 0., 'lin', 'thresh',  self._thresh),
-                          SLMap(1., 10., 'lin', 'ratio',  self._ratio),
-                          SLMap(0.001, .3, 'lin', 'risetime',  self._risetime),
-                          SLMap(0.001, .3, 'lin', 'falltime',  self._falltime),
-                          SLMap(0, 25, 'lin', 'lookahead',  self._lookahead, dataOnly=True),
-                          SLMap(0, 1, 'lin', 'knee',  self._knee, dataOnly=True),
+        self._map_list = [SLMap(-60., 0., 'lin', 'thresh', self._thresh),
+                          SLMap(1., 10., 'lin', 'ratio', self._ratio),
+                          SLMap(0.001, .3, 'lin', 'risetime', self._risetime),
+                          SLMap(0.001, .3, 'lin', 'falltime', self._falltime),
+                          SLMap(0, 25, 'lin', 'lookahead', self._lookahead, dataOnly=True),
+                          SLMap(0, 1, 'lin', 'knee', self._knee, dataOnly=True),
                           SLMapMul(self._mul)]
         PyoObject.ctrl(self, map_list, title, wxnoserver)
 
@@ -579,19 +601,19 @@ class Gate(PyoObject):
 
     :Args:
 
-        input : PyoObject
+        input: PyoObject
             Input signal to process.
-        thresh : float or PyoObject, optional
+        thresh: float or PyoObject, optional
             Level, expressed in dB, below which the gate is closed.
             Reference level is 0dB. Defaults to -70.
-        risetime : float or PyoObject, optional
+        risetime: float or PyoObject, optional
             Time to open the gate in seconds. Defaults to 0.01.
-        falltime : float or PyoObject, optional
+        falltime: float or PyoObject, optional
             Time to close the gate in seconds. Defaults to 0.05.
-        lookahead : float, optional
+        lookahead: float, optional
             Delay length, in ms, for the "look-ahead" buffer. Range is
             0 -> 25 ms. Defaults to 5.0.
-        outputAmp : boolean, optional
+        outputAmp: boolean, optional
             If True, the object's output signal will be the gating level
             alone, not the gated signal.
 
@@ -607,6 +629,7 @@ class Gate(PyoObject):
 
     """
     def __init__(self, input, thresh=-70, risetime=0.01, falltime=0.05, lookahead=5.0, outputAmp=False, mul=1, add=0):
+        pyoArgsAssert(self, "oOOOnbOO", input, thresh, risetime, falltime, lookahead, outputAmp, mul, add)
         PyoObject.__init__(self, mul, add)
         self._input = input
         self._thresh = thresh
@@ -623,12 +646,13 @@ class Gate(PyoObject):
 
         :Args:
 
-            x : PyoObject
+            x: PyoObject
                 New signal to process.
-            fadetime : float, optional
+            fadetime: float, optional
                 Crossfade time between old and new input. Defaults to 0.05.
 
         """
+        pyoArgsAssert(self, "oN", x, fadetime)
         self._input = x
         self._in_fader.setInput(x, fadetime)
 
@@ -638,10 +662,11 @@ class Gate(PyoObject):
 
         :Args:
 
-            x : float or PyoObject
+            x: float or PyoObject
                 New `thresh` attribute.
 
         """
+        pyoArgsAssert(self, "O", x)
         self._thresh = x
         x, lmax = convertArgsToLists(x)
         [obj.setThresh(wrap(x,i)) for i, obj in enumerate(self._base_objs)]
@@ -652,10 +677,11 @@ class Gate(PyoObject):
 
         :Args:
 
-            x : float or PyoObject
+            x: float or PyoObject
                 New `risetime` attribute.
 
         """
+        pyoArgsAssert(self, "O", x)
         self._risetime = x
         x, lmax = convertArgsToLists(x)
         [obj.setRiseTime(wrap(x,i)) for i, obj in enumerate(self._base_objs)]
@@ -666,10 +692,11 @@ class Gate(PyoObject):
 
         :Args:
 
-            x : float or PyoObject
+            x: float or PyoObject
                 New `falltime` attribute.
 
         """
+        pyoArgsAssert(self, "O", x)
         self._falltime = x
         x, lmax = convertArgsToLists(x)
         [obj.setFallTime(wrap(x,i)) for i, obj in enumerate(self._base_objs)]
@@ -680,19 +707,20 @@ class Gate(PyoObject):
 
         :Args:
 
-            x : float
+            x: float
                 New `lookahead` attribute.
 
         """
+        pyoArgsAssert(self, "n", x)
         self._lookahead = x
         x, lmax = convertArgsToLists(x)
         [obj.setLookAhead(wrap(x,i)) for i, obj in enumerate(self._base_objs)]
 
     def ctrl(self, map_list=None, title=None, wxnoserver=False):
-        self._map_list = [SLMap(-100., 0., 'lin', 'thresh',  self._thresh),
-                          SLMap(0.0001, .3, 'lin', 'risetime',  self._risetime),
-                          SLMap(0.0001, .3, 'lin', 'falltime',  self._falltime),
-                          SLMap(0, 25, 'lin', 'lookahead',  self._lookahead, dataOnly=True),
+        self._map_list = [SLMap(-100., 0., 'lin', 'thresh', self._thresh),
+                          SLMap(0.0001, .3, 'lin', 'risetime', self._risetime),
+                          SLMap(0.0001, .3, 'lin', 'falltime', self._falltime),
+                          SLMap(0, 25, 'lin', 'lookahead', self._lookahead, dataOnly=True),
                           SLMapMul(self._mul)]
         PyoObject.ctrl(self, map_list, title, wxnoserver)
 
@@ -741,11 +769,11 @@ class Balance(PyoObject):
 
     :Args:
 
-        input : PyoObject
+        input: PyoObject
             Input signal to process.
-        input2 : PyoObject
+        input2: PyoObject
             Comparator signal.
-        freq : float or PyoObject, optional
+        freq: float or PyoObject, optional
             Cutoff frequency of the lowpass filter in hertz. Default to 10.
 
     >>> s = Server().boot()
@@ -756,6 +784,7 @@ class Balance(PyoObject):
 
     """
     def __init__(self, input, input2, freq=10, mul=1, add=0):
+        pyoArgsAssert(self, "ooOOO", input, input2, freq, mul, add)
         PyoObject.__init__(self, mul, add)
         self._input = input
         self._input2 = input2
@@ -773,12 +802,13 @@ class Balance(PyoObject):
 
         :Args:
 
-            x : PyoObject
+            x: PyoObject
                 New signal to process.
-            fadetime : float, optional
+            fadetime: float, optional
                 Crossfade time between old and new input. Default to 0.05.
 
         """
+        pyoArgsAssert(self, "oN", x, fadetime)
         self._input = x
         self._in_fader.setInput(x, fadetime)
 
@@ -790,12 +820,13 @@ class Balance(PyoObject):
 
         :Args:
 
-            x : PyoObject
+            x: PyoObject
                 New signal to process.
-            fadetime : float, optional
+            fadetime: float, optional
                 Crossfade time between old and new input. Default to 0.05.
 
         """
+        pyoArgsAssert(self, "oN", x, fadetime)
         self._input2 = x
         self._in_fader2.setInput(x, fadetime)
 
@@ -807,10 +838,11 @@ class Balance(PyoObject):
 
         :Args:
 
-            x : float or PyoObject
+            x: float or PyoObject
                 New `freq` attribute.
 
         """
+        pyoArgsAssert(self, "O", x)
         self._freq = x
         x, lmax = convertArgsToLists(x)
         [obj.setFreq(wrap(x,i)) for i, obj in enumerate(self._base_objs)]
@@ -848,9 +880,9 @@ class Min(PyoObject):
 
     :Args:
 
-        input : PyoObject
+        input: PyoObject
             Input signal to process.
-        comp : float or PyoObject, optional
+        comp: float or PyoObject, optional
             Comparison value. If `input` is lower than this value,
             `input` is send to the output, otherwise, `comp` is outputted.
 
@@ -863,6 +895,7 @@ class Min(PyoObject):
 
     """
     def __init__(self, input, comp=0.5, mul=1, add=0):
+        pyoArgsAssert(self, "oOOO", input, comp, mul, add)
         PyoObject.__init__(self, mul, add)
         self._input = input
         self._comp = comp
@@ -876,12 +909,13 @@ class Min(PyoObject):
 
         :Args:
 
-            x : PyoObject
+            x: PyoObject
                 New signal to process.
-            fadetime : float, optional
+            fadetime: float, optional
                 Crossfade time between old and new input. Default to 0.05.
 
         """
+        pyoArgsAssert(self, "oN", x, fadetime)
         self._input = x
         self._in_fader.setInput(x, fadetime)
 
@@ -891,10 +925,11 @@ class Min(PyoObject):
 
         :Args:
 
-            x : float or PyoObject
+            x: float or PyoObject
                 New `comp` attribute.
 
         """
+        pyoArgsAssert(self, "O", x)
         self._comp = x
         x, lmax = convertArgsToLists(x)
         [obj.setComp(wrap(x,i)) for i, obj in enumerate(self._base_objs)]
@@ -925,9 +960,9 @@ class Max(PyoObject):
 
     :Args:
 
-        input : PyoObject
+        input: PyoObject
             Input signal to process.
-        comp : float or PyoObject, optional
+        comp: float or PyoObject, optional
             Comparison value. If `input` is higher than this value,
             `input` is send to the output, otherwise, `comp` is outputted.
 
@@ -940,6 +975,7 @@ class Max(PyoObject):
 
     """
     def __init__(self, input, comp=0.5, mul=1, add=0):
+        pyoArgsAssert(self, "oOOO", input, comp, mul, add)
         PyoObject.__init__(self, mul, add)
         self._input = input
         self._comp = comp
@@ -953,12 +989,13 @@ class Max(PyoObject):
 
         :Args:
 
-            x : PyoObject
+            x: PyoObject
                 New signal to process.
-            fadetime : float, optional
+            fadetime: float, optional
                 Crossfade time between old and new input. Default to 0.05.
 
         """
+        pyoArgsAssert(self, "oN", x, fadetime)
         self._input = x
         self._in_fader.setInput(x, fadetime)
 
@@ -968,10 +1005,11 @@ class Max(PyoObject):
 
         :Args:
 
-            x : float or PyoObject
+            x: float or PyoObject
                 New `comp` attribute.
 
         """
+        pyoArgsAssert(self, "O", x)
         self._comp = x
         x, lmax = convertArgsToLists(x)
         [obj.setComp(wrap(x,i)) for i, obj in enumerate(self._base_objs)]
@@ -992,4 +1030,4 @@ class Max(PyoObject):
         """float or PyoObject. Comparison value."""
         return self._comp
     @comp.setter
-    def comp(self, x): self.setComp(x)
\ No newline at end of file
+    def comp(self, x): self.setComp(x)
