@@ -9,6 +9,8 @@ spectral domain.
 
 """
 
+from __future__ import division
+from __future__ import absolute_import
 """
 Copyright 2009-2015 Olivier Belanger
 
@@ -28,8 +30,9 @@ GNU Lesser General Public License for more details.
 You should have received a copy of the GNU Lesser General Public
 License along with pyo.  If not, see <http://www.gnu.org/licenses/>.
 """
-from _core import *
-from _maps import *
+
+from ._core import *
+from ._maps import *
 
 class FFT(PyoObject):
     """
@@ -48,18 +51,18 @@ class FFT(PyoObject):
 
     :Args:
 
-        input : PyoObject
+        input: PyoObject
             Input signal to process.
-        size : int {pow-of-two > 4}, optional
+        size: int {pow-of-two > 4}, optional
             FFT size. Must be a power of two greater than 4.
             The FFT size is the number of samples used in each
             analysis frame. Defaults to 1024.
-        overlaps : int, optional
+        overlaps: int, optional
             The number of overlaped analysis block. Must be a
             positive integer. More overlaps can greatly improved
             sound quality synthesis but it is also more CPU
             expensive. Defaults to 4.
-        wintype : int, optional
+        wintype: int, optional
             Shape of the envelope used to filter each input frame.
             Possible shapes are :
                 0. rectangular (no windowing)
@@ -98,6 +101,7 @@ class FFT(PyoObject):
 
     """
     def __init__(self, input, size=1024, overlaps=4, wintype=2):
+        pyoArgsAssert(self, "oiIi", input, size, overlaps, wintype)
         PyoObject.__init__(self)
         self._real_dummy = []
         self._imag_dummy = []
@@ -111,7 +115,7 @@ class FFT(PyoObject):
         self._base_players = []
         for j in range(overlaps):
             for i in range(lmax):
-                hopsize = wrap(size,i) * j / overlaps
+                hopsize = wrap(size,i) * j // overlaps
                 self._base_players.append(FFTMain_base(wrap(in_fader,i), wrap(size,i), hopsize, wrap(wintype,i)))
         self._real_objs = []
         self._imag_objs = []
@@ -146,10 +150,10 @@ class FFT(PyoObject):
 
         :Args:
 
-            identifier : string {"real", "imag", "bin"}
+            identifier: string {"real", "imag", "bin"}
                 Address string parameter identifying audio stream.
                 Defaults to "real".
-            all : boolean, optional
+            all: boolean, optional
                 If True, the first value of each object's stream
                 will be returned as a list. Otherwise, only the value
                 of the first object's stream will be returned as a float.
@@ -167,12 +171,13 @@ class FFT(PyoObject):
 
         :Args:
 
-            x : PyoObject
+            x: PyoObject
                 New signal to process.
-            fadetime : float, optional
+            fadetime: float, optional
                 Crossfade time between old and new input. Default to 0.05.
 
         """
+        pyoArgsAssert(self, "oN", x, fadetime)
         self._input = x
         self._in_fader.setInput(x, fadetime)
 
@@ -200,16 +205,17 @@ class FFT(PyoObject):
 
         :Args:
 
-            x : int
+            x: int
                 new `size` attribute.
 
         """
+        pyoArgsAssert(self, "i", x)
         self._size = x
         x, lmax = convertArgsToLists(x)
-        poly = len(self._base_players) / self._overlaps
+        poly = len(self._base_players) // self._overlaps
         for j in range(self._overlaps):
             for i in range(poly):
-                hopsize = wrap(x,i) * j / self._overlaps
+                hopsize = wrap(x,i) * j // self._overlaps
                 self._base_players[j*poly+i].setSize(wrap(x,i), hopsize)
 
     def setWinType(self, x):
@@ -218,10 +224,11 @@ class FFT(PyoObject):
 
         :Args:
 
-            x : int
+            x: int
                 new `wintype` attribute.
 
         """
+        pyoArgsAssert(self, "i", x)
         self._wintype = x
         x, lmax = convertArgsToLists(x)
         [obj.setWinType(wrap(x,i)) for i, obj in enumerate(self._base_players)]
@@ -262,22 +269,22 @@ class IFFT(PyoObject):
 
     :Args:
 
-        inreal : PyoObject
+        inreal: PyoObject
             Input `real` signal.
-        inimag : PyoObject
+        inimag: PyoObject
             Input `imaginary` signal.
-        size : int {pow-of-two > 4}, optional
+        size: int {pow-of-two > 4}, optional
             FFT size. Must be a power of two greater than 4.
             The FFT size is the number of samples used in each
             analysis frame. This value must match the `size`
             attribute of the former FFT object. Defaults to 1024.
-        overlaps : int, optional
+        overlaps: int, optional
             The number of overlaped analysis block. Must be a
             positive integer. More overlaps can greatly improved
             sound quality synthesis but it is also more CPU
             expensive. This value must match the `overlaps`
             atribute of the former FFT object. Defaults to 4.
-        wintype : int, optional
+        wintype: int, optional
             Shape of the envelope used to filter each output frame.
             Possible shapes are :
                 0. rectangular (no windowing)
@@ -313,6 +320,7 @@ class IFFT(PyoObject):
 
     """
     def __init__(self, inreal, inimag, size=1024, overlaps=4, wintype=2, mul=1, add=0):
+        pyoArgsAssert(self, "ooiIiOO", inreal, inimag, size, overlaps, wintype, mul, add)
         PyoObject.__init__(self, mul, add)
         self._inreal = inreal
         self._inimag = inimag
@@ -323,9 +331,9 @@ class IFFT(PyoObject):
         self._in_fader2 = InputFader(inimag)
         in_fader, in_fader2, size, wintype, mul, add, lmax = convertArgsToLists(self._in_fader, self._in_fader2, size, wintype, mul, add)
         self._base_objs = []
-        ratio = lmax / overlaps
+        ratio = lmax // overlaps
         for i in range(lmax):
-            hopsize = wrap(size,i) * ((i/ratio)%overlaps) / overlaps
+            hopsize = wrap(size,i) * ((i // ratio) % overlaps) // overlaps
             self._base_objs.append(IFFT_base(wrap(in_fader,i), wrap(in_fader2,i), wrap(size,i), hopsize, wrap(wintype,i), wrap(mul,i), wrap(add,i)))
 
     def __len__(self):
@@ -337,12 +345,13 @@ class IFFT(PyoObject):
 
         :Args:
 
-            x : PyoObject
+            x: PyoObject
                 New input `real` signal.
-            fadetime : float, optional
+            fadetime: float, optional
                 Crossfade time between old and new input. Default to 0.05.
 
         """
+        pyoArgsAssert(self, "oN", x, fadetime)
         self._inreal = x
         self._in_fader.setInput(x, fadetime)
 
@@ -352,12 +361,13 @@ class IFFT(PyoObject):
 
         :Args:
 
-            x : PyoObject
+            x: PyoObject
                 New input `imag` signal.
-            fadetime : float, optional
+            fadetime: float, optional
                 Crossfade time between old and new input. Default to 0.05.
 
         """
+        pyoArgsAssert(self, "oN", x, fadetime)
         self._inimag = x
         self._in_fader2.setInput(x, fadetime)
 
@@ -367,15 +377,16 @@ class IFFT(PyoObject):
 
         :Args:
 
-            x : int
+            x: int
                 new `size` attribute.
 
         """
+        pyoArgsAssert(self, "i", x)
         self._size = x
         x, lmax = convertArgsToLists(x)
-        ratio = len(self._base_objs) / self._overlaps
+        ratio = len(self._base_objs) // self._overlaps
         for i, obj in enumerate(self._base_objs):
-            hopsize = wrap(x,i) * ((i/ratio)%self._overlaps) / self._overlaps
+            hopsize = wrap(x,i) * ((i // ratio) % self._overlaps) // self._overlaps
             self._base_objs[i].setSize(wrap(x,i), hopsize)
 
     def setWinType(self, x):
@@ -384,10 +395,11 @@ class IFFT(PyoObject):
 
         :Args:
 
-            x : int
+            x: int
                 new `wintype` attribute.
 
         """
+        pyoArgsAssert(self, "i", x)
         self._wintype = x
         x, lmax = convertArgsToLists(x)
         [obj.setWinType(wrap(x,i)) for i, obj in enumerate(self._base_objs)]
@@ -440,9 +452,9 @@ class CarToPol(PyoObject):
 
     :Args:
 
-        inreal : PyoObject
+        inreal: PyoObject
             Real input signal.
-        inimag : PyoObject
+        inimag: PyoObject
             Imaginary input signal.
 
     .. note::
@@ -473,6 +485,7 @@ class CarToPol(PyoObject):
 
     """
     def __init__(self, inreal, inimag, mul=1, add=0):
+        pyoArgsAssert(self, "ooOO", inreal, inimag, mul, add)
         PyoObject.__init__(self, mul, add)
         self._mag_dummy = []
         self._ang_dummy = []
@@ -508,10 +521,10 @@ class CarToPol(PyoObject):
 
         :Args:
 
-            identifier : string {"mag", "ang"}
+            identifier: string {"mag", "ang"}
                 Address string parameter identifying audio stream.
                 Defaults to "mag".
-            all : boolean, optional
+            all: boolean, optional
                 If True, the first value of each object's stream
                 will be returned as a list. Otherwise, only the value
                 of the first object's stream will be returned as a float.
@@ -529,12 +542,13 @@ class CarToPol(PyoObject):
 
         :Args:
 
-            x : PyoObject
+            x: PyoObject
                 New signal to process.
-            fadetime : float, optional
+            fadetime: float, optional
                 Crossfade time between old and new input. Default to 0.05.
 
         """
+        pyoArgsAssert(self, "oN", x, fadetime)
         self._inreal = x
         self._in_fader.setInput(x, fadetime)
 
@@ -544,12 +558,13 @@ class CarToPol(PyoObject):
 
         :Args:
 
-            x : PyoObject
+            x: PyoObject
                 New signal to process.
-            fadetime : float, optional
+            fadetime: float, optional
                 Crossfade time between old and new input. Default to 0.05.
 
         """
+        pyoArgsAssert(self, "oN", x, fadetime)
         self._inimag = x
         self._in_fader2.setInput(x, fadetime)
 
@@ -583,9 +598,9 @@ class PolToCar(PyoObject):
 
     :Args:
 
-        inmag : PyoObject
+        inmag: PyoObject
             Magintude input signal.
-        inang : PyoObject
+        inang: PyoObject
             Angle input signal.
 
     .. note::
@@ -616,6 +631,7 @@ class PolToCar(PyoObject):
 
     """
     def __init__(self, inmag, inang, mul=1, add=0):
+        pyoArgsAssert(self, "ooOO", inmag, inang, mul, add)
         PyoObject.__init__(self, mul, add)
         self._real_dummy = []
         self._imag_dummy = []
@@ -651,10 +667,10 @@ class PolToCar(PyoObject):
 
         :Args:
 
-            identifier : string {"real", "imag"}
+            identifier: string {"real", "imag"}
                 Address string parameter identifying audio stream.
                 Defaults to "mag".
-            all : boolean, optional
+            all: boolean, optional
                 If True, the first value of each object's stream
                 will be returned as a list. Otherwise, only the value
                 of the first object's stream will be returned as a float.
@@ -672,12 +688,13 @@ class PolToCar(PyoObject):
 
         :Args:
 
-            x : PyoObject
+            x: PyoObject
                 New signal to process.
-            fadetime : float, optional
+            fadetime: float, optional
                 Crossfade time between old and new input. Default to 0.05.
 
         """
+        pyoArgsAssert(self, "oN", x, fadetime)
         self._inmag = x
         self._in_fader.setInput(x, fadetime)
 
@@ -687,12 +704,13 @@ class PolToCar(PyoObject):
 
         :Args:
 
-            x : PyoObject
+            x: PyoObject
                 New signal to process.
-            fadetime : float, optional
+            fadetime: float, optional
                 Crossfade time between old and new input. Default to 0.05.
 
         """
+        pyoArgsAssert(self, "oN", x, fadetime)
         self._inang = x
         self._in_fader2.setInput(x, fadetime)
 
@@ -728,12 +746,12 @@ class FrameDelta(PyoObject):
 
     :Args:
 
-        input : PyoObject
+        input: PyoObject
             Phase input signal, usually from an FFT analysis.
-        framesize : int, optional
+        framesize: int, optional
             Frame size in samples. Usually the same as the FFT size.
             Defaults to 1024.
-        overlaps : int, optional
+        overlaps: int, optional
             Number of overlaps in incomming signal. Usually the same
             as the FFT overlaps. Defaults to 4.
 
@@ -764,13 +782,14 @@ class FrameDelta(PyoObject):
 
     """
     def __init__(self, input, framesize=1024, overlaps=4, mul=1, add=0):
+        pyoArgsAssert(self, "oiiOO", input, framesize, overlaps, mul, add)
         PyoObject.__init__(self, mul, add)
         self._input = input
         self._framesize = framesize
         self._overlaps = overlaps
         self._in_fader = InputFader(input)
         in_fader, framesize, overlaps, mul, add, lmax = convertArgsToLists(self._in_fader, framesize, overlaps, mul, add)
-        num_of_mains = len(self._in_fader) / self._overlaps
+        num_of_mains = len(self._in_fader) // self._overlaps
         self._base_players = []
         for j in range(num_of_mains):
             objs_list = []
@@ -781,7 +800,7 @@ class FrameDelta(PyoObject):
         self._base_objs = []
         for i in range(lmax):
             base_player = i % num_of_mains
-            overlap = i / num_of_mains
+            overlap = i // num_of_mains
             self._base_objs.append(FrameDelta_base(self._base_players[base_player], overlap, wrap(mul,i), wrap(add,i)))
 
     def out(self, chnl=0, inc=1, dur=0, delay=0):
@@ -793,12 +812,13 @@ class FrameDelta(PyoObject):
 
         :Args:
 
-            x : PyoObject
+            x: PyoObject
                 New signal to process.
-            fadetime : float, optional
+            fadetime: float, optional
                 Crossfade time between old and new input. Default to 0.05.
 
         """
+        pyoArgsAssert(self, "oN", x, fadetime)
         self._input = x
         self._in_fader.setInput(x, fadetime)
 
@@ -808,10 +828,11 @@ class FrameDelta(PyoObject):
 
         :Args:
 
-            x : int
+            x: int
                 new `framesize` attribute.
 
         """
+        pyoArgsAssert(self, "i", x)
         self._framesize = x
         x, lmax = convertArgsToLists(x)
         [obj.setFrameSize(wrap(x,i)) for i, obj in enumerate(self._base_players)]
@@ -848,12 +869,12 @@ class FrameAccum(PyoObject):
 
     :Args:
 
-        input : PyoObject
+        input: PyoObject
             Phase input signal.
-        framesize : int, optional
+        framesize: int, optional
             Frame size in samples. Usually same as the FFT size.
             Defaults to 1024.
-        overlaps : int, optional
+        overlaps: int, optional
             Number of overlaps in incomming signal. Usually the same
             as the FFT overlaps. Defaults to 4.
 
@@ -884,13 +905,14 @@ class FrameAccum(PyoObject):
 
     """
     def __init__(self, input, framesize=1024, overlaps=4, mul=1, add=0):
+        pyoArgsAssert(self, "oiiOO", input, framesize, overlaps, mul, add)
         PyoObject.__init__(self, mul, add)
         self._input = input
         self._framesize = framesize
         self._overlaps = overlaps
         self._in_fader = InputFader(input)
         in_fader, framesize, overlaps, mul, add, lmax = convertArgsToLists(self._in_fader, framesize, overlaps, mul, add)
-        num_of_mains = len(self._in_fader) / self._overlaps
+        num_of_mains = len(self._in_fader) // self._overlaps
         self._base_players = []
         for j in range(num_of_mains):
             objs_list = []
@@ -901,7 +923,7 @@ class FrameAccum(PyoObject):
         self._base_objs = []
         for i in range(lmax):
             base_player = i % num_of_mains
-            overlap = i / num_of_mains
+            overlap = i // num_of_mains
             self._base_objs.append(FrameAccum_base(self._base_players[base_player], overlap, wrap(mul,i), wrap(add,i)))
 
     def out(self, chnl=0, inc=1, dur=0, delay=0):
@@ -913,12 +935,13 @@ class FrameAccum(PyoObject):
 
         :Args:
 
-            x : PyoObject
+            x: PyoObject
                 New signal to process.
-            fadetime : float, optional
+            fadetime: float, optional
                 Crossfade time between old and new input. Default to 0.05.
 
         """
+        pyoArgsAssert(self, "oN", x, fadetime)
         self._input = x
         self._in_fader.setInput(x, fadetime)
 
@@ -928,10 +951,11 @@ class FrameAccum(PyoObject):
 
         :Args:
 
-            x : int
+            x: int
                 new `framesize` attribute.
 
         """
+        pyoArgsAssert(self, "i", x)
         self._framesize = x
         x, lmax = convertArgsToLists(x)
         [obj.setFrameSize(wrap(x,i)) for i, obj in enumerate(self._base_players)]
@@ -961,23 +985,23 @@ class Vectral(PyoObject):
 
     :Args:
 
-        input : PyoObject
+        input: PyoObject
             Magnitude input signal, usually from an FFT analysis.
-        framesize : int, optional
+        framesize: int, optional
             Frame size in samples. Usually the same as the FFT size.
             Defaults to 1024.
-        overlaps : int, optional
+        overlaps: int, optional
             Number of overlaps in incomming signal. Usually the same
             as the FFT overlaps. Defaults to 4.
-        up : float or PyoObject, optional
+        up: float or PyoObject, optional
             Filter coefficient for increasing bins, between 0 and 1.
             Lower values results in a longer ramp time for bin magnitude.
             Defaults to 1.
-        down : float or PyoObject, optional
+        down: float or PyoObject, optional
             Filter coefficient for decreasing bins, between 0 and 1.
             Lower values results in a longer decay time for bin magnitude.
             Defaults to 0.7
-        damp : float or PyoObject, optional
+        damp: float or PyoObject, optional
             High frequencies damping factor, between 0 and 1. Lower values
             mean more damping. Defaults to 0.9.
 
@@ -999,6 +1023,7 @@ class Vectral(PyoObject):
 
     """
     def __init__(self, input, framesize=1024, overlaps=4, up=1.0, down=0.7, damp=0.9, mul=1, add=0):
+        pyoArgsAssert(self, "oiiOOOOO", input, framesize, overlaps, up, down, damp, mul, add)
         PyoObject.__init__(self, mul, add)
         self._input = input
         self._framesize = framesize
@@ -1008,7 +1033,7 @@ class Vectral(PyoObject):
         self._damp = damp
         self._in_fader = InputFader(input)
         in_fader, framesize, overlaps, up, down, damp, mul, add, lmax = convertArgsToLists(self._in_fader, framesize, overlaps, up, down, damp, mul, add)
-        num_of_mains = len(self._in_fader) / self._overlaps
+        num_of_mains = len(self._in_fader) // self._overlaps
         self._base_players = []
         for j in range(num_of_mains):
             objs_list = []
@@ -1019,7 +1044,7 @@ class Vectral(PyoObject):
         self._base_objs = []
         for i in range(lmax):
             base_player = i % num_of_mains
-            overlap = i / num_of_mains
+            overlap = i // num_of_mains
             self._base_objs.append(Vectral_base(self._base_players[base_player], overlap, wrap(mul,i), wrap(add,i)))
 
     def out(self, chnl=0, inc=1, dur=0, delay=0):
@@ -1031,12 +1056,13 @@ class Vectral(PyoObject):
 
         :Args:
 
-            x : PyoObject
+            x: PyoObject
                 New signal to process.
-            fadetime : float, optional
+            fadetime: float, optional
                 Crossfade time between old and new input. Default to 0.05.
 
         """
+        pyoArgsAssert(self, "oN", x, fadetime)
         self._input = x
         self._in_fader.setInput(x, fadetime)
 
@@ -1046,10 +1072,11 @@ class Vectral(PyoObject):
 
         :Args:
 
-            x : int
+            x: int
                 new `framesize` attribute.
 
         """
+        pyoArgsAssert(self, "i", x)
         self._framesize = x
         x, lmax = convertArgsToLists(x)
         [obj.setFrameSize(wrap(x,i)) for i, obj in enumerate(self._base_players)]
@@ -1060,10 +1087,11 @@ class Vectral(PyoObject):
 
         :Args:
 
-            x : float or PyoObject
+            x: float or PyoObject
                 new `up` attribute.
 
         """
+        pyoArgsAssert(self, "O", x)
         self._up = x
         x, lmax = convertArgsToLists(x)
         [obj.setUp(wrap(x,i)) for i, obj in enumerate(self._base_players)]
@@ -1074,10 +1102,11 @@ class Vectral(PyoObject):
 
         :Args:
 
-            x : float or PyoObject
+            x: float or PyoObject
                 new `down` attribute.
 
         """
+        pyoArgsAssert(self, "O", x)
         self._down = x
         x, lmax = convertArgsToLists(x)
         [obj.setDown(wrap(x,i)) for i, obj in enumerate(self._base_players)]
@@ -1088,10 +1117,11 @@ class Vectral(PyoObject):
 
         :Args:
 
-            x : float or PyoObject
+            x: float or PyoObject
                 new `damp` attribute.
 
         """
+        pyoArgsAssert(self, "O", x)
         self._damp = x
         x, lmax = convertArgsToLists(x)
         [obj.setDamp(wrap(x,i)) for i, obj in enumerate(self._base_players)]
@@ -1150,20 +1180,20 @@ class CvlVerb(PyoObject):
 
     :Args:
 
-        input : PyoObject
+        input: PyoObject
             Input signal to process.
-        impulse : string, optional
+        impulse: string, optional
             Path to the impulse response soundfile. The file must have the same
             sampling rate as the server to get the proper convolution. Available at
             initialization time only. Defaults to 'IRMediumHallStereo.wav', located
             in pyolib SNDS_PATH folder.
-        size : int {pow-of-two}, optional
+        size: int {pow-of-two}, optional
             The size in samples of each partition of the impulse file. Small size means
             smaller latency but more computation time. If not a power-of-2, the object
             will find the next power-of-2 greater and use that as the actual partition size.
             This value must also be greater or equal than the server's buffer size.
             Available at initialization time only. Defaults to 1024.
-        bal : float or PyoObject, optional
+        bal: float or PyoObject, optional
             Balance between wet and dry signal, between 0 and 1. 0 means no
             reverb. Defaults to 0.25.
 
@@ -1174,6 +1204,7 @@ class CvlVerb(PyoObject):
 
     """
     def __init__(self, input, impulse=SNDS_PATH+"/IRMediumHallStereo.wav", bal=0.25, size=1024, mul=1, add=0):
+        pyoArgsAssert(self, "osOiOO", input, impulse, bal, size, mul, add)
         PyoObject.__init__(self, mul, add)
         self._input = input
         self._impulse = impulse
@@ -1194,12 +1225,13 @@ class CvlVerb(PyoObject):
 
         :Args:
 
-            x : PyoObject
+            x: PyoObject
                 New signal to process.
-            fadetime : float, optional
+            fadetime: float, optional
                 Crossfade time between old and new input. Default to 0.05.
 
         """
+        pyoArgsAssert(self, "oN", x, fadetime)
         self._input = x
         self._in_fader.setInput(x, fadetime)
 
@@ -1209,10 +1241,11 @@ class CvlVerb(PyoObject):
 
         :Args:
 
-            x : float or PyoObject
+            x: float or PyoObject
                 new `bal` attribute.
 
         """
+        pyoArgsAssert(self, "O", x)
         self._bal = x
         x, lmax = convertArgsToLists(x)
         [obj.setBal(wrap(x,i)) for i, obj in enumerate(self._base_objs)]
@@ -1234,4 +1267,4 @@ class CvlVerb(PyoObject):
         """float or PyoObject. Wet / dry balance."""
         return self._bal
     @bal.setter
-    def bal(self, x): self.setBal(x)
\ No newline at end of file
+    def bal(self, x): self.setBal(x)
