@@ -6,6 +6,7 @@ processing chain or as parameter's modifiers.
 
 """
 
+from __future__ import absolute_import
 """
 Copyright 2009-2015 Olivier Belanger
 
@@ -25,8 +26,8 @@ GNU Lesser General Public License for more details.
 You should have received a copy of the GNU Lesser General Public
 License along with pyo.  If not, see <http://www.gnu.org/licenses/>.
 """
-from _core import *
-from _maps import *
+from ._core import *
+from ._maps import *
 
 ######################################################################
 ### Sources
@@ -39,9 +40,9 @@ class Sine(PyoObject):
 
     :Args:
 
-        freq : float or PyoObject, optional
+        freq: float or PyoObject, optional
             Frequency in cycles per second. Defaults to 1000.
-        phase : float or PyoObject, optional
+        phase: float or PyoObject, optional
             Phase of sampling, expressed as a fraction of a cycle (0 to 1).
             Defaults to 0.
 
@@ -68,7 +69,7 @@ class Sine(PyoObject):
 
         :Args:
 
-            x : float or PyoObject
+            x: float or PyoObject
                 new `freq` attribute.
 
         """
@@ -83,7 +84,7 @@ class Sine(PyoObject):
 
         :Args:
 
-            x : float or PyoObject
+            x: float or PyoObject
                 new `phase` attribute.
 
         """
@@ -99,7 +100,6 @@ class Sine(PyoObject):
         """
         [obj.reset() for i, obj in enumerate(self._base_objs)]
 
-
     def ctrl(self, map_list=None, title=None, wxnoserver=False):
         self._map_list = [SLMapFreq(self._freq), SLMapPhase(self._phase), SLMapMul(self._mul)]
         PyoObject.ctrl(self, map_list, title, wxnoserver)
@@ -118,6 +118,107 @@ class Sine(PyoObject):
     @phase.setter
     def phase(self, x): self.setPhase(x)
 
+class FastSine(PyoObject):
+    """
+    A fast sine wave approximation using the formula of a parabola.
+
+    This object implements two sin approximations that are even faster
+    than a linearly interpolated table lookup. With `quality` set to 1,
+    the approximation is more accurate but also more expensive on the CPU
+    (still cheaper than a Sine object). With `quality` = 0, the algorithm
+    gives a worse approximation of the sin function but it is very fast
+    (and well suitable for generating LFO).
+
+    :Parent: :py:class:`PyoObject`
+
+    :Args:
+
+        freq: float or PyoObject, optional
+            Frequency in cycles per second. Defaults to 1000.
+        initphase: float, optional
+            Initial phase of the oscillator, between 0 and 1. Available
+            at initialization time only. Defaults to 0.
+        quality: int, optional
+            Sets the approximation quality. 1 is more accurate but also
+            more expensive on the CPU. 0 is a cheaper algorithm but is
+            very fast. Defaults to 1.
+
+    .. seealso::
+
+        :py:class:`Sine`
+
+    >>> s = Server().boot()
+    >>> s.start()
+    >>> lfo = FastSine(freq=[4,5], quality=0, mul=0.02, add=1)
+    >>> syn = FastSine(freq=500*lfo, quality=1, mul=0.4).out()
+
+    """
+    def __init__(self, freq=1000, initphase=0.0, quality=1, mul=1, add=0):
+        pyoArgsAssert(self, "OniOO", freq, initphase, quality, mul, add)
+        PyoObject.__init__(self, mul, add)
+        self._freq = freq
+        self._initphase = initphase
+        self._quality = quality
+        freq, initphase, quality, mul, add, lmax = convertArgsToLists(freq, initphase, quality, mul, add)
+        self._base_objs = [FastSine_base(wrap(freq,i), wrap(initphase,i), wrap(quality,i), wrap(mul,i), wrap(add,i)) for i in range(lmax)]
+
+    def setFreq(self, x):
+        """
+        Replace the `freq` attribute.
+
+        :Args:
+
+            x: float or PyoObject
+                new `freq` attribute.
+
+        """
+        pyoArgsAssert(self, "O", x)
+        self._freq = x
+        x, lmax = convertArgsToLists(x)
+        [obj.setFreq(wrap(x,i)) for i, obj in enumerate(self._base_objs)]
+
+    def setQuality(self, x):
+        """
+        Replace the `quality` attribute.
+
+        :Args:
+
+            x: int {0 or 1}
+                new `quality` attribute.
+
+        """
+        pyoArgsAssert(self, "i", x)
+        self._quality = x
+        x, lmax = convertArgsToLists(x)
+        [obj.setQuality(wrap(x,i)) for i, obj in enumerate(self._base_objs)]
+
+    def reset(self):
+        """
+        Resets current phase to 0.
+
+        """
+        [obj.reset() for i, obj in enumerate(self._base_objs)]
+
+    def ctrl(self, map_list=None, title=None, wxnoserver=False):
+        self._map_list = [SLMapFreq(self._freq),
+                          SLMap(0, 1, "lin", "quality", self._quality, res="int", dataOnly=True),
+                          SLMapMul(self._mul)]
+        PyoObject.ctrl(self, map_list, title, wxnoserver)
+
+    @property
+    def freq(self):
+        """float or PyoObject. Frequency in cycles per second."""
+        return self._freq
+    @freq.setter
+    def freq(self, x): self.setFreq(x)
+
+    @property
+    def quality(self):
+        """int. Quality of the sin approximation."""
+        return self._quality
+    @quality.setter
+    def quality(self, x): self.setQuality(x)
+
 class SineLoop(PyoObject):
     """
     A simple sine wave oscillator with feedback.
@@ -129,9 +230,9 @@ class SineLoop(PyoObject):
 
     :Args:
 
-        freq : float or PyoObject, optional
+        freq: float or PyoObject, optional
             Frequency in cycles per second. Defaults to 1000.
-        feedback : float or PyoObject, optional
+        feedback: float or PyoObject, optional
             Amount of the output signal added to position increment, between 0 and 1.
             Controls the brightness. Defaults to 0.
 
@@ -159,7 +260,7 @@ class SineLoop(PyoObject):
 
         :Args:
 
-            x : float or PyoObject
+            x: float or PyoObject
                 new `freq` attribute.
 
         """
@@ -174,7 +275,7 @@ class SineLoop(PyoObject):
 
         :Args:
 
-            x : float or PyoObject
+            x: float or PyoObject
                 new `feedback` attribute.
 
         """
@@ -211,9 +312,9 @@ class Phasor(PyoObject):
 
     :Args:
 
-        freq : float or PyoObject, optional
+        freq: float or PyoObject, optional
             Frequency in cycles per second. Defaults to 100.
-        phase : float or PyoObject, optional
+        phase: float or PyoObject, optional
             Phase of sampling, expressed as a fraction of a cycle (0 to 1).
             Defaults to 0.
 
@@ -241,7 +342,7 @@ class Phasor(PyoObject):
 
         :Args:
 
-            x : float or PyoObject
+            x: float or PyoObject
                 new `freq` attribute.
 
         """
@@ -256,7 +357,7 @@ class Phasor(PyoObject):
 
         :Args:
 
-            x : float or PyoObject
+            x: float or PyoObject
                 new `phase` attribute.
 
         """
@@ -272,7 +373,6 @@ class Phasor(PyoObject):
         """
         [obj.reset() for i, obj in enumerate(self._base_objs)]
 
-
     def ctrl(self, map_list=None, title=None, wxnoserver=False):
         self._map_list = [SLMapFreq(self._freq), SLMapPhase(self._phase), SLMapMul(self._mul)]
         PyoObject.ctrl(self, map_list, title, wxnoserver)
@@ -299,7 +399,7 @@ class Input(PyoObject):
 
     :Args:
 
-        chnl : int, optional
+        chnl: int, optional
             Input channel to read from. Defaults to 0.
 
     .. note::
@@ -347,7 +447,7 @@ class Noise(PyoObject):
 
         :Args:
 
-            x : int, {0, 1}
+            x: int, {0, 1}
                 0 uses the system rand() method to generate number. Used as default.
                 1 uses a simple linear congruential generator, cheaper than rand().
 
@@ -429,12 +529,12 @@ class FM(PyoObject):
 
     :Args:
 
-        carrier : float or PyoObject, optional
+        carrier: float or PyoObject, optional
             Carrier frequency in cycles per second. Defaults to 100.
-        ratio : float or PyoObject, optional
+        ratio: float or PyoObject, optional
             A factor that, when multiplied by the `carrier` parameter,
             gives the modulator frequency. Defaults to 0.5.
-        index : float or PyoObject, optional
+        index: float or PyoObject, optional
             The modulation index. This value multiplied by the modulator
             frequency gives the modulator amplitude. Defaults to 5.
 
@@ -461,7 +561,7 @@ class FM(PyoObject):
 
         :Args:
 
-            x : float or PyoObject
+            x: float or PyoObject
                 new `carrier` attribute.
 
         """
@@ -476,7 +576,7 @@ class FM(PyoObject):
 
         :Args:
 
-            x : float or PyoObject
+            x: float or PyoObject
                 new `ratio` attribute.
 
         """
@@ -491,7 +591,7 @@ class FM(PyoObject):
 
         :Args:
 
-            x : float or PyoObject
+            x: float or PyoObject
                 new `index` attribute.
 
         """
@@ -539,17 +639,17 @@ class CrossFM(PyoObject):
 
     :Args:
 
-        carrier : float or PyoObject, optional
+        carrier: float or PyoObject, optional
             Carrier frequency in cycles per second. Defaults to 100.
-        ratio : float or PyoObject, optional
+        ratio: float or PyoObject, optional
             A factor that, when multiplied by the `carrier` parameter,
             gives the modulator frequency. Defaults to 0.5.
-        ind1 : float or PyoObject, optional
+        ind1: float or PyoObject, optional
             The carrier index. This value multiplied by the carrier
             frequency gives the carrier amplitude for modulating the
             modulation oscillator frequency.
             Defaults to 2.
-        ind1 : float or PyoObject, optional
+        ind1: float or PyoObject, optional
             The modulation index. This value multiplied by the modulation
             frequency gives the modulation amplitude for modulating the
             carrier oscillator frequency.
@@ -579,7 +679,7 @@ class CrossFM(PyoObject):
 
         :Args:
 
-            x : float or PyoObject
+            x: float or PyoObject
                 new `carrier` attribute.
 
         """
@@ -594,7 +694,7 @@ class CrossFM(PyoObject):
 
         :Args:
 
-            x : float or PyoObject
+            x: float or PyoObject
                 new `ratio` attribute.
 
         """
@@ -609,7 +709,7 @@ class CrossFM(PyoObject):
 
         :Args:
 
-            x : float or PyoObject
+            x: float or PyoObject
                 new `ind1` attribute.
 
         """
@@ -624,7 +724,7 @@ class CrossFM(PyoObject):
 
         :Args:
 
-            x : float or PyoObject
+            x: float or PyoObject
                 new `ind2` attribute.
 
         """
@@ -680,9 +780,9 @@ class Blit(PyoObject):
 
     :Args:
 
-        freq : float or PyoObject, optional
+        freq: float or PyoObject, optional
             Frequency in cycles per second. Defaults to 100.
-        harms : float or PyoObject, optional
+        harms: float or PyoObject, optional
             Number of harmonics in the generated spectrum. Defaults to 40.
 
     >>> s = Server().boot()
@@ -706,7 +806,7 @@ class Blit(PyoObject):
 
         :Args:
 
-            x : float or PyoObject
+            x: float or PyoObject
                 new `freq` attribute.
 
         """
@@ -721,7 +821,7 @@ class Blit(PyoObject):
 
         :Args:
 
-            x : float or PyoObject
+            x: float or PyoObject
                 new `harms` attribute.
 
         """
@@ -763,12 +863,12 @@ class Rossler(PyoObject):
 
     :Args:
 
-        pitch : float or PyoObject, optional
+        pitch: float or PyoObject, optional
             Controls the speed, in the range 0 -> 1, of the variations. With values
             below 0.2, this object can be used as a low frequency oscillator (LFO)
             and above 0.2, it will generate a broad spectrum noise with harmonic peaks.
             Defaults to 0.25.
-        chaos : float or PyoObject, optional
+        chaos: float or PyoObject, optional
             Controls the chaotic behavior, in the range 0 -> 1, of the oscillator.
             0 means nearly periodic while 1 is totally chaotic. Defaults to 0.5.
         stereo, boolean, optional
@@ -780,7 +880,7 @@ class Rossler(PyoObject):
 
     .. seealso::
 
-        :py:class:`Lorenz`
+        :py:class:`Lorenz`, :py:class:`ChenLee`
 
     >>> s = Server().boot()
     >>> s.start()
@@ -808,7 +908,7 @@ class Rossler(PyoObject):
 
         :Args:
 
-            x : float or PyoObject
+            x: float or PyoObject
                 new `pitch` attribute. {0. -> 1.}
 
         """
@@ -826,7 +926,7 @@ class Rossler(PyoObject):
 
         :Args:
 
-            x : float or PyoObject
+            x: float or PyoObject
                 new `chaos` attribute. {0. -> 1.}
 
         """
@@ -870,12 +970,12 @@ class Lorenz(PyoObject):
 
     :Args:
 
-        pitch : float or PyoObject, optional
+        pitch: float or PyoObject, optional
             Controls the speed, in the range 0 -> 1, of the variations. With values
             below 0.2, this object can be used as a low frequency oscillator (LFO)
             and above 0.2, it will generate a broad spectrum noise with harmonic peaks.
             Defaults to 0.25.
-        chaos : float or PyoObject, optional
+        chaos: float or PyoObject, optional
             Controls the chaotic behavior, in the range 0 -> 1, of the oscillator.
             0 means nearly periodic while 1 is totally chaotic. Defaults to 0.5
         stereo, boolean, optional
@@ -887,7 +987,7 @@ class Lorenz(PyoObject):
 
     .. seealso::
 
-        :py:class:`Rossler`
+        :py:class:`Rossler`, :py:class:`ChenLee`
 
     >>> s = Server().boot()
     >>> s.start()
@@ -915,7 +1015,7 @@ class Lorenz(PyoObject):
 
         :Args:
 
-            x : float or PyoObject
+            x: float or PyoObject
                 new `pitch` attribute. {0. -> 1.}
 
         """
@@ -933,7 +1033,114 @@ class Lorenz(PyoObject):
 
         :Args:
 
-            x : float or PyoObject
+            x: float or PyoObject
+                new `chaos` attribute. {0. -> 1.}
+
+        """
+        pyoArgsAssert(self, "O", x)
+        self._chaos = x
+        x, lmax = convertArgsToLists(x)
+        if self._stereo:
+            [obj.setChaos(wrap(x,i)) for i, obj in enumerate(self._base_objs) if (i % 2) == 0]
+        else:
+            [obj.setChaos(wrap(x,i)) for i, obj in enumerate(self._base_objs)]
+
+    def ctrl(self, map_list=None, title=None, wxnoserver=False):
+        self._map_list = [SLMap(0., 1., "lin", "pitch", self._pitch),
+                          SLMap(0., 1., "lin", "chaos", self._chaos), SLMapMul(self._mul)]
+        PyoObject.ctrl(self, map_list, title, wxnoserver)
+
+    @property
+    def pitch(self):
+        """float or PyoObject. Speed of the variations."""
+        return self._pitch
+    @pitch.setter
+    def pitch(self, x): self.setPitch(x)
+
+    @property
+    def chaos(self):
+        """float or PyoObject. Chaotic behavior."""
+        return self._chaos
+    @chaos.setter
+    def chaos(self, x): self.setChaos(x)
+
+class ChenLee(PyoObject):
+    """
+    Chaotic attractor for the Chen-Lee system.
+
+    The ChenLee attractor is a system of three non-linear ordinary differential
+    equations. These differential equations define a continuous-time dynamical
+    system that exhibits chaotic dynamics associated with the fractal properties
+    of the attractor.
+
+    :Parent: :py:class:`PyoObject`
+
+    :Args:
+
+        pitch: float or PyoObject, optional
+            Controls the speed, in the range 0 -> 1, of the variations. With values
+            below 0.2, this object can be used as a low frequency oscillator (LFO)
+            and above 0.2, it will generate a broad spectrum noise with harmonic peaks.
+            Defaults to 0.25.
+        chaos: float or PyoObject, optional
+            Controls the chaotic behavior, in the range 0 -> 1, of the oscillator.
+            0 means nearly periodic while 1 is totally chaotic. Defaults to 0.5
+        stereo, boolean, optional
+            If True, 2 streams will be generated, one with the X variable signal of
+            the algorithm and a second composed of the Y variable signal of the algorithm.
+            These two signal are strongly related in their frequency spectrum but
+            the Y signal is slightly out-of-phase. Useful to create alternating LFOs.
+            Available at initialization only. Defaults to False.
+
+    .. seealso::
+
+        :py:class:`Rossler`, :py:class:`Lorenz`
+
+    >>> s = Server().boot()
+    >>> s.start()
+    >>> a = ChenLee(pitch=.001, chaos=0.2, stereo=True, mul=.5, add=.5)
+    >>> b = ChenLee(pitch=1, chaos=a, mul=0.5).out()
+
+    """
+    def __init__(self, pitch=0.25, chaos=0.5, stereo=False, mul=1, add=0):
+        pyoArgsAssert(self, "OObOO", pitch, chaos, stereo, mul, add)
+        PyoObject.__init__(self, mul, add)
+        self._pitch = pitch
+        self._chaos = chaos
+        self._stereo = stereo
+        pitch, chaos, mul, add, lmax = convertArgsToLists(pitch, chaos, mul, add)
+        self._base_objs = []
+        self._alt_objs = []
+        for i in range(lmax):
+            self._base_objs.append(ChenLee_base(wrap(pitch,i), wrap(chaos,i), wrap(mul,i), wrap(add,i)))
+            if self._stereo:
+                self._base_objs.append(ChenLeeAlt_base(self._base_objs[-1], wrap(mul,i), wrap(add,i)))
+
+    def setPitch(self, x):
+        """
+        Replace the `pitch` attribute.
+
+        :Args:
+
+            x: float or PyoObject
+                new `pitch` attribute. {0. -> 1.}
+
+        """
+        pyoArgsAssert(self, "O", x)
+        self._pitch = x
+        x, lmax = convertArgsToLists(x)
+        if self._stereo:
+            [obj.setPitch(wrap(x,i)) for i, obj in enumerate(self._base_objs) if (i % 2) == 0]
+        else:
+            [obj.setPitch(wrap(x,i)) for i, obj in enumerate(self._base_objs)]
+
+    def setChaos(self, x):
+        """
+        Replace the `chaos` attribute.
+
+        :Args:
+
+            x: float or PyoObject
                 new `chaos` attribute. {0. -> 1.}
 
         """
@@ -972,12 +1179,13 @@ class LFO(PyoObject):
 
     :Args:
 
-        freq : float or PyoObject, optional
-            Oscillator frequency in cycles per second. Defaults to 100.
-        sharp : float or PyoObject, optional
+        freq: float or PyoObject, optional
+            Oscillator frequency in cycles per second. The frequency is
+            internally clamped between 0.00001 and sr/4. Defaults to 100.
+        sharp: float or PyoObject, optional
             Sharpness factor between 0 and 1. Sharper waveform results
             in more harmonics in the spectrum. Defaults to 0.5.
-        type : int, optional
+        type: int, optional
             Waveform type. eight possible values :
                 0. Saw up (default)
                 1. Saw down
@@ -1011,7 +1219,7 @@ class LFO(PyoObject):
 
         :Args:
 
-            x : float or PyoObject
+            x: float or PyoObject
                 New `freq` attribute, in cycles per seconds.
 
         """
@@ -1026,7 +1234,7 @@ class LFO(PyoObject):
 
         :Args:
 
-            x : float or PyoObject
+            x: float or PyoObject
                 New `sharp` attribute, in the range 0 -> 1.
 
         """
@@ -1041,7 +1249,7 @@ class LFO(PyoObject):
 
         :Args:
 
-            x : int
+            x: int
                 New `type` attribute. Choices are :
                     0. Saw up
                     1. Saw down
@@ -1066,9 +1274,8 @@ class LFO(PyoObject):
         """
         [obj.reset() for i, obj in enumerate(self._base_objs)]
 
-
     def ctrl(self, map_list=None, title=None, wxnoserver=False):
-        self._map_list = [SLMapFreq(self._freq),
+        self._map_list = [SLMap(0.1, self.getSamplingRate()*0.25, "log", "freq", self._freq),
                           SLMap(0., 1., "lin", "sharp", self._sharp),
                           SLMap(0, 7, "lin", "type", self._type, "int", dataOnly=True),
                           SLMapMul(self._mul)]
@@ -1118,16 +1325,16 @@ class SumOsc(PyoObject):
 
     :Args:
 
-        freq : float or PyoObject, optional
+        freq: float or PyoObject, optional
             Base frequency in cycles per second. Defaults to 100.
-        ratio : float or PyoObject, optional
+        ratio: float or PyoObject, optional
             A factor used to stretch or compress the partial serie by
             manipulating the frequency of the modulation oscillator.
             Integer ratios give harmonic spectra. Defaults to 0.5.
-        index : float or PyoObject, optional
+        index: float or PyoObject, optional
             Damping of successive partials, between 0 and 1. With a
             value of 0.5, each partial is 6dB lower than the previous
-            partial. Defaults to 5.
+            partial. Defaults to 0.5.
 
     >>> s = Server().boot()
     >>> s.start()
@@ -1152,7 +1359,7 @@ class SumOsc(PyoObject):
 
         :Args:
 
-            x : float or PyoObject
+            x: float or PyoObject
                 new `freq` attribute.
 
         """
@@ -1167,7 +1374,7 @@ class SumOsc(PyoObject):
 
         :Args:
 
-            x : float or PyoObject
+            x: float or PyoObject
                 new `ratio` attribute.
 
         """
@@ -1182,7 +1389,7 @@ class SumOsc(PyoObject):
 
         :Args:
 
-            x : float or PyoObject
+            x: float or PyoObject
                 new `index` attribute.
 
         """
@@ -1232,13 +1439,13 @@ class SuperSaw(PyoObject):
 
     :Args:
 
-        freq : float or PyoObject, optional
+        freq: float or PyoObject, optional
             Frequency in cycles per second. Defaults to 100.
-        detune : float or PyoObject, optional
+        detune: float or PyoObject, optional
             Depth of the detuning, between 0 and 1. 0 means all oscillators are
             tuned to the same frequency and 1 means sideband oscillators are at
             maximum detuning regarding the central frequency. Defaults to 0.5.
-        bal : float or PyoObject, optional
+        bal: float or PyoObject, optional
             Balance between central oscillator and sideband oscillators. A value
             of 0 outputs only the central oscillator while a value of 1 gives a
             mix of all oscillators with the central one lower than the sidebands.
@@ -1269,7 +1476,7 @@ class SuperSaw(PyoObject):
 
         :Args:
 
-            x : float or PyoObject
+            x: float or PyoObject
                 new `freq` attribute.
 
         """
@@ -1284,7 +1491,7 @@ class SuperSaw(PyoObject):
 
         :Args:
 
-            x : float or PyoObject
+            x: float or PyoObject
                 new `detune` attribute.
 
         """
@@ -1299,7 +1506,7 @@ class SuperSaw(PyoObject):
 
         :Args:
 
-            x : float or PyoObject
+            x: float or PyoObject
                 new `bal` attribute.
 
         """
@@ -1346,9 +1553,9 @@ class RCOsc(PyoObject):
 
     :Args:
 
-        freq : float or PyoObject, optional
+        freq: float or PyoObject, optional
             Frequency in cycles per second. Defaults to 100.
-        sharp : float or PyoObject, optional
+        sharp: float or PyoObject, optional
             Slope of the attack and decay of the waveform, between 0 and 1.
             A value of 0 gives a triangular waveform and 1 gives almost a
             square wave. Defaults to 0.25.
@@ -1377,7 +1584,7 @@ class RCOsc(PyoObject):
 
         :Args:
 
-            x : float or PyoObject
+            x: float or PyoObject
                 new `freq` attribute.
 
         """
@@ -1392,7 +1599,7 @@ class RCOsc(PyoObject):
 
         :Args:
 
-            x : float or PyoObject
+            x: float or PyoObject
                 new `sharp` attribute.
 
         """
@@ -1424,4 +1631,4 @@ class RCOsc(PyoObject):
         """float or PyoObject. Sharpness of the waveform."""
         return self._sharp
     @sharp.setter
-    def sharp(self, x): self.setSharp(x)
\ No newline at end of file
+    def sharp(self, x): self.setSharp(x)
