@@ -7,6 +7,7 @@ are done by giving row and column positions.
 
 """
 
+from __future__ import absolute_import
 """
 Copyright 2009-2015 Olivier Belanger
 
@@ -26,9 +27,8 @@ GNU Lesser General Public License for more details.
 You should have received a copy of the GNU Lesser General Public
 License along with pyo.  If not, see <http://www.gnu.org/licenses/>.
 """
-from _core import *
-from _maps import *
-from types import SliceType
+from ._core import *
+from ._maps import *
 
 class MatrixRec(PyoObject):
     """
@@ -46,14 +46,14 @@ class MatrixRec(PyoObject):
 
     :Args:
 
-        input : PyoObject
+        input: PyoObject
             Audio signal to write in the matrix.
-        matrix : PyoMatrixObject
+        matrix: PyoMatrixObject
             The matrix where to write samples.
-        fadetime : float, optional
+        fadetime: float, optional
             Fade time at the beginning and the end of the recording
             in seconds. Defaults to 0.
-        delay : int, optional
+        delay: int, optional
             Delay time, in samples, before the recording begins.
             Available at initialization time only. Defaults to 0.
 
@@ -63,7 +63,7 @@ class MatrixRec(PyoObject):
 
         MatrixRec has no `mul` and `add` attributes.
 
-        MatrixRec will sends a trigger signal at the end of the recording.
+        MatrixRec will send a trigger signal at the end of the recording.
         User can retreive the trigger streams by calling obj['trig']. See
         `TableRec` documentation for an example.
 
@@ -87,6 +87,7 @@ class MatrixRec(PyoObject):
 
     """
     def __init__(self, input, matrix, fadetime=0, delay=0):
+        pyoArgsAssert(self, "omni", input, matrix, fadetime, delay)
         PyoObject.__init__(self)
         self._input = input
         self._matrix = matrix
@@ -110,12 +111,13 @@ class MatrixRec(PyoObject):
 
         :Args:
 
-            x : PyoObject
+            x: PyoObject
                 New signal to process.
-            fadetime : float, optional
+            fadetime: float, optional
                 Crossfade time between old and new input. Defaults to 0.05.
 
         """
+        pyoArgsAssert(self, "oN", x, fadetime)
         self._input = x
         self._in_fader.setInput(x, fadetime)
 
@@ -125,10 +127,11 @@ class MatrixRec(PyoObject):
 
         :Args:
 
-            x : NewMatrix
+            x: NewMatrix
                 new `matrix` attribute.
 
         """
+        pyoArgsAssert(self, "m", x)
         self._matrix = x
         x, lmax = convertArgsToLists(x)
         [obj.setMatrix(wrap(x,i)) for i, obj in enumerate(self._base_objs)]
@@ -160,9 +163,9 @@ class MatrixRecLoop(PyoObject):
 
     :Args:
 
-        input : PyoObject
+        input: PyoObject
             Audio signal to write in the matrix.
-        matrix : PyoMatrixObject
+        matrix: PyoMatrixObject
             The matrix where to write samples.
 
     .. note::
@@ -171,7 +174,7 @@ class MatrixRecLoop(PyoObject):
 
         MatrixRecLoop has no `mul` and `add` attributes.
 
-        MatrixRecLoop will sends a trigger signal when reaching the end
+        MatrixRecLoop will send a trigger signal when reaching the end
         of the matrix. User can retreive the trigger streams by calling
         obj['trig']. See `TableRec` documentation for an example.
 
@@ -194,6 +197,7 @@ class MatrixRecLoop(PyoObject):
 
     """
     def __init__(self, input, matrix):
+        pyoArgsAssert(self, "om", input, matrix)
         PyoObject.__init__(self)
         self._input = input
         self._matrix = matrix
@@ -217,12 +221,13 @@ class MatrixRecLoop(PyoObject):
 
         :Args:
 
-            x : PyoObject
+            x: PyoObject
                 New signal to process.
-            fadetime : float, optional
+            fadetime: float, optional
                 Crossfade time between old and new input. Defaults to 0.05.
 
         """
+        pyoArgsAssert(self, "oN", x, fadetime)
         self._input = x
         self._in_fader.setInput(x, fadetime)
 
@@ -232,10 +237,11 @@ class MatrixRecLoop(PyoObject):
 
         :Args:
 
-            x : NewMatrix
+            x: NewMatrix
                 new `matrix` attribute.
 
         """
+        pyoArgsAssert(self, "m", x)
         self._matrix = x
         x, lmax = convertArgsToLists(x)
         [obj.setMatrix(wrap(x,i)) for i, obj in enumerate(self._base_objs)]
@@ -262,11 +268,11 @@ class MatrixPointer(PyoObject):
 
     :Args:
 
-        matrix : PyoMatrixObject
+        matrix: PyoMatrixObject
             Matrix containing the waveform samples.
-        x : PyoObject
+        x: PyoObject
             Normalized X position in the matrix between 0 and 1.
-        y : PyoObject
+        y: PyoObject
             Normalized Y position in the matrix between 0 and 1.
 
     >>> s = Server().boot()
@@ -285,6 +291,7 @@ class MatrixPointer(PyoObject):
 
     """
     def __init__(self, matrix, x, y, mul=1, add=0):
+        pyoArgsAssert(self, "mooOO", matrix, x, y, mul, add)
         PyoObject.__init__(self, mul, add)
         self._matrix = matrix
         self._x = x
@@ -298,10 +305,11 @@ class MatrixPointer(PyoObject):
 
         :Args:
 
-            x : PyoTableObject
+            x: PyoTableObject
                 new `matrix` attribute.
 
         """
+        pyoArgsAssert(self, "m", x)
         self._matrix = x
         x, lmax = convertArgsToLists(x)
         [obj.setMatrix(wrap(x,i)) for i, obj in enumerate(self._base_objs)]
@@ -312,10 +320,11 @@ class MatrixPointer(PyoObject):
 
         :Args:
 
-            x : PyoObject
+            x: PyoObject
                 new `x` attribute.
 
         """
+        pyoArgsAssert(self, "o", x)
         self._x = x
         x, lmax = convertArgsToLists(x)
         [obj.setX(wrap(x,i)) for i, obj in enumerate(self._base_objs)]
@@ -326,10 +335,11 @@ class MatrixPointer(PyoObject):
 
         :Args:
 
-            y : PyoObject
+            y: PyoObject
                 new `y` attribute.
 
         """
+        pyoArgsAssert(self, "o", x)
         self._y = x
         x, lmax = convertArgsToLists(x)
         [obj.setY(wrap(x,i)) for i, obj in enumerate(self._base_objs)]
@@ -372,12 +382,12 @@ class MatrixMorph(PyoObject):
 
     :Args:
 
-        input : PyoObject
+        input: PyoObject
             Morphing index between 0 and 1. 0 is the first matrix in the list
             and 1 is the last.
-        matrix : NewMatrix
+        matrix: NewMatrix
             The matrix where to write morphed function.
-        sources : list of PyoMatrixObject
+        sources: list of PyoMatrixObject
             List of matrices to interpolate from.
 
     .. note::
@@ -401,6 +411,7 @@ class MatrixMorph(PyoObject):
 
     """
     def __init__(self, input, matrix, sources):
+        pyoArgsAssert(self, "oml", input, matrix, sources)
         PyoObject.__init__(self)
         self._input = input
         self._matrix = matrix
@@ -425,12 +436,13 @@ class MatrixMorph(PyoObject):
 
         :Args:
 
-            x : PyoObject
+            x: PyoObject
                 New signal to process.
-            fadetime : float, optional
+            fadetime: float, optional
                 Crossfade time between old and new input. Defaults to 0.05.
 
         """
+        pyoArgsAssert(self, "oN", x, fadetime)
         self._input = x
         self._in_fader.setInput(x, fadetime)
 
@@ -440,10 +452,11 @@ class MatrixMorph(PyoObject):
 
         :Args:
 
-            x : NewMatrix
+            x: NewMatrix
                 new `matrix` attribute.
 
         """
+        pyoArgsAssert(self, "m", x)
         self._matrix = x
         x, lmax = convertArgsToLists(x)
         [obj.setMatrix(wrap(x,i)) for i, obj in enumerate(self._base_objs)]
@@ -454,10 +467,11 @@ class MatrixMorph(PyoObject):
 
         :Args:
 
-            x : list of PyoMatrixObject
+            x: list of PyoMatrixObject
                 new `sources` attribute.
 
         """
+        pyoArgsAssert(self, "l", x)
         self._sources = x
         self._base_sources = [source[0] for source in x]
         [obj.setSources(self._base_sources) for i, obj in enumerate(self._base_objs)]
@@ -481,4 +495,4 @@ class MatrixMorph(PyoObject):
         """list of PyoMatrixObject. List of matrices to interpolate from."""
         return self._sources
     @sources.setter
-    def sources(self, x): self.setSources(x)
\ No newline at end of file
+    def sources(self, x): self.setSources(x)
