@@ -8,6 +8,7 @@ used as controls and can't be sent to the output soundcard.
 
 """
 
+from __future__ import absolute_import
 """
 Copyright 2009-2015 Olivier Belanger
 
@@ -28,8 +29,8 @@ You should have received a copy of the GNU Lesser General Public
 License along with pyo.  If not, see <http://www.gnu.org/licenses/>.
 """
 import sys
-from _core import *
-from _maps import *
+from ._core import *
+from ._maps import *
 
 ######################################################################
 ### MIDI
@@ -45,15 +46,15 @@ class Midictl(PyoObject):
 
     :Args:
 
-        ctlnumber : int
+        ctlnumber: int
             Controller number.
-        minscale : float, optional
+        minscale: float, optional
             Low range value for mapping. Defaults to 0.
-        maxscale : float, optional
+        maxscale: float, optional
             High range value for mapping. Defaults to 1.
-        init : float, optional
+        init: float, optional
             Initial value. Defaults to 0.
-        channel : int, optional
+        channel: int, optional
             Midi channel. 0 means all channels. Defaults to 0.
 
     .. note::
@@ -71,6 +72,7 @@ class Midictl(PyoObject):
 
     """
     def __init__(self, ctlnumber, minscale=0, maxscale=1, init=0, channel=0, mul=1, add=0):
+        pyoArgsAssert(self, "innniOO", ctlnumber, minscale, maxscale, init, channel, mul, add)
         PyoObject.__init__(self, mul, add)
         self._ctlnumber = ctlnumber
         self._minscale = minscale
@@ -88,10 +90,11 @@ class Midictl(PyoObject):
 
         :Args:
 
-            x : int
+            x: int
                 new `ctlnumber` attribute.
 
         """
+        pyoArgsAssert(self, "i", x)
         self._ctlnumber = x
         x, lmax = convertArgsToLists(x)
         [obj.setCtlNumber(wrap(x,i)) for i, obj in enumerate(self._base_objs)]
@@ -102,10 +105,11 @@ class Midictl(PyoObject):
 
         :Args:
 
-            x : int
+            x: float
                 new `minscale` attribute.
 
         """
+        pyoArgsAssert(self, "n", x)
         self._minscale = x
         x, lmax = convertArgsToLists(x)
         [obj.setMinScale(wrap(x,i)) for i, obj in enumerate(self._base_objs)]
@@ -116,10 +120,11 @@ class Midictl(PyoObject):
 
         :Args:
 
-            x : int
+            x: float
                 new `maxscale` attribute.
 
         """
+        pyoArgsAssert(self, "n", x)
         self._maxscale = x
         x, lmax = convertArgsToLists(x)
         [obj.setMaxScale(wrap(x,i)) for i, obj in enumerate(self._base_objs)]
@@ -130,10 +135,11 @@ class Midictl(PyoObject):
 
         :Args:
 
-            x : int
+            x: int
                 new `channel` attribute.
 
         """
+        pyoArgsAssert(self, "i", x)
         self._channel = x
         x, lmax = convertArgsToLists(x)
         [obj.setChannel(wrap(x,i)) for i, obj in enumerate(self._base_objs)]
@@ -144,26 +150,36 @@ class Midictl(PyoObject):
 
         :Args:
 
-            x : float
+            x: float
                 new current value.
 
         """
+        pyoArgsAssert(self, "n", x)
         x, lmax = convertArgsToLists(x)
         [obj.setValue(wrap(x,i)) for i, obj in enumerate(self._base_objs)]
 
     def setInterpolation(self, x):
         """
-        Activate/Deactivate interpolation. Activated by default.
+        Activate/Deactivate interpolation. Off by default.
 
         :Args:
 
-            x : boolean
+            x: boolean
                 True activates the interpolation, False deactivates it.
 
         """
+        pyoArgsAssert(self, "b", x)
         x, lmax = convertArgsToLists(x)
         [obj.setInterpolation(wrap(x,i)) for i, obj in enumerate(self._base_objs)]
 
+    def ctrl(self, map_list=None, title=None, wxnoserver=False):
+        self._map_list = [SLMap(0, 127, 'lin', 'ctlnumber', self._ctlnumber, res="int", dataOnly=True),
+                         SLMap(-1.0, 0.0, 'lin', 'minscale', self._minscale, dataOnly=True),
+                         SLMap(0.0, 1.0, 'lin', 'maxscale', self._maxscale, dataOnly=True),
+                         SLMap(0, 16, 'lin', 'channel', self._channel, res="int", dataOnly=True)
+                         ]
+        PyoObject.ctrl(self, map_list, title, wxnoserver)
+
     @property
     def ctlnumber(self):
         """int. Controller number."""
@@ -208,14 +224,14 @@ class CtlScan(PyoObject):
 
     :Args:
 
-        function : Python function (can't be a list)
+        function: Python function (can't be a list)
             Function to be called. The function must be declared
             with an argument for the controller number in input. Ex.:
 
             def ctl_scan(ctlnum):
                 print ctlnum
 
-        toprint : boolean, optional
+        toprint: boolean, optional
             If True, controller number and value will be print to
             the console.
 
@@ -234,10 +250,8 @@ class CtlScan(PyoObject):
 
     """
     def __init__(self, function, toprint=True):
+        pyoArgsAssert(self, "CB", function, toprint)
         PyoObject.__init__(self)
-        if not callable(function):
-            print >> sys.stderr, 'TypeError: "function" argument of %s must be callable.\n' % self.__class__.__name__
-            exit()
         self._function = WeakMethod(function)
         self._toprint = toprint
         self._base_objs = [CtlScan_base(self._function, self._toprint)]
@@ -270,10 +284,11 @@ class CtlScan(PyoObject):
 
         :Args:
 
-            x : Python function
+            x: Python function
                 new `function` attribute.
 
         """
+        pyoArgsAssert(self, "C", x)
         self._function = WeakMethod(x)
         [obj.setFunction(self._function) for i, obj in enumerate(self._base_objs)]
 
@@ -283,10 +298,11 @@ class CtlScan(PyoObject):
 
         :Args:
 
-            x : int
+            x: int
                 new `toprint` attribute.
 
         """
+        pyoArgsAssert(self, "b", x)
         self._toprint = x
         x, lmax = convertArgsToLists(x)
         [obj.setToprint(wrap(x,i)) for i, obj in enumerate(self._base_objs)]
@@ -318,7 +334,7 @@ class CtlScan2(PyoObject):
 
     :Args:
 
-        function : Python function (can't be a list)
+        function: Python function (can't be a list)
             Function to be called. The function must be declared
             with two arguments, one for the controller number and
             one for the midi channel. Ex.:
@@ -326,7 +342,7 @@ class CtlScan2(PyoObject):
             def ctl_scan(ctlnum, midichnl):
                 print ctlnum, midichnl
 
-        toprint : boolean, optional
+        toprint: boolean, optional
             If True, controller number and value will be print to
             the console.
 
@@ -345,10 +361,8 @@ class CtlScan2(PyoObject):
 
     """
     def __init__(self, function, toprint=True):
+        pyoArgsAssert(self, "CB", function, toprint)
         PyoObject.__init__(self)
-        if not callable(function):
-            print >> sys.stderr, 'TypeError: "function" argument of %s must be callable.\n' % self.__class__.__name__
-            exit()
         self._function = WeakMethod(function)
         self._toprint = toprint
         self._base_objs = [CtlScan2_base(self._function, self._toprint)]
@@ -381,10 +395,11 @@ class CtlScan2(PyoObject):
 
         :Args:
 
-            x : Python function
+            x: Python function
                 new `function` attribute.
 
         """
+        pyoArgsAssert(self, "C", x)
         self._function = WeakMethod(x)
         [obj.setFunction(self._function) for i, obj in enumerate(self._base_objs)]
 
@@ -394,10 +409,11 @@ class CtlScan2(PyoObject):
 
         :Args:
 
-            x : int
+            x: int
                 new `toprint` attribute.
 
         """
+        pyoArgsAssert(self, "b", x)
         self._toprint = x
         x, lmax = convertArgsToLists(x)
         [obj.setToprint(wrap(x,i)) for i, obj in enumerate(self._base_objs)]
@@ -430,9 +446,9 @@ class Notein(PyoObject):
 
     :Args:
 
-        poly : int, optional
+        poly: int, optional
             Number of streams of polyphony generated. Defaults to 10.
-        scale : int, optional
+        scale: int, optional
             Pitch output format.
                 0. Midi
                 1. Hertz
@@ -442,11 +458,11 @@ class Notein(PyoObject):
             there is no transposition) is (`first` + `last`) / 2.
 
             The central key can be changed with the setCentralKey method.
-        first : int, optional
+        first: int, optional
             Lowest Midi value. Defaults to 0.
-        last : int, optional
+        last: int, optional
             Highest Midi value. Defaults to 127.
-        channel : int, optional
+        channel: int, optional
             Midi channel. 0 means all channels. Defaults to 0.
 
     .. note::
@@ -478,6 +494,7 @@ class Notein(PyoObject):
 
     """
     def __init__(self, poly=10, scale=0, first=0, last=127, channel=0, mul=1, add=0):
+        pyoArgsAssert(self, "IIIIIOO", poly, scale, first, last, channel, mul, add)
         PyoObject.__init__(self, mul, add)
         self._pitch_dummy = []
         self._velocity_dummy = []
@@ -512,16 +529,59 @@ class Notein(PyoObject):
             self._trigoff_dummy.append(Dummy([self._trig_objs[i*2+1] for i in range(self._poly)]))
             return self._trigoff_dummy[-1]
 
+    def setScale(self, x):
+        """
+        Replace the `scale` attribute.
+
+        :Args:
+
+            x: int
+                new `scale` attribute. 0 = midi, 1 = hertz, 2 = transpo.
+
+        """
+        pyoArgsAssert(self, "I", x)
+        self._scale = x
+        self._base_handler.setScale(x)
+
+    def setFirst(self, x):
+        """
+        Replace the `first` attribute.
+
+        :Args:
+
+            x: int
+                new `first` attribute, between 0 and 127.
+
+        """
+        pyoArgsAssert(self, "I", x)
+        self._first = x
+        self._base_handler.setFirst(x)
+
+    def setLast(self, x):
+        """
+        Replace the `last` attribute.
+
+        :Args:
+
+            x: int
+                new `last` attribute, between 0 and 127.
+
+        """
+        pyoArgsAssert(self, "I", x)
+        self._last = x
+        self._base_handler.setLast(x)
+
     def setChannel(self, x):
         """
         Replace the `channel` attribute.
 
         :Args:
 
-            x : int
+            x: int
                 new `channel` attribute.
 
         """
+        pyoArgsAssert(self, "I", x)
         self._channel = x
         self._base_handler.setChannel(x)
 
@@ -534,10 +594,11 @@ class Notein(PyoObject):
 
         :Args:
 
-            x : int
+            x: int
                 new centralkey value.
 
         """
+        pyoArgsAssert(self, "I", x)
         self._base_handler.setCentralKey(x)
 
     def setStealing(self, x):
@@ -550,10 +611,11 @@ class Notein(PyoObject):
 
         :Args:
 
-            x : boolean
+            x: boolean
                 True for stealing mode, False for non-stealing.
 
         """
+        pyoArgsAssert(self, "B", x)
         self._base_handler.setStealing(x)
 
     def get(self, identifier="pitch", all=False):
@@ -567,10 +629,10 @@ class Notein(PyoObject):
 
         :Args:
 
-            identifier : string {"pitch", "velocity"}
+            identifier: string {"pitch", "velocity"}
                 Address string parameter identifying audio stream.
                 Defaults to "pitch".
-            all : boolean, optional
+            all: boolean, optional
                 If True, the first value of each object's stream
                 will be returned as a list.
 
@@ -594,6 +656,35 @@ class Notein(PyoObject):
         self._base_handler.stop()
         return PyoObject.stop(self)
 
+    def ctrl(self, map_list=None, title=None, wxnoserver=False):
+        self._map_list = [SLMap(0, 2, 'lin', 'scale', self._scale, res="int", dataOnly=True),
+                         SLMap(0, 127, 'lin', 'first', self._first, res="int", dataOnly=True),
+                         SLMap(0, 127, 'lin', 'last', self._last, res="int", dataOnly=True),
+                         SLMap(0, 16, 'lin', 'channel', self._channel, res="int", dataOnly=True)
+                         ]
+        PyoObject.ctrl(self, map_list, title, wxnoserver)
+
+    @property
+    def scale(self):
+        """int. Output format. 0 = midi, 1 = hertz, 2 = transpo."""
+        return self._scale
+    @scale.setter
+    def scale(self, x): self.setScale(x)
+
+    @property
+    def first(self):
+        """int. Lowest midi value."""
+        return self._first
+    @first.setter
+    def first(self, x): self.setFirst(x)
+
+    @property
+    def last(self):
+        """int. Highest midi value."""
+        return self._last
+    @last.setter
+    def last(self, x): self.setLast(x)
+
     @property
     def channel(self):
         """int. Midi channel. 0 means all channels."""
@@ -613,17 +704,17 @@ class Bendin(PyoObject):
 
     :Args:
 
-        brange : float, optional
+        brange: float, optional
             Bipolar range of the pitch bend in semitones. Defaults to 2.
             -brange <= value < brange.
-        scale : int, optional
+        scale: int, optional
             Output format. Defaults to 0.
                 0. Midi
                 1. transpo.
 
             The transpo mode is useful if you want to transpose values that
             are in a frequency (Hz) format.
-        channel : int, optional
+        channel: int, optional
             Midi channel. 0 means all channels. Defaults to 0.
 
     .. note::
@@ -642,6 +733,7 @@ class Bendin(PyoObject):
 
     """
     def __init__(self, brange=2, scale=0, channel=0, mul=1, add=0):
+        pyoArgsAssert(self, "niiOO", brange, scale, channel, mul, add)
         PyoObject.__init__(self, mul, add)
         self._brange = brange
         self._scale = scale
@@ -658,10 +750,11 @@ class Bendin(PyoObject):
 
         :Args:
 
-            x : int
+            x: float
                 new `brange` attribute.
 
         """
+        pyoArgsAssert(self, "n", x)
         self._brange = x
         x, lmax = convertArgsToLists(x)
         [obj.setBrange(wrap(x,i)) for i, obj in enumerate(self._base_objs)]
@@ -672,10 +765,11 @@ class Bendin(PyoObject):
 
         :Args:
 
-            x : int
+            x: int
                 new `scale` attribute.
 
         """
+        pyoArgsAssert(self, "i", x)
         self._scale = x
         x, lmax = convertArgsToLists(x)
         [obj.setScale(wrap(x,i)) for i, obj in enumerate(self._base_objs)]
@@ -686,14 +780,36 @@ class Bendin(PyoObject):
 
         :Args:
 
-            x : int
+            x: int
                 new `channel` attribute.
 
         """
+        pyoArgsAssert(self, "i", x)
         self._channel = x
         x, lmax = convertArgsToLists(x)
         [obj.setChannel(wrap(x,i)) for i, obj in enumerate(self._base_objs)]
 
+    def setInterpolation(self, x):
+        """
+        Activate/Deactivate interpolation. Off by default.
+
+        :Args:
+
+            x: boolean
+                True activates the interpolation, False deactivates it.
+
+        """
+        pyoArgsAssert(self, "b", x)
+        x, lmax = convertArgsToLists(x)
+        [obj.setInterpolation(wrap(x,i)) for i, obj in enumerate(self._base_objs)]
+
+    def ctrl(self, map_list=None, title=None, wxnoserver=False):
+        self._map_list = [SLMap(0.0, 12.0, 'lin', 'brange', self._brange, dataOnly=True),
+                         SLMap(0, 1, 'lin', 'scale', self._scale, res="int", dataOnly=True),
+                         SLMap(0, 16, 'lin', 'channel', self._channel, res="int", dataOnly=True)
+                         ]
+        PyoObject.ctrl(self, map_list, title, wxnoserver)
+
     @property
     def brange(self):
         """float. Bipolar range of the pitch bend in semitones."""
@@ -726,13 +842,13 @@ class Touchin(PyoObject):
 
     :Args:
 
-        minscale : float, optional
+        minscale: float, optional
             Low range value for mapping. Defaults to 0.
-        maxscale : float, optional
+        maxscale: float, optional
             High range value for mapping. Defaults to 1.
-        init : float, optional
+        init: float, optional
             Initial value. Defaults to 0.
-        channel : int, optional
+        channel: int, optional
             Midi channel. 0 means all channels. Defaults to 0.
 
     .. note::
@@ -751,6 +867,7 @@ class Touchin(PyoObject):
 
     """
     def __init__(self, minscale=0, maxscale=1, init=0, channel=0, mul=1, add=0):
+        pyoArgsAssert(self, "nnniOO", minscale, maxscale, init, channel, mul, add)
         PyoObject.__init__(self, mul, add)
         self._minscale = minscale
         self._maxscale = maxscale
@@ -767,10 +884,11 @@ class Touchin(PyoObject):
 
         :Args:
 
-            x : int
+            x: float
                 new `minscale` attribute.
 
         """
+        pyoArgsAssert(self, "n", x)
         self._minscale = x
         x, lmax = convertArgsToLists(x)
         [obj.setMinScale(wrap(x,i)) for i, obj in enumerate(self._base_objs)]
@@ -781,10 +899,11 @@ class Touchin(PyoObject):
 
         :Args:
 
-            x : int
+            x: float
                 new `maxscale` attribute.
 
         """
+        pyoArgsAssert(self, "n", x)
         self._maxscale = x
         x, lmax = convertArgsToLists(x)
         [obj.setMaxScale(wrap(x,i)) for i, obj in enumerate(self._base_objs)]
@@ -795,14 +914,36 @@ class Touchin(PyoObject):
 
         :Args:
 
-            x : int
+            x: int
                 new `channel` attribute.
 
         """
+        pyoArgsAssert(self, "i", x)
         self._channel = x
         x, lmax = convertArgsToLists(x)
         [obj.setChannel(wrap(x,i)) for i, obj in enumerate(self._base_objs)]
 
+    def setInterpolation(self, x):
+        """
+        Activate/Deactivate interpolation. Off by default.
+
+        :Args:
+
+            x: boolean
+                True activates the interpolation, False deactivates it.
+
+        """
+        pyoArgsAssert(self, "b", x)
+        x, lmax = convertArgsToLists(x)
+        [obj.setInterpolation(wrap(x,i)) for i, obj in enumerate(self._base_objs)]
+
+    def ctrl(self, map_list=None, title=None, wxnoserver=False):
+        self._map_list = [SLMap(-1.0, 0.0, 'lin', 'minscale', self._minscale, dataOnly=True),
+                         SLMap(0.0, 1.0, 'lin', 'maxscale', self._maxscale, dataOnly=True),
+                         SLMap(0, 16, 'lin', 'channel', self._channel, res="int", dataOnly=True)
+                         ]
+        PyoObject.ctrl(self, map_list, title, wxnoserver)
+
     @property
     def minscale(self):
         """float. Minimum value for scaling."""
@@ -837,7 +978,7 @@ class Programin(PyoObject):
 
     :Args:
 
-        channel : int, optional
+        channel: int, optional
             Midi channel. 0 means all channels. Defaults to 0.
 
     .. note::
@@ -856,6 +997,7 @@ class Programin(PyoObject):
 
     """
     def __init__(self, channel=0, mul=1, add=0):
+        pyoArgsAssert(self, "iOO", channel, mul, add)
         PyoObject.__init__(self, mul, add)
         self._channel = channel
         channel, mul, add, lmax = convertArgsToLists(channel, mul, add)
@@ -870,14 +1012,19 @@ class Programin(PyoObject):
 
         :Args:
 
-            x : int
+            x: int
                 new `channel` attribute.
 
         """
+        pyoArgsAssert(self, "i", x)
         self._channel = x
         x, lmax = convertArgsToLists(x)
         [obj.setChannel(wrap(x,i)) for i, obj in enumerate(self._base_objs)]
 
+    def ctrl(self, map_list=None, title=None, wxnoserver=False):
+        self._map_list = [SLMap(0, 16, 'lin', 'channel', self._channel, res="int", dataOnly=True)]
+        PyoObject.ctrl(self, map_list, title, wxnoserver)
+
     @property
     def channel(self):
         """int. Midi channel. 0 means all channels."""
@@ -901,24 +1048,27 @@ class MidiAdsr(PyoObject):
 
     :Args:
 
-        input : PyoObject
+        input: PyoObject
             Input signal used to trigger the envelope. A positive value
             sets the peak amplitude and starts the envelope. A 0 starts
             the release part of the envelope.
-        attack : float, optional
+        attack: float, optional
             Duration of the attack phase in seconds. Defaults to 0.01.
-        decay : float, optional
+        decay: float, optional
             Duration of the decay phase in seconds. Defaults to 0.05.
-        sustain : float, optional
+        sustain: float, optional
             Amplitude of the sustain phase, as a fraction of the peak
             amplitude at the start of the envelope. Defaults to 0.7.
-        release : float, optional
+        release: float, optional
             Duration of the release phase in seconds. Defaults to 0.1.
 
     .. note::
 
         The out() method is bypassed. MidiAdsr's signal can not be sent to audio outs.
 
+        As of version 0.8.0, exponential or logarithmic envelopes can be created
+        with the exponent factor (see setExp() method).
+
     >>> s = Server().boot()
     >>> s.start()
     >>> mid = Notein(scale=1)
@@ -928,12 +1078,14 @@ class MidiAdsr(PyoObject):
 
     """
     def __init__(self, input, attack=0.01, decay=0.05, sustain=0.7, release=0.1, mul=1, add=0):
+        pyoArgsAssert(self, "onnnnOO", input, attack, decay, sustain, release, mul, add)
         PyoObject.__init__(self, mul, add)
         self._input = input
         self._attack = attack
         self._decay = decay
         self._sustain = sustain
         self._release = release
+        self._exp = 1.0;
         self._in_fader = InputFader(input)
         in_fader, attack, decay, sustain, release, mul, add, lmax = convertArgsToLists(self._in_fader, attack, decay, sustain, release, mul, add)
         self._base_objs = [MidiAdsr_base(wrap(in_fader,i), wrap(attack,i), wrap(decay,i), wrap(sustain,i), wrap(release,i), wrap(mul,i), wrap(add,i)) for i in range(lmax)]
@@ -947,12 +1099,13 @@ class MidiAdsr(PyoObject):
 
         :Args:
 
-            x : PyoObject
+            x: PyoObject
                 New signal used to trigger the envelope.
-            fadetime : float, optional
+            fadetime: float, optional
                 Crossfade time between old and new input. Defaults to 0.05.
 
         """
+        pyoArgsAssert(self, "oN", x, fadetime)
         self._input = x
         self._in_fader.setInput(x, fadetime)
 
@@ -962,10 +1115,11 @@ class MidiAdsr(PyoObject):
 
         :Args:
 
-            x : float
+            x: float
                 new `attack` attribute.
 
         """
+        pyoArgsAssert(self, "n", x)
         self._attack = x
         x, lmax = convertArgsToLists(x)
         [obj.setAttack(wrap(x,i)) for i, obj in enumerate(self._base_objs)]
@@ -976,10 +1130,11 @@ class MidiAdsr(PyoObject):
 
         :Args:
 
-            x : float
+            x: float
                 new `decay` attribute.
 
         """
+        pyoArgsAssert(self, "n", x)
         self._decay = x
         x, lmax = convertArgsToLists(x)
         [obj.setDecay(wrap(x,i)) for i, obj in enumerate(self._base_objs)]
@@ -990,10 +1145,11 @@ class MidiAdsr(PyoObject):
 
         :Args:
 
-            x : float
+            x: float
                 new `sustain` attribute.
 
         """
+        pyoArgsAssert(self, "n", x)
         self._sustain = x
         x, lmax = convertArgsToLists(x)
         [obj.setSustain(wrap(x,i)) for i, obj in enumerate(self._base_objs)]
@@ -1004,14 +1160,43 @@ class MidiAdsr(PyoObject):
 
         :Args:
 
-            x : float
+            x: float
                 new `sustain` attribute.
 
         """
+        pyoArgsAssert(self, "n", x)
         self._release = x
         x, lmax = convertArgsToLists(x)
         [obj.setRelease(wrap(x,i)) for i, obj in enumerate(self._base_objs)]
 
+    def setExp(self, x):
+        """
+        Sets an exponent factor to create exponential or logarithmic envelopes.
+
+        The default value is 1.0, which means linear segments. A value
+        higher than 1.0 will produce exponential segments while a value
+        between 0 and 1 will produce logarithmic segments. Must be > 0.0.
+
+        :Args:
+
+            x: float
+                new `exp` attribute.
+
+        """
+        pyoArgsAssert(self, "n", x)
+        self._exp = x
+        x, lmax = convertArgsToLists(x)
+        [obj.setExp(wrap(x,i)) for i, obj in enumerate(self._base_objs)]
+
+    def ctrl(self, map_list=None, title=None, wxnoserver=False):
+        self._map_list = [SLMap(0.0, 1.0, 'lin', 'attack', self._attack, dataOnly=True),
+                          SLMap(0.0, 1.0, 'lin', 'decay', self._decay, dataOnly=True),
+                          SLMap(0.0, 1.0, 'lin', 'sustain', self._sustain, dataOnly=True),
+                          SLMap(0.0, 1.0, 'lin', 'release', self._release, dataOnly=True),
+                          SLMap(0.1, 10.0, 'log', 'exp', self._exp, dataOnly=True),
+                         ]
+        PyoObject.ctrl(self, map_list, title, wxnoserver)
+
     @property
     def attack(self):
         """float. Duration of the attack phase in seconds."""
@@ -1040,6 +1225,13 @@ class MidiAdsr(PyoObject):
     @release.setter
     def release(self, x): self.setRelease(x)
 
+    @property
+    def exp(self):
+        """float. Exponent factor of the envelope."""
+        return self._exp
+    @exp.setter
+    def exp(self, x): self.setExp(x)
+
 class MidiDelAdsr(PyoObject):
     """
     Midi triggered ADSR envelope generator with pre-delay.
@@ -1055,21 +1247,21 @@ class MidiDelAdsr(PyoObject):
 
     :Args:
 
-        input : PyoObject
+        input: PyoObject
             Input signal used to trigger the envelope. A positive value
             sets the peak amplitude and starts the envelope. A 0 starts
             the release part of the envelope.
-        delay : float, optional
+        delay: float, optional
             Duration of the delay phase, before calling the envelope
             in seconds. Defaults to 0.
-        attack : float, optional
+        attack: float, optional
             Duration of the attack phase in seconds. Defaults to 0.01.
-        decay : float, optional
+        decay: float, optional
             Duration of the decay phase in seconds. Defaults to 0.05.
-        sustain : float, optional
+        sustain: float, optional
             Amplitude of the sustain phase, as a fraction of the peak
             amplitude at the start of the envelope. Defaults to 0.7.
-        release : float, optional
+        release: float, optional
             Duration of the release phase in seconds. Defaults to 0.1.
 
     .. note::
@@ -1077,6 +1269,9 @@ class MidiDelAdsr(PyoObject):
         The out() method is bypassed. MidiDelAdsr's signal can not be sent
         to audio outs.
 
+        As of version 0.8.0, exponential or logarithmic envelopes can be created
+        with the exponent factor (see setExp() method).
+
     >>> s = Server().boot()
     >>> s.start()
     >>> mid = Notein(scale=1)
@@ -1086,6 +1281,7 @@ class MidiDelAdsr(PyoObject):
 
     """
     def __init__(self, input, delay=0, attack=0.01, decay=0.05, sustain=0.7, release=0.1, mul=1, add=0):
+        pyoArgsAssert(self, "onnnnnOO", input, delay, attack, decay, sustain, release, mul, add)
         PyoObject.__init__(self, mul, add)
         self._input = input
         self._delay = delay
@@ -1093,6 +1289,7 @@ class MidiDelAdsr(PyoObject):
         self._decay = decay
         self._sustain = sustain
         self._release = release
+        self._exp = 1.0;
         self._in_fader = InputFader(input)
         in_fader, delay, attack, decay, sustain, release, mul, add, lmax = convertArgsToLists(self._in_fader, delay, attack, decay, sustain, release, mul, add)
         self._base_objs = [MidiDelAdsr_base(wrap(in_fader,i), wrap(delay,i), wrap(attack,i), wrap(decay,i), wrap(sustain,i), wrap(release,i), wrap(mul,i), wrap(add,i)) for i in range(lmax)]
@@ -1106,12 +1303,13 @@ class MidiDelAdsr(PyoObject):
 
         :Args:
 
-            x : PyoObject
+            x: PyoObject
                 New signal used to trigger the envelope.
-            fadetime : float, optional
+            fadetime: float, optional
                 Crossfade time between old and new input. Defaults to 0.05.
 
         """
+        pyoArgsAssert(self, "oN", x, fadetime)
         self._input = x
         self._in_fader.setInput(x, fadetime)
 
@@ -1121,10 +1319,11 @@ class MidiDelAdsr(PyoObject):
 
         :Args:
 
-            x : float
+            x: float
                 new `delay` attribute.
 
         """
+        pyoArgsAssert(self, "n", x)
         self._delay = x
         x, lmax = convertArgsToLists(x)
         [obj.setDelay(wrap(x,i)) for i, obj in enumerate(self._base_objs)]
@@ -1135,10 +1334,11 @@ class MidiDelAdsr(PyoObject):
 
         :Args:
 
-            x : float
+            x: float
                 new `attack` attribute.
 
         """
+        pyoArgsAssert(self, "n", x)
         self._attack = x
         x, lmax = convertArgsToLists(x)
         [obj.setAttack(wrap(x,i)) for i, obj in enumerate(self._base_objs)]
@@ -1149,10 +1349,11 @@ class MidiDelAdsr(PyoObject):
 
         :Args:
 
-            x : float
+            x: float
                 new `decay` attribute.
 
         """
+        pyoArgsAssert(self, "n", x)
         self._decay = x
         x, lmax = convertArgsToLists(x)
         [obj.setDecay(wrap(x,i)) for i, obj in enumerate(self._base_objs)]
@@ -1163,10 +1364,11 @@ class MidiDelAdsr(PyoObject):
 
         :Args:
 
-            x : float
+            x: float
                 new `sustain` attribute.
 
         """
+        pyoArgsAssert(self, "n", x)
         self._sustain = x
         x, lmax = convertArgsToLists(x)
         [obj.setSustain(wrap(x,i)) for i, obj in enumerate(self._base_objs)]
@@ -1177,14 +1379,44 @@ class MidiDelAdsr(PyoObject):
 
         :Args:
 
-            x : float
+            x: float
                 new `sustain` attribute.
 
         """
+        pyoArgsAssert(self, "n", x)
         self._release = x
         x, lmax = convertArgsToLists(x)
         [obj.setRelease(wrap(x,i)) for i, obj in enumerate(self._base_objs)]
 
+    def setExp(self, x):
+        """
+        Sets an exponent factor to create exponential or logarithmic envelope.
+
+        The default value is 1.0, which means linear segments. A value
+        higher than 1.0 will produce exponential segments while a value
+        between 0 and 1 will produce logarithmic segments. Must be > 0.0.
+
+        :Args:
+
+            x: float
+                new `exp` attribute.
+
+        """
+        pyoArgsAssert(self, "n", x)
+        self._exp = x
+        x, lmax = convertArgsToLists(x)
+        [obj.setExp(wrap(x,i)) for i, obj in enumerate(self._base_objs)]
+
+    def ctrl(self, map_list=None, title=None, wxnoserver=False):
+        self._map_list = [SLMap(0.0, 1.0, 'lin', 'delay', self._delay, dataOnly=True),
+                          SLMap(0.0, 1.0, 'lin', 'attack', self._attack, dataOnly=True),
+                          SLMap(0.0, 1.0, 'lin', 'decay', self._decay, dataOnly=True),
+                          SLMap(0.0, 1.0, 'lin', 'sustain', self._sustain, dataOnly=True),
+                          SLMap(0.0, 1.0, 'lin', 'release', self._release, dataOnly=True),
+                          SLMap(0.1, 10.0, 'log', 'exp', self._exp, dataOnly=True),
+                         ]
+        PyoObject.ctrl(self, map_list, title, wxnoserver)
+
     @property
     def delay(self):
         """float. Duration of the delay phase in seconds."""
@@ -1218,4 +1450,87 @@ class MidiDelAdsr(PyoObject):
         """float. Duration of the release phase in seconds."""
         return self._release
     @release.setter
-    def release(self, x): self.setRelease(x)
\ No newline at end of file
+    def release(self, x): self.setRelease(x)
+
+    @property
+    def exp(self):
+        """float. Exponent factor of the envelope."""
+        return self._exp
+    @exp.setter
+    def exp(self, x): self.setExp(x)
+
+class RawMidi(PyoObject):
+    """
+    Raw Midi handler.
+
+    This object calls a python function for each raw midi data
+    (status, data1, data2) event for further processing in Python.
+
+    :Parent: :py:class:`PyoObject`
+
+    :Args:
+
+        function: Python function (can't be a list)
+            Function to be called. The function must be declared
+            with three arguments, one for the status byte and two
+            for the data bytes. Ex.:
+
+            def event(status, data1, data2):
+                print status, data1, data2
+
+    .. note::
+
+        The out() method is bypassed. RawMidi's signal can not be sent
+        to audio outs.
+
+    >>> s = Server()
+    >>> s.setMidiInputDevice(99) # opens all devices
+    >>> s.boot()
+    >>> s.start()
+    >>> def event(status, data1, data2):
+    ...     print status, data1, data2
+    >>> a = RawMidi(event)
+
+    """
+    def __init__(self, function):
+        pyoArgsAssert(self, "C", function)
+        PyoObject.__init__(self)
+        self._function = WeakMethod(function)
+        self._base_objs = [RawMidi_base(self._function)]
+
+    def out(self, chnl=0, inc=1, dur=0, delay=0):
+        return self.play(dur, delay)
+
+    def setMul(self, x):
+        pass
+
+    def setAdd(self, x):
+        pass
+
+    def setSub(self, x):
+        pass
+
+    def setDiv(self, x):
+        pass
+
+    def setFunction(self, x):
+        """
+        Replace the `function` attribute.
+
+        :Args:
+
+            x: Python function
+                new `function` attribute.
+
+        """
+        pyoArgsAssert(self, "C", x)
+        self._function = WeakMethod(x)
+        [obj.setFunction(self._function) for i, obj in enumerate(self._base_objs)]
+
+    @property
+    def function(self):
+        """Python function. Function to be called."""
+        return self._function
+    @function.setter
+    def function(self, x):
+        self.setFunction(x)
