@@ -1,3 +1,5 @@
+from __future__ import print_function
+from __future__ import absolute_import
 """
 Objects to manage values on an Open Sound Control port.
 
@@ -10,6 +12,9 @@ input port.
 The audio streams of these objects are essentially intended to be
 controls and can't be sent to the output soundcard.
 
+These objects are available only if pyo is built with OSC (Open Sound
+Control) support.
+
 """
 
 """
@@ -31,9 +36,17 @@ GNU Lesser General Public License for more details.
 You should have received a copy of the GNU Lesser General Public
 License along with pyo.  If not, see <http://www.gnu.org/licenses/>.
 """
-from _core import *
-from _maps import *
-from types import ListType
+from ._core import *
+from ._maps import *
+
+def assertOSCSupport(obj):
+    if not withOSC():
+        raise Exception("Pyo built without OSC support! '%s' objects is not available." % obj.__class__.__name__)
+
+### TODO - Know bugs:
+### OscListReceive.setValue(address, value) make the program segfault on quit (python 2.7 and 3.5).
+### OscSend/OscReceive don't work with unicode on python 2.7 (ok on python 3.5)
+###
 
 ######################################################################
 ### Open Sound Control
@@ -50,15 +63,15 @@ class OscSend(PyoObject):
 
     :Args:
 
-        input : PyoObject
+        input: PyoObject
             Input signal.
-        port : int
+        port: int
             Port on which values are sent. Receiver should listen on the
             same port.
-        address : string
+        address: string
             Address used on the port to identify values. Address is in
             the form of a Unix path (ex.: '/pitch').
-        host : string, optional
+        host: string, optional
             IP address of the target computer. The default, '127.0.0.1',
             is the localhost.
 
@@ -76,6 +89,7 @@ class OscSend(PyoObject):
 
     """
     def __init__(self, input, port, address, host="127.0.0.1"):
+        assertOSCSupport(self)
         pyoArgsAssert(self, "oiss", input, port, address, host)
         PyoObject.__init__(self)
         self._input = input
@@ -89,9 +103,9 @@ class OscSend(PyoObject):
 
         :Args:
 
-            x : PyoObject
+            x: PyoObject
                 New signal to process.
-            fadetime : float, optional
+            fadetime: float, optional
                 Crossfade time between old and new input. Defaults to 0.05.
 
         """
@@ -114,7 +128,7 @@ class OscSend(PyoObject):
 
         :Args:
 
-            x : int
+            x: int
                 Changes the data output frequency in multiples of the buffer size.
                 Should be greater or equal to 1.
 
@@ -141,7 +155,7 @@ class OscReceive(PyoObject):
 
     :Args:
 
-        port : int
+        port: int
             Port on which values are received. Sender should output on
             the same port.
 
@@ -149,7 +163,7 @@ class OscReceive(PyoObject):
             object.
 
             Available at initialization time only.
-        address : string
+        address: string
             Address used on the port to identify values. Address is in
             the form of a Unix path (ex.: '/pitch').
 
@@ -171,6 +185,7 @@ class OscReceive(PyoObject):
     """
 
     def __init__(self, port, address, mul=1, add=0):
+        assertOSCSupport(self)
         pyoArgsAssert(self, "IsOO", port, address, mul, add)
         PyoObject.__init__(self, mul, add)
         address, mul, add, lmax = convertArgsToLists(address, mul, add)
@@ -179,12 +194,12 @@ class OscReceive(PyoObject):
         self._base_objs = [OscReceive_base(self._mainReceiver, wrap(address,i), wrap(mul,i), wrap(add,i)) for i in range(lmax)]
 
     def __getitem__(self, i):
-        if type(i) == type(''):
+        if type(i) in [bytes_t, unicode_t]:
             return self._base_objs[self._address.index(i)]
         elif i < len(self._base_objs):
             return self._base_objs[i]
         else:
-            print "'i' too large!"
+            print("'i' too large!")
 
     def getAddresses(self):
         """
@@ -199,11 +214,11 @@ class OscReceive(PyoObject):
 
         :Args:
 
-            path : string or list of strings
+            path: string or list of strings
                 New path(s) to receive from.
-            mul : float or PyoObject
+            mul: float or PyoObject
                 Multiplication factor. Defaults to 1.
-            add : float or PyoObject
+            add: float or PyoObject
                 Addition factor. Defaults to 0.
 
         """
@@ -222,7 +237,7 @@ class OscReceive(PyoObject):
 
         :Args:
 
-            path : string or list of strings
+            path: string or list of strings
                 Path(s) to remove.
 
         """
@@ -240,7 +255,7 @@ class OscReceive(PyoObject):
 
         :Args:
 
-            x : boolean
+            x: boolean
                 True activates the interpolation, False deactivates it.
 
         """
@@ -253,9 +268,9 @@ class OscReceive(PyoObject):
 
         :Args:
 
-            path : string
+            path: string
                 Address to which the value should be attributed.
-            value : float
+            value: float
                 Value to attribute to the given address.
 
         """
@@ -266,7 +281,7 @@ class OscReceive(PyoObject):
             if p in self._address:
                 self._mainReceiver.setValue(p, wrap(value,i))
             else:
-                print 'Error: OscReceive.setValue, Illegal address "%s"' % p
+                print('Error: OscReceive.setValue, Illegal address "%s"' % p)
 
     def get(self, identifier=None, all=False):
         """
@@ -279,11 +294,11 @@ class OscReceive(PyoObject):
 
         :Args:
 
-            identifier : string
+            identifier: string
                 Address string parameter identifying audio stream.
                 Defaults to None, useful when `all` is True to
                 retreive all streams values.
-            all : boolean, optional
+            all: boolean, optional
                 If True, the first value of each object's stream
                 will be returned as a list. Otherwise, only the value
                 of the first object's stream will be returned as a float.
@@ -310,30 +325,30 @@ class OscDataSend(PyoObject):
 
     :Args:
 
-        types : str
+        types: str
             String specifying the types sequence of the message to be sent.
             Possible values are:
-                - "i" : integer
-                - "h" : long integer
-                - "f" : float
-                - "d" : double
+                - "i": integer
+                - "h": long integer
+                - "f": float
+                - "d": double
                 - "s" ; string
-                - "b" : blob (list of chars)
-                - "m" : MIDI packet (list of 4 bytes: [midi port, status, data1, data2])
-                - "c" : char
-                - "T" : True
-                - "F" : False
-                - "N" : None (nil)
+                - "b": blob (list of chars)
+                - "m": MIDI packet (list of 4 bytes: [midi port, status, data1, data2])
+                - "c": char
+                - "T": True
+                - "F": False
+                - "N": None (nil)
 
             The string "ssfi" indicates that the value to send will be a list
             containing two strings followed by a float and an integer.
-        port : int
+        port: int
             Port on which values are sent. Receiver should listen on the
             same port.
-        address : string
+        address: string
             Address used on the port to identify values. Address is in
             the form of a Unix path (ex.: '/pitch').
-        host : string, optional
+        host: string, optional
             IP address of the target computer. The default, '127.0.0.1',
             is the localhost.
 
@@ -364,6 +379,7 @@ class OscDataSend(PyoObject):
 
     """
     def __init__(self, types, port, address, host="127.0.0.1"):
+        assertOSCSupport(self)
         pyoArgsAssert(self, "siss", types, port, address, host)
         PyoObject.__init__(self)
         types, port, address, host, lmax = convertArgsToLists(types, port, address, host)
@@ -386,7 +402,7 @@ class OscDataSend(PyoObject):
         Returns the addresses managed by the object.
 
         """
-        return self._addresses.keys()
+        return list(self._addresses.keys())
 
     def addAddress(self, types, port, address, host="127.0.0.1"):
         """
@@ -394,24 +410,30 @@ class OscDataSend(PyoObject):
 
         :Args:
 
-            types : str
+            types: str
                 String specifying the types sequence of the message to be sent.
                 Possible values are:
-                    - integer : "i"
-                    - long integer : "h"
-                    - float : "f"
-                    - double : "d"
-                    - string : "s"
+                - "i": integer
+                - "h": long integer
+                - "f": float
+                - "d": double
+                - "s" ; string
+                - "b": blob (list of chars)
+                - "m": MIDI packet (list of 4 bytes: [midi port, status, data1, data2])
+                - "c": char
+                - "T": True
+                - "F": False
+                - "N": None (nil)
 
                 The string "ssfi" indicates that the value to send will be a list
                 containing two strings followed by a float and an integer.
-            port : int
+            port: int
                 Port on which values are sent. Receiver should listen on the
                 same port.
-            address : string
+            address: string
                 Address used on the port to identify values. Address is in
                 the form of a Unix path (ex.: '/pitch').
-            host : string, optional
+            host: string, optional
                 IP address of the target computer. The default, '127.0.0.1',
                 is the localhost.
 
@@ -429,15 +451,16 @@ class OscDataSend(PyoObject):
 
         :Args:
 
-            path : string or list of strings
+            path: string or list of strings
                 Path(s) to remove.
 
         """
         pyoArgsAssert(self, "s", path)
         path, lmax = convertArgsToLists(path)
         for p in path:
-            self._base_objs.remove(self._addresses[p])
-            del self._addresses[p]
+            if p in self._addresses:
+                self._base_objs.remove(self._addresses[p])
+                del self._addresses[p]
 
     def send(self, msg, address=None):
         """
@@ -445,16 +468,16 @@ class OscDataSend(PyoObject):
 
         :Args:
 
-            msg : list
+            msg: list
                 List of values to send. Types of values in list
                 must be of the kind defined of `types` argument
                 given at the object's initialization.
-            address : string, optional
+            address: string, optional
                 Address destination to send values. If None, values
                 will be sent to all addresses managed by the object.
 
         """
-        if address == None:
+        if address is None:
             pyoArgsAssert(self, "l", msg)
             [obj.send(msg) for obj in self._base_objs]
         else:
@@ -474,16 +497,16 @@ class OscDataReceive(PyoObject):
 
     :Args:
 
-        port : int
+        port: int
             Port on which values are received. Sender should output on
             the same port. Unlike OscDataSend object, there can be only
             one port per OscDataReceive object. Available at initialization
             time only.
-        address : string
+        address: string
             Address used on the port to identify values. Address is in
             the form of a Unix path (ex.: "/pitch"). There can be as many
             addresses as needed on a single port.
-        function : callable (can't be a list)
+        function: callable (can't be a list)
             This function will be called whenever a message with a known
             address is received. there can be only one function per
             OscDataReceive object. Available at initialization time only.
@@ -521,6 +544,7 @@ class OscDataReceive(PyoObject):
     """
 
     def __init__(self, port, address, function):
+        assertOSCSupport(self)
         pyoArgsAssert(self, "IsC", port, address, function)
         PyoObject.__init__(self)
         self._port = port
@@ -551,7 +575,7 @@ class OscDataReceive(PyoObject):
 
         :Args:
 
-            path : string or list of strings
+            path: string or list of strings
                 New path(s) to receive from.
 
         """
@@ -559,6 +583,7 @@ class OscDataReceive(PyoObject):
         path, lmax = convertArgsToLists(path)
         for p in path:
             if p not in self._address:
+                self._address.append(p)
                 self._base_objs[0].addAddress(p)
 
     def delAddress(self, path):
@@ -567,15 +592,17 @@ class OscDataReceive(PyoObject):
 
         :Args:
 
-            path : string or list of strings
+            path: string or list of strings
                 Path(s) to remove.
 
         """
         pyoArgsAssert(self, "s", path)
         path, lmax = convertArgsToLists(path)
         for p in path:
-            index = self._address.index(p)
-            self._base_objs[0].delAddress(index)
+            if p in self._address:
+                index = self._address.index(p)
+                self._base_objs[0].delAddress(index)
+                self._address.remove(p)
 
 class OscListReceive(PyoObject):
     """
@@ -589,15 +616,15 @@ class OscListReceive(PyoObject):
 
     :Args:
 
-        port : int
+        port: int
             Port on which values are received. Sender should output on
             the same port. Unlike OscSend object, there can be only one
             port per OscListReceive object. Available at initialization time
             only.
-        address : string
+        address: string
             Address used on the port to identify values. Address is in
             the form of a Unix path (ex.: '/pitch').
-        num : int, optional
+        num: int, optional
             Length of the lists in input. The object will generate `num` audio
             streams per given address. Available at initialization time only.
             This value can't be a list. That means all addresses managed by an
@@ -622,6 +649,7 @@ class OscListReceive(PyoObject):
     """
 
     def __init__(self, port, address, num=8, mul=1, add=0):
+        assertOSCSupport(self)
         pyoArgsAssert(self, "IsIOO", port, address, num, mul, add)
         PyoObject.__init__(self, mul, add)
         self._num = num
@@ -632,14 +660,14 @@ class OscListReceive(PyoObject):
         self._base_objs = [OscListReceive_base(self._mainReceiver, wrap(address,i), j, wrap(mul,i), wrap(add,i)) for i in range(lmax) for j in range(self._num)]
 
     def __getitem__(self, i):
-        if type(i) == type(''):
+        if type(i) in [bytes_t, unicode_t]:
             first = self._address.index(i) * self._num
             return self._base_objs[first:first+self._num]
         elif i < len(self._base_objs):
             first = i * self._num
             return self._base_objs[first:first+self._num]
         else:
-            print "'i' too large!"
+            print("'i' too large!")
 
     def getAddresses(self):
         """
@@ -654,11 +682,11 @@ class OscListReceive(PyoObject):
 
         :Args:
 
-            path : string or list of strings
+            path: string or list of strings
                 New path(s) to receive from.
-            mul : float or PyoObject
+            mul: float or PyoObject
                 Multiplication factor. Defaults to 1.
-            add : float or PyoObject
+            add: float or PyoObject
                 Addition factor. Defaults to 0.
 
         """
@@ -677,18 +705,18 @@ class OscListReceive(PyoObject):
 
         :Args:
 
-            path : string or list of strings
+            path: string or list of strings
                 Path(s) to remove.
 
         """
         pyoArgsAssert(self, "s", path)
         path, lmax = convertArgsToLists(path)
         self._mainReceiver.delAddress(path)
-        indexes = [self._address.index(p) for p in path]
+        indexes = [self._address.index(p) for p in path if p in self._address]
         for ind in reversed(indexes):
             self._address.pop(ind)
             first = ind * self._num
-            for i in reversed(range(first, first+self._num)):
+            for i in reversed(list(range(first, first+self._num))):
                 obj = self._base_objs.pop(i)
 
     def setInterpolation(self, x):
@@ -697,7 +725,7 @@ class OscListReceive(PyoObject):
 
         :Args:
 
-            x : boolean
+            x: boolean
                 True activates the interpolation, False deactivates it.
 
         """
@@ -710,9 +738,9 @@ class OscListReceive(PyoObject):
 
         :Args:
 
-            path : string
+            path: string
                 Address to which the value should be attributed.
-            value : list of floats
+            value: list of floats
                 List of values to attribute to the given address.
 
         """
@@ -721,16 +749,16 @@ class OscListReceive(PyoObject):
         for i in range(lmax):
             p = wrap(path,i)
             if p in self._address:
-                if type(value[0]) == ListType:
+                if type(value[0]) == list:
                     val = wrap(value,i)
                 else:
                     val = value
                 if len(val) == self._num:
                     self._mainReceiver.setValue(p, val)
                 else:
-                    print 'Error: OscListReceive.setValue, value must be of the same length as the `num` attribute.'
+                    print('Error: OscListReceive.setValue, value must be of the same length as the `num` attribute.')
             else:
-                print 'Error: OscListReceive.setValue, Illegal address "%s"' % p
+                print('Error: OscListReceive.setValue, Illegal address "%s"' % p)
 
     def get(self, identifier=None, all=False):
         """
@@ -743,11 +771,11 @@ class OscListReceive(PyoObject):
 
         :Args:
 
-            identifier : string
+            identifier: string
                 Address string parameter identifying audio stream.
                 Defaults to None, useful when `all` is True to
                 retreive all streams values.
-            all : boolean, optional
+            all: boolean, optional
                 If True, the first list of values of each object's stream
                 will be returned as a list of lists. Otherwise, only the
                 the list of the object's identifier will be returned as a
@@ -766,4 +794,4 @@ class OscListReceive(PyoObject):
             return outlist
 
     def out(self, chnl=0, inc=1, dur=0, delay=0):
-        return self.play(dur, delay)
\ No newline at end of file
+        return self.play(dur, delay)
