@@ -1,3 +1,6 @@
+from __future__ import division
+from __future__ import print_function
+from __future__ import absolute_import
 """
 Set of objects to manage audio voice routing and spread of a sound
 signal into a new stereo or multi-channel sound field.
@@ -24,9 +27,8 @@ You should have received a copy of the GNU Lesser General Public
 License along with pyo.  If not, see <http://www.gnu.org/licenses/>.
 """
 import sys, random
-from _core import *
-from _maps import *
-from types import SliceType, IntType
+from ._core import *
+from ._maps import *
 
 class Pan(PyoObject):
     """
@@ -36,15 +38,15 @@ class Pan(PyoObject):
 
     :Args:
 
-        input : PyoObject
+        input: PyoObject
             Input signal to process.
-        outs : int, optional
+        outs: int, optional
             Number of channels on the panning circle. Available at
             initialization time only. Defaults to 2.
-        pan : float or PyoObject
+        pan: float or PyoObject
             Position of the sound on the panning circle, between 0 and 1.
             Defaults to 0.5.
-        spread : float or PyoObject
+        spread: float or PyoObject
             Amount of sound leaking to the surrounding channels,
             between 0 and 1. Defaults to 0.5.
 
@@ -56,6 +58,7 @@ class Pan(PyoObject):
 
     """
     def __init__(self, input, outs=2, pan=0.5, spread=0.5, mul=1, add=0):
+        pyoArgsAssert(self, "oIOOOO", input, outs, pan, spread, mul, add)
         PyoObject.__init__(self, mul, add)
         self._input = input
         self._pan = pan
@@ -75,12 +78,13 @@ class Pan(PyoObject):
 
         :Args:
 
-            x : PyoObject
+            x: PyoObject
                 New signal to process.
-            fadetime : float, optional
+            fadetime: float, optional
                 Crossfade time between old and new input. Default to 0.05.
 
         """
+        pyoArgsAssert(self, "oN", x, fadetime)
         self._input = x
         self._in_fader.setInput(x, fadetime)
 
@@ -90,10 +94,11 @@ class Pan(PyoObject):
 
         :Args:
 
-            x : float or PyoObject
+            x: float or PyoObject
                 new `pan` attribute.
 
         """
+        pyoArgsAssert(self, "O", x)
         self._pan = x
         x, lmax = convertArgsToLists(x)
         [obj.setPan(wrap(x,i)) for i, obj in enumerate(self._base_players)]
@@ -104,10 +109,11 @@ class Pan(PyoObject):
 
         :Args:
 
-            x : float or PyoObject
+            x: float or PyoObject
                 new `spread` attribute.
 
         """
+        pyoArgsAssert(self, "O", x)
         self._spread = x
         x, lmax = convertArgsToLists(x)
         [obj.setSpread(wrap(x,i)) for i, obj in enumerate(self._base_players)]
@@ -147,12 +153,12 @@ class SPan(PyoObject):
 
     :Args:
 
-        input : PyoObject
+        input: PyoObject
             Input signal to process.
-        outs : int, optional
+        outs: int, optional
             Number of channels on the panning circle. Available at
             initialization time only. Defaults to 2.
-        pan : float or PyoObject
+        pan: float or PyoObject
             Position of the sound on the panning circle, between 0 and 1.
             Defaults to 0.5.
 
@@ -164,6 +170,7 @@ class SPan(PyoObject):
 
     """
     def __init__(self, input, outs=2, pan=0.5, mul=1, add=0):
+        pyoArgsAssert(self, "oIOOO", input, outs, pan, mul, add)
         PyoObject.__init__(self, mul, add)
         self._input = input
         self._outs = outs
@@ -182,12 +189,13 @@ class SPan(PyoObject):
 
         :Args:
 
-            x : PyoObject
+            x: PyoObject
                 New signal to process.
-            fadetime : float, optional
+            fadetime: float, optional
                 Crossfade time between old and new input. Default to 0.05.
 
         """
+        pyoArgsAssert(self, "oN", x, fadetime)
         self._input = x
         self._in_fader.setInput(x, fadetime)
 
@@ -197,10 +205,11 @@ class SPan(PyoObject):
 
         :Args:
 
-            x : float or PyoObject
+            x: float or PyoObject
                 new `pan` attribute.
 
         """
+        pyoArgsAssert(self, "O", x)
         self._pan = x
         x, lmax = convertArgsToLists(x)
         [obj.setPan(wrap(x,i)) for i, obj in enumerate(self._base_players)]
@@ -237,12 +246,12 @@ class Switch(PyoObject):
 
     :Args:
 
-        input : PyoObject
+        input: PyoObject
             Input signal to process.
-        outs : int, optional
+        outs: int, optional
             Number of outputs. Available at initialization time only.
             Defaults to 2.
-        voice : float or PyoObject
+        voice: float or PyoObject
             Voice position pointer, between 0 and (outs-1) / len(input).
             Defaults to 0.
 
@@ -257,6 +266,7 @@ class Switch(PyoObject):
 
     """
     def __init__(self, input, outs=2, voice=0., mul=1, add=0):
+        pyoArgsAssert(self, "oIOOO", input, outs, voice, mul, add)
         PyoObject.__init__(self, mul, add)
         self._input = input
         self._outs = outs
@@ -275,12 +285,13 @@ class Switch(PyoObject):
 
         :Args:
 
-            x : PyoObject
+            x: PyoObject
                 New signal to process.
-            fadetime : float, optional
+            fadetime: float, optional
                 Crossfade time between old and new input. Default to 0.05.
 
         """
+        pyoArgsAssert(self, "oN", x, fadetime)
         self._input = x
         self._in_fader.setInput(x, fadetime)
 
@@ -290,10 +301,11 @@ class Switch(PyoObject):
 
         :Args:
 
-            x : float or PyoObject
+            x: float or PyoObject
                 new `voice` attribute.
 
         """
+        pyoArgsAssert(self, "O", x)
         self._voice = x
         x, lmax = convertArgsToLists(x)
         [obj.setVoice(wrap(x,i)) for i, obj in enumerate(self._base_players)]
@@ -327,9 +339,9 @@ class Selector(PyoObject):
 
     :Args:
 
-        inputs : list of PyoObject
+        inputs: list of PyoObject
             Audio objects to interpolate from.
-        voice : float or PyoObject, optional
+        voice: float or PyoObject, optional
             Voice position pointer, between 0 and len(inputs)-1.
             Defaults to 0.
 
@@ -343,6 +355,7 @@ class Selector(PyoObject):
 
     """
     def __init__(self, inputs, voice=0., mul=1, add=0):
+        pyoArgsAssert(self, "lOOO", inputs, voice, mul, add)
         PyoObject.__init__(self, mul, add)
         self._inputs = inputs
         self._voice = voice
@@ -370,10 +383,11 @@ class Selector(PyoObject):
 
         :Args:
 
-            x : list of PyoObject
+            x: list of PyoObject
                 new `inputs` attribute.
 
         """
+        pyoArgsAssert(self, "l", x)
         self._inputs = x
         for i in range(self._lmax):
             for j in range(self._length):
@@ -391,14 +405,15 @@ class Selector(PyoObject):
 
         :Args:
 
-            x : float or PyoObject
+            x: float or PyoObject
                 new `voice` attribute.
 
         """
+        pyoArgsAssert(self, "O", x)
         self._voice = x
         x, lmax = convertArgsToLists(x)
         for i, obj in enumerate(self._base_objs):
-            obj.setVoice(wrap(x, i/self._length))
+            obj.setVoice(wrap(x, i // self._length))
 
     def ctrl(self, map_list=None, title=None, wxnoserver=False):
         self._map_list = [SLMap(0, len(self._inputs)-1, "lin", "voice", self._voice), SLMapMul(self._mul)]
@@ -439,9 +454,9 @@ class VoiceManager(PyoObject):
 
     :Args:
 
-        input : PyoObject
+        input: PyoObject
             Trigger stream asking for new voice numbers.
-        triggers : PyoObject or list of PyoObject, optional
+        triggers: PyoObject or list of PyoObject, optional
             List of mono PyoObject sending triggers. Can be a multi-streams
             PyoObject but not a mix of both.
 
@@ -460,13 +475,14 @@ class VoiceManager(PyoObject):
 
     """
     def __init__(self, input, triggers=None, mul=1, add=0):
+        #pyoArgsAssert(self, "ooOO", input, triggers, mul, add)
         PyoObject.__init__(self, mul, add)
         self._input = input
         self._triggers = triggers
         self._in_fader = InputFader(input)
         in_fader, mul, add, lmax = convertArgsToLists(self._in_fader, mul, add)
-        if triggers != None:
-            if type(triggers) == ListType:
+        if triggers is not None:
+            if type(triggers) == list:
                 try:
                     t_streams = [obj[0] for obj in triggers]
                 except TypeError:
@@ -485,12 +501,13 @@ class VoiceManager(PyoObject):
 
         :Args:
 
-            x : PyoObject
+            x: PyoObject
                 New signal to process.
-            fadetime : float, optional
+            fadetime: float, optional
                 Crossfade time between old and new input. Defaults to 0.05.
 
         """
+        pyoArgsAssert(self, "oN", x, fadetime)
         self._input = x
         self._in_fader.setInput(x, fadetime)
 
@@ -500,13 +517,14 @@ class VoiceManager(PyoObject):
 
         :Args:
 
-            x : PyoObject or list of PyoObject
+            x: PyoObject or list of PyoObject
                 New `triggers` attribute.
 
         """
+        #pyoArgsAssert(self, "o", x)
         self._triggers = x
-        if x != None:
-            if type(x) == ListType:
+        if x is not None:
+            if type(x) == list:
                 try:
                     t_streams = [obj[0] for obj in x]
                 except TypeError:
@@ -544,13 +562,13 @@ class Mixer(PyoObject):
 
     :Args:
 
-        outs : int, optional
+        outs: int, optional
             Number of outputs of the mixer. Available at initialization
             time only. Defaults to 2.
-        chnls : int, optional
+        chnls: int, optional
             Number of channels per output. Available at initialization
             time only. Defaults to 1.
-        time : float, optional
+        time: float, optional
             Duration, in seconds, of a portamento applied on
             a new amplitude value for a mixing channel.
             Defaults to 0.025.
@@ -577,13 +595,8 @@ class Mixer(PyoObject):
 
     """
     def __init__(self, outs=2, chnls=1, time=0.025, mul=1, add=0):
+        pyoArgsAssert(self, "IInOO", outs, chnls, time, mul, add)
         PyoObject.__init__(self, mul, add)
-        if type(outs) != IntType:
-            print >> sys.stderr, 'TypeError: "outs" argument of %s must be an integer.\n' % self.__class__.__name__
-            exit()
-        if type(chnls) != IntType:
-            print >> sys.stderr, 'TypeError: "chnls" argument of %s must be an integer.\n' % self.__class__.__name__
-            exit()
         self._outs = outs
         self._chnls = chnls
         self._time = time
@@ -593,12 +606,12 @@ class Mixer(PyoObject):
         self._base_objs = [MixerVoice_base(self._base_players[j], i, wrap(mul,i), wrap(add,i)) for i in range(outs) for j in range(chnls)]
 
     def __getitem__(self, x):
-        if type(x) == SliceType:
-            return [self._base_objs[j*self._chnls+i] for j in range(x.start or 0, x.stop or sys.maxint, x.step or 1) for i in range(self._chnls)]
+        if type(x) == slice:
+            return [self._base_objs[j*self._chnls+i] for j in range(x.start or 0, x.stop or sys.maxsize, x.step or 1) for i in range(self._chnls)]
         elif x < len(self._base_objs):
             return [self._base_objs[x*self._chnls+i] for i in range(self._chnls)]
         else:
-            print "'x' too large!"
+            print("'x' too large!")
 
     def setTime(self, x):
         """
@@ -606,10 +619,11 @@ class Mixer(PyoObject):
 
         :Args:
 
-            x : float
+            x: float
                 New portamento duration.
 
         """
+        pyoArgsAssert(self, "n", x)
         self._time = x
         x, lmax = convertArgsToLists(x)
         [obj.setTime(wrap(x,i)) for i, obj in enumerate(self._base_players)]
@@ -622,19 +636,20 @@ class Mixer(PyoObject):
 
         :Args:
 
-            voice : int or string
+            voice: int or string
                 Key in the mixer dictionary for this input. If None, a unique key
                 between 0 and 32767 will be automatically generated.
-            input : PyoObject
+            input: PyoObject
                 Audio object to add to the mixer.
 
         """
-        if voice == None:
+        pyoArgsAssert(self, "o", input)
+        if voice is None:
             voice = random.randint(0, 32767)
-            while self._inputs.has_key(voice):
+            while voice in self._inputs:
                 voice = random.randint(0, 32767)
-        if self._inputs.has_key(voice):
-            print >> sys.stderr, "Mixer has already a key named %s" % voice
+        if voice in self._inputs:
+            print("Mixer has already a key named %s" % voice, file=sys.stderr)
             return
         self._inputs[voice] = input
         input, lmax = convertArgsToLists(input)
@@ -647,11 +662,11 @@ class Mixer(PyoObject):
 
         :Args:
 
-            voice : int or string
+            voice: int or string
                 Key in the mixer dictionary assigned to the input to remove.
 
         """
-        if self._inputs.has_key(voice):
+        if voice in self._inputs:
             del self._inputs[voice]
             [obj.delInput(str(voice)) for i, obj in enumerate(self._base_players)]
 
@@ -661,15 +676,15 @@ class Mixer(PyoObject):
 
         :Args:
 
-            vin : int or string
+            vin: int or string
                 Key in the mixer dictionary of the desired input.
-            vout : int
+            vout: int
                 Ouput channel where to send the signal.
-            amp : float
+            amp: float
                 Amplitude value for this mixing channel.
 
         """
-        if self._inputs.has_key(vin) and vout < self._outs:
+        if vin in self._inputs and vout < self._outs:
             [obj.setAmp(str(vin), vout, amp) for i, obj in enumerate(self._base_players)]
 
     def getChannels(self):
@@ -684,7 +699,7 @@ class Mixer(PyoObject):
         Returns the list of current keys in the Mixer's channels dictionary.
 
         """
-        return self._inputs.keys()
+        return list(self._inputs.keys())
 
     def ctrl(self, map_list=None, title=None, wxnoserver=False):
         self._map_list = [SLMap(0, 10, 'lin', 'time', self._time, dataOnly=True),
@@ -696,4 +711,4 @@ class Mixer(PyoObject):
         """float. Portamento."""
         return self._time
     @time.setter
-    def time(self, x): self.setTime(x)
\ No newline at end of file
+    def time(self, x): self.setTime(x)
