@@ -4,6 +4,7 @@ Useful for event sequencing.
 
 """
 
+from __future__ import absolute_import
 """
 Copyright 2009-2015 Olivier Belanger
 
@@ -23,10 +24,8 @@ GNU Lesser General Public License for more details.
 You should have received a copy of the GNU Lesser General Public
 License along with pyo.  If not, see <http://www.gnu.org/licenses/>.
 """
-import sys
-from _core import *
-from _maps import *
-from types import ListType, TupleType
+from ._core import *
+from ._maps import *
 
 class Pattern(PyoObject):
     """
@@ -39,10 +38,13 @@ class Pattern(PyoObject):
 
     :Args:
 
-        function : Python function
+        function: Python function
             Python function to be called periodically.
-        time : float or PyoObject, optional
+        time: float or PyoObject, optional
             Time, in seconds, between each call. Default to 1.
+        arg: anything, optional
+            Argument sent to the function's call. If None, the function
+            will be called without argument. Defaults to None.
 
     .. note::
 
@@ -50,6 +52,16 @@ class Pattern(PyoObject):
 
         Pattern has no `mul` and `add` attributes.
 
+        If `arg` is None, the function must be defined without argument:
+
+        >>> def tocall():
+        >>>     # function's body
+
+        If `arg` is not None, the function must be defined with one argument:
+
+        >>> def tocall(arg):
+        >>>     print arg
+
     >>> s = Server().boot()
     >>> s.start()
     >>> t = HarmTable([1,0,.33,0,.2,0,.143,0,.111])
@@ -61,20 +73,14 @@ class Pattern(PyoObject):
     >>> p.play()
 
     """
-    def __init__(self, function, time=1):
+    def __init__(self, function, time=1, arg=None):
+        pyoArgsAssert(self, "cO", function, time)
         PyoObject.__init__(self)
-        if type(function) == ListType or type(function) == TupleType:
-            if not callable(function[0]):
-                print >> sys.stderr, 'TypeError: "function" argument of %s must be callable.\n' % self.__class__.__name__
-                exit()
-        else:
-            if not callable(function):
-                print >> sys.stderr, 'TypeError: "function" argument of %s must be callable.\n' % self.__class__.__name__
-                exit()
         self._function = getWeakMethodRef(function)
         self._time = time
-        function, time, lmax = convertArgsToLists(function, time)
-        self._base_objs = [Pattern_base(WeakMethod(wrap(function,i)), wrap(time,i)) for i in range(lmax)]
+        self._arg = arg
+        function, time, arg, lmax = convertArgsToLists(function, time, arg)
+        self._base_objs = [Pattern_base(WeakMethod(wrap(function,i)), wrap(time,i), wrap(arg,i)) for i in range(lmax)]
 
     def setFunction(self, x):
         """
@@ -82,10 +88,11 @@ class Pattern(PyoObject):
 
         :Args:
 
-            x : Python function
+            x: Python function
                 new `function` attribute.
 
         """
+        pyoArgsAssert(self, "c", x)
         self._function = getWeakMethodRef(x)
         x, lmax = convertArgsToLists(x)
         [obj.setFunction(WeakMethod(wrap(x,i))) for i, obj in enumerate(self._base_objs)]
@@ -96,14 +103,29 @@ class Pattern(PyoObject):
 
         :Args:
 
-            x : float or PyoObject
+            x: float or PyoObject
                 New `time` attribute.
 
         """
+        pyoArgsAssert(self, "O", x)
         self._time = x
         x, lmax = convertArgsToLists(x)
         [obj.setTime(wrap(x,i)) for i, obj in enumerate(self._base_objs)]
 
+    def setArg(self, x):
+        """
+        Replace the `arg` attribute.
+
+        :Args:
+
+            x: Anything
+                new `arg` attribute.
+
+        """
+        self._arg = x
+        x, lmax = convertArgsToLists(x)
+        [obj.setArg(wrap(x,i)) for i, obj in enumerate(self._base_objs)]
+
     def out(self, x=0, inc=1, dur=0, delay=0):
         return self.play(dur, delay)
 
@@ -136,6 +158,13 @@ class Pattern(PyoObject):
         return self._time
     @time.setter
     def time(self, x): self.setTime(x)
+    @property
+    def arg(self):
+        """Anything. Callable's argument."""
+        return self._arg
+    @arg.setter
+    def arg(self, x):
+        self.setArg(x)
 
 class Score(PyoObject):
     """
@@ -153,10 +182,10 @@ class Score(PyoObject):
 
     :Args:
 
-        input : PyoObject
+        input: PyoObject
             Audio signal. Must contains integer numbers. Integer must change
             before calling its function again.
-        fname : string, optional
+        fname: string, optional
             Name of the functions to be called. Defaults to "event_", meaning
             that the object will call the function "event_0", "event_1", "event_2",
             and so on... Available at initialization time only.
@@ -185,6 +214,7 @@ class Score(PyoObject):
 
     """
     def __init__(self, input, fname="event_"):
+        pyoArgsAssert(self, "os", input, fname)
         PyoObject.__init__(self)
         self._input = input
         self._fname = fname
@@ -207,12 +237,13 @@ class Score(PyoObject):
 
         :Args:
 
-            x : PyoObject
+            x: PyoObject
                 New signal to process.
-            fadetime : float, optional
+            fadetime: float, optional
                 Crossfade time between old and new input. Defaults to 0.05.
 
         """
+        pyoArgsAssert(self, "oN", x, fadetime)
         self._input = x
         self._in_fader.setInput(x, fadetime)
 
@@ -232,11 +263,11 @@ class CallAfter(PyoObject):
 
     :Args:
 
-        function : Python function
+        function: Python function
             Python callable execute after `time` seconds.
-        time : float, optional
+        time: float, optional
             Time, in seconds, before the call. Default to 1.
-        arg : any Python object, optional
+        arg: any Python object, optional
             Argument sent to the called function. Default to None.
 
     .. note::
@@ -245,6 +276,16 @@ class CallAfter(PyoObject):
 
         CallAfter has no `mul` and `add` attributes.
 
+        If `arg` is None, the function must be defined without argument:
+
+        >>> def tocall():
+        >>>     # function's body
+
+        If `arg` is not None, the function must be defined with one argument:
+
+        >>> def tocall(arg):
+        >>>     print arg
+
         The object is not deleted after the call. The user must delete it himself.
 
     >>> s = Server().boot()
@@ -258,15 +299,8 @@ class CallAfter(PyoObject):
 
     """
     def __init__(self, function, time=1, arg=None):
+        pyoArgsAssert(self, "cn", function, time)
         PyoObject.__init__(self)
-        if type(function) == ListType or type(function) == TupleType:
-            if not callable(function[0]):
-                print >> sys.stderr, 'TypeError: "function" argument of %s must be callable.\n' % self.__class__.__name__
-                exit()
-        else:
-            if not callable(function):
-                print >> sys.stderr, 'TypeError: "function" argument of %s must be callable.\n' % self.__class__.__name__
-                exit()
         self._function = getWeakMethodRef(function)
         function, time, arg, lmax = convertArgsToLists(function, time, arg)
         self._base_objs = [CallAfter_base(WeakMethod(wrap(function,i)), wrap(time,i), wrap(arg,i)) for i in range(lmax)]
@@ -284,4 +318,4 @@ class CallAfter(PyoObject):
         pass
 
     def setDiv(self, x):
-        pass
\ No newline at end of file
+        pass
