@@ -1,3 +1,6 @@
+from __future__ import division
+from __future__ import print_function
+from __future__ import absolute_import
 """
 Play soundfiles from the disk.
 
@@ -24,10 +27,9 @@ GNU Lesser General Public License for more details.
 You should have received a copy of the GNU Lesser General Public
 License along with pyo.  If not, see <http://www.gnu.org/licenses/>.
 """
-from _core import *
-from _maps import *
+from ._core import *
+from ._maps import *
 import aifc
-from types import ListType
 
 class SfPlayer(PyoObject):
     """
@@ -42,9 +44,9 @@ class SfPlayer(PyoObject):
 
     :Args:
 
-        path : string
+        path: string
             Full path name of the sound to read.
-        speed : float or PyoObject, optional
+        speed: float or PyoObject, optional
             Transpose the pitch of input sound by this factor.
             Defaults to 1.
 
@@ -55,12 +57,12 @@ class SfPlayer(PyoObject):
 
             Although the `speed` attribute accepts audio
             rate signal, its value is updated only once per buffer size.
-        loop : bool, optional
+        loop: bool, optional
             If set to True, sound will play in loop. Defaults to False.
-        offset : float, optional
+        offset: float, optional
             Time in seconds of input sound to be skipped, assuming speed = 1.
             Defaults to 0.
-        interp : int, optional
+        interp: int, optional
             Interpolation type. Defaults to 2.
                 1. no interpolation
                 2. linear
@@ -69,13 +71,22 @@ class SfPlayer(PyoObject):
 
     .. note::
 
-        SfPlayer will sends a trigger signal at the end of the playback if
+        SfPlayer will send a trigger signal at the end of the playback if
         loop is off or any time it wraps around if loop is on. User can
         retrieve the trigger streams by calling obj['trig']:
 
         >>> sf = SfPlayer(SNDS_PATH + "/transparent.aif").out()
         >>> trig = TrigRand(sf['trig'])
 
+        Note that the object will send as many trigs as there is channels
+        in the sound file. If you want to retrieve only one trig, only give
+        the first stream to the next object:
+
+        >>> def printing():
+        ...     print "one trig!"
+        >>> sf = SfPlayer("/stereo/sound/file.aif").out()
+        >>> trig = TrigFunc(sf['trig'][0], printing)
+
     >>> s = Server().boot()
     >>> s.start()
     >>> snd = SNDS_PATH + "/transparent.aif"
@@ -111,16 +122,16 @@ class SfPlayer(PyoObject):
 
         :Args:
 
-            path : string
+            path: string
                 Full path of the new sound.
 
         """
         pyoArgsAssert(self, "s", path)
-        if type(self._path) == ListType:
+        if type(self._path) == list:
             curNchnls = sndinfo(self._path[0])[3]
         else:
             curNchnls = sndinfo(self._path)[3]
-        if type(path) == ListType:
+        if type(path) == list:
             p = path[0]
         else:
             p = path
@@ -129,7 +140,7 @@ class SfPlayer(PyoObject):
         except:
             return
         if _snd_chnls != curNchnls:
-            print "Soundfile must contains exactly %d channels." % curNchnls
+            print("Soundfile must contains exactly %d channels." % curNchnls)
             return
 
         self._path = path
@@ -145,7 +156,7 @@ class SfPlayer(PyoObject):
 
         :Args:
 
-            path : string
+            path: string
                 Full path of the new sound.
 
         """
@@ -157,7 +168,7 @@ class SfPlayer(PyoObject):
 
         :Args:
 
-            x : float or PyoObject
+            x: float or PyoObject
                 new `speed` attribute.
 
         """
@@ -172,7 +183,7 @@ class SfPlayer(PyoObject):
 
         :Args:
 
-            x : bool {True, False}
+            x: bool {True, False}
                 new `loop` attribute.
 
         """
@@ -189,7 +200,7 @@ class SfPlayer(PyoObject):
 
         :Args:
 
-            x : float
+            x: float
                 new `offset` attribute.
 
         """
@@ -204,7 +215,7 @@ class SfPlayer(PyoObject):
 
         :Args:
 
-            x : int {1, 2, 3, 4}
+            x: int {1, 2, 3, 4}
                 new `interp` attribute.
 
         """
@@ -279,10 +290,10 @@ class SfMarkerShuffler(PyoObject):
 
     :Args:
 
-        path : string
+        path: string
             Full path name of the sound to read. Can't e changed after
             initialization.
-        speed : float or PyoObject, optional
+        speed: float or PyoObject, optional
             Transpose the pitch of input sound by this factor.
             Defaults to 1.
 
@@ -293,7 +304,7 @@ class SfMarkerShuffler(PyoObject):
 
             Although the `speed` attribute accepts audio
             rate signal, its value is updated only once per buffer size.
-        interp : int, optional
+        interp: int, optional
             Choice of the interpolation method. Defaults to 2.
                 1. no interpolation
                 2. linear
@@ -324,7 +335,7 @@ class SfMarkerShuffler(PyoObject):
                 self._markers = []
             self._base_players.append(SfMarkerShuffler_base(wrap(path,i), self._markers, wrap(speed,i), wrap(interp,i)))
         for i in range(lmax * self._snd_chnls):
-            j = i / self._snd_chnls
+            j = i // self._snd_chnls
             self._base_objs.append(SfMarkerShuffle_base(wrap(self._base_players,j), i % self._snd_chnls, wrap(mul,j), wrap(add,j)))
 
     def setSpeed(self, x):
@@ -333,7 +344,7 @@ class SfMarkerShuffler(PyoObject):
 
         :Args:
 
-            x : float or PyoObject
+            x: float or PyoObject
                 new `speed` attribute.
 
         """
@@ -348,7 +359,7 @@ class SfMarkerShuffler(PyoObject):
 
         :Args:
 
-            x : int {1, 2, 3, 4}
+            x: int {1, 2, 3, 4}
                 new `interp` attribute.
 
         """
@@ -401,9 +412,9 @@ class SfMarkerLooper(PyoObject):
 
     :Args:
 
-        path : string
+        path: string
             Full path name of the sound to read.
-        speed : float or PyoObject, optional
+        speed: float or PyoObject, optional
             Transpose the pitch of input sound by this factor.
             Defaults to 1.
 
@@ -414,10 +425,10 @@ class SfMarkerLooper(PyoObject):
 
             Although the `speed` attribute accepts audio
             rate signal, its value is updated only once per buffer size.
-        mark : float or PyoObject, optional
+        mark: float or PyoObject, optional
             Integer denoting the marker to loop, in the range
             0 -> len(getMarkers()). Defaults to 0.
-        interp : int, optional
+        interp: int, optional
             Choice of the interpolation method. Defaults to 2.
                 1. no interpolation
                 2. linear
@@ -451,7 +462,7 @@ class SfMarkerLooper(PyoObject):
                 self._markers = []
             self._base_players.append(SfMarkerLooper_base(wrap(path,i), self._markers, wrap(speed,i), wrap(mark,i), wrap(interp,i)))
         for i in range(lmax * self._snd_chnls):
-            j = i / self._snd_chnls
+            j = i // self._snd_chnls
             self._base_objs.append(SfMarkerLoop_base(wrap(self._base_players,j), i % self._snd_chnls, wrap(mul,j), wrap(add,j)))
 
     def setSpeed(self, x):
@@ -460,7 +471,7 @@ class SfMarkerLooper(PyoObject):
 
         :Args:
 
-            x : float or PyoObject
+            x: float or PyoObject
                 new `speed` attribute.
 
         """
@@ -475,7 +486,7 @@ class SfMarkerLooper(PyoObject):
 
         :Args:
 
-            x : float or PyoObject
+            x: float or PyoObject
                 new `mark` attribute.
 
         """
@@ -490,7 +501,7 @@ class SfMarkerLooper(PyoObject):
 
         :Args:
 
-            x : int {1, 2, 3, 4}
+            x: int {1, 2, 3, 4}
                 new `interp` attribute.
 
         """
@@ -532,4 +543,4 @@ class SfMarkerLooper(PyoObject):
         """int {1, 2, 3, 4}. Interpolation method."""
         return self._interp
     @interp.setter
-    def interp(self, x): self.setInterp(x)
\ No newline at end of file
+    def interp(self, x): self.setInterp(x)
