@@ -2,6 +2,7 @@
 Set of objects that implement different kinds of random noise generators.
 
 """
+from __future__ import absolute_import
 """
 Copyright 2009-2015 Olivier Belanger
 
@@ -21,10 +22,8 @@ GNU Lesser General Public License for more details.
 You should have received a copy of the GNU Lesser General Public
 License along with pyo.  If not, see <http://www.gnu.org/licenses/>.
 """
-import sys
-from _core import *
-from _maps import *
-from types import StringType, ListType
+from ._core import *
+from ._maps import *
 
 class Randi(PyoObject):
     """
@@ -38,11 +37,11 @@ class Randi(PyoObject):
 
     :Args:
 
-        min : float or PyoObject, optional
+        min: float or PyoObject, optional
             Minimum value for the random generation. Defaults to 0.
-        max : float or PyoObject, optional
+        max: float or PyoObject, optional
             Maximum value for the random generation. Defaults to 1.
-        freq : float or PyoObject, optional
+        freq: float or PyoObject, optional
             Polling frequency. Defaults to 1.
 
     >>> s = Server().boot()
@@ -53,6 +52,7 @@ class Randi(PyoObject):
 
     """
     def __init__(self, min=0., max=1., freq=1., mul=1, add=0):
+        pyoArgsAssert(self, "OOOOO", min, max, freq, mul, add)
         PyoObject.__init__(self, mul, add)
         self._min = min
         self._max = max
@@ -66,10 +66,11 @@ class Randi(PyoObject):
 
         :Args:
 
-            x : float or PyoObject
+            x: float or PyoObject
                 new `min` attribute.
 
         """
+        pyoArgsAssert(self, "O", x)
         self._min = x
         x, lmax = convertArgsToLists(x)
         [obj.setMin(wrap(x,i)) for i, obj in enumerate(self._base_objs)]
@@ -80,10 +81,11 @@ class Randi(PyoObject):
 
         :Args:
 
-            x : float or PyoObject
+            x: float or PyoObject
                 new `max` attribute.
 
         """
+        pyoArgsAssert(self, "O", x)
         self._max = x
         x, lmax = convertArgsToLists(x)
         [obj.setMax(wrap(x,i)) for i, obj in enumerate(self._base_objs)]
@@ -94,11 +96,12 @@ class Randi(PyoObject):
 
         :Args:
 
-            x : float or PyoObject
+            x: float or PyoObject
                 new `freq` attribute.
 
         """
-        self._port = x
+        pyoArgsAssert(self, "O", x)
+        self._freq = x
         x, lmax = convertArgsToLists(x)
         [obj.setFreq(wrap(x,i)) for i, obj in enumerate(self._base_objs)]
 
@@ -143,11 +146,11 @@ class Randh(PyoObject):
 
     :Args:
 
-        min : float or PyoObject, optional
+        min: float or PyoObject, optional
             Minimum value for the random generation. Defaults to 0.
-        max : float or PyoObject, optional
+        max: float or PyoObject, optional
             Maximum value for the random generation. Defaults to 1.
-        freq : float or PyoObject, optional
+        freq: float or PyoObject, optional
             Polling frequency. Defaults to 1.
 
     >>> s = Server().boot()
@@ -158,6 +161,7 @@ class Randh(PyoObject):
 
     """
     def __init__(self, min=0., max=1., freq=1., mul=1, add=0):
+        pyoArgsAssert(self, "OOOOO", min, max, freq, mul, add)
         PyoObject.__init__(self, mul, add)
         self._min = min
         self._max = max
@@ -171,10 +175,11 @@ class Randh(PyoObject):
 
         :Args:
 
-            x : float or PyoObject
+            x: float or PyoObject
                 new `min` attribute.
 
         """
+        pyoArgsAssert(self, "O", x)
         self._min = x
         x, lmax = convertArgsToLists(x)
         [obj.setMin(wrap(x,i)) for i, obj in enumerate(self._base_objs)]
@@ -185,10 +190,11 @@ class Randh(PyoObject):
 
         :Args:
 
-            x : float or PyoObject
+            x: float or PyoObject
                 new `max` attribute.
 
         """
+        pyoArgsAssert(self, "O", x)
         self._max = x
         x, lmax = convertArgsToLists(x)
         [obj.setMax(wrap(x,i)) for i, obj in enumerate(self._base_objs)]
@@ -199,11 +205,12 @@ class Randh(PyoObject):
 
         :Args:
 
-            x : float or PyoObject
+            x: float or PyoObject
                 new `freq` attribute.
 
         """
-        self._port = x
+        pyoArgsAssert(self, "O", x)
+        self._freq = x
         x, lmax = convertArgsToLists(x)
         [obj.setFreq(wrap(x,i)) for i, obj in enumerate(self._base_objs)]
 
@@ -248,9 +255,9 @@ class Choice(PyoObject):
 
     :Args:
 
-        choice : list of floats or list of lists of floats
+        choice: list of floats or list of lists of floats
             Possible values for the random generation.
-        freq : float or PyoObject, optional
+        freq: float or PyoObject, optional
             Polling frequency. Defaults to 1.
 
     >>> s = Server().boot()
@@ -261,14 +268,12 @@ class Choice(PyoObject):
 
     """
     def __init__(self, choice, freq=1., mul=1, add=0):
+        pyoArgsAssert(self, "lOOO", choice, freq, mul, add)
         PyoObject.__init__(self, mul, add)
-        if type(choice) != ListType:
-            print >> sys.stderr, 'TypeError: "choice" argument of %s must be a list.\n' % self.__class__.__name__
-            exit()
         self._choice = choice
         self._freq = freq
         freq, mul, add, lmax = convertArgsToLists(freq, mul, add)
-        if type(choice[0]) != ListType:
+        if type(choice[0]) != list:
             self._base_objs = [Choice_base(choice, wrap(freq,i), wrap(mul,i), wrap(add,i)) for i in range(lmax)]
         else:
             choicelen = len(choice)
@@ -281,12 +286,13 @@ class Choice(PyoObject):
 
         :Args:
 
-            x : list of floats or list of lists of floats
+            x: list of floats or list of lists of floats
                 new `choice` attribute.
 
         """
+        pyoArgsAssert(self, "l", x)
         self._choice = x
-        if type(x[0]) != ListType:
+        if type(x[0]) != list:
             [obj.setChoice(self._choice) for i, obj in enumerate(self._base_objs)]
         else:
             [obj.setChoice(wrap(self._choice,i)) for i, obj in enumerate(self._base_objs)]
@@ -297,10 +303,11 @@ class Choice(PyoObject):
 
         :Args:
 
-            x : float or PyoObject
+            x: float or PyoObject
                 new `freq` attribute.
 
         """
+        pyoArgsAssert(self, "O", x)
         self._freq = x
         x, lmax = convertArgsToLists(x)
         [obj.setFreq(wrap(x,i)) for i, obj in enumerate(self._base_objs)]
@@ -336,9 +343,9 @@ class RandInt(PyoObject):
 
     :Args:
 
-        max : float or PyoObject, optional
+        max: float or PyoObject, optional
             Maximum value for the random generation. Defaults to 100.
-        freq : float or PyoObject, optional
+        freq: float or PyoObject, optional
             Polling frequency. Defaults to 1.
 
     >>> s = Server().boot()
@@ -349,6 +356,7 @@ class RandInt(PyoObject):
 
     """
     def __init__(self, max=100, freq=1., mul=1, add=0):
+        pyoArgsAssert(self, "OOOO", max, freq, mul, add)
         PyoObject.__init__(self, mul, add)
         self._max = max
         self._freq = freq
@@ -361,10 +369,11 @@ class RandInt(PyoObject):
 
         :Args:
 
-            x : float or PyoObject
+            x: float or PyoObject
                 new `max` attribute.
 
         """
+        pyoArgsAssert(self, "O", x)
         self._max = x
         x, lmax = convertArgsToLists(x)
         [obj.setMax(wrap(x,i)) for i, obj in enumerate(self._base_objs)]
@@ -375,10 +384,11 @@ class RandInt(PyoObject):
 
         :Args:
 
-            x : float or PyoObject
+            x: float or PyoObject
                 new `freq` attribute.
 
         """
+        pyoArgsAssert(self, "O", x)
         self._freq = x
         x, lmax = convertArgsToLists(x)
         [obj.setFreq(wrap(x,i)) for i, obj in enumerate(self._base_objs)]
@@ -416,9 +426,9 @@ class RandDur(PyoObject):
 
     :Args:
 
-        min : float or PyoObject, optional
+        min: float or PyoObject, optional
             Minimum value for the random generation. Defaults to 0.
-        max : float or PyoObject, optional
+        max: float or PyoObject, optional
             Maximum value for the random generation. Defaults to 1.
 
     >>> s = Server().boot()
@@ -432,6 +442,7 @@ class RandDur(PyoObject):
 
     """
     def __init__(self, min=0., max=1., mul=1, add=0):
+        pyoArgsAssert(self, "OOOO", min, max, mul, add)
         PyoObject.__init__(self, mul, add)
         self._min = min
         self._max = max
@@ -444,10 +455,11 @@ class RandDur(PyoObject):
 
         :Args:
 
-            x : float or PyoObject
+            x: float or PyoObject
                 new `min` attribute.
 
         """
+        pyoArgsAssert(self, "O", x)
         self._min = x
         x, lmax = convertArgsToLists(x)
         [obj.setMin(wrap(x,i)) for i, obj in enumerate(self._base_objs)]
@@ -458,10 +470,11 @@ class RandDur(PyoObject):
 
         :Args:
 
-            x : float or PyoObject
+            x: float or PyoObject
                 new `max` attribute.
 
         """
+        pyoArgsAssert(self, "O", x)
         self._max = x
         x, lmax = convertArgsToLists(x)
         [obj.setMax(wrap(x,i)) for i, obj in enumerate(self._base_objs)]
@@ -498,13 +511,13 @@ class Xnoise(PyoObject):
 
     :Args:
 
-        dist : string or int, optional
+        dist: string or int, optional
             Distribution type. Defaults to 0.
-        freq : float or PyoObject, optional
+        freq: float or PyoObject, optional
             Polling frequency. Defaults to 1.
-        x1 : float or PyoObject, optional
+        x1: float or PyoObject, optional
             First parameter. Defaults to 0.5.
-        x2 : float or PyoObject, optional
+        x2: float or PyoObject, optional
             Second parameter. Defaults to 0.5.
 
     .. note::
@@ -529,44 +542,44 @@ class Xnoise(PyoObject):
         parameter):
 
             0. uniform
-                - x1 : not used
-                - x2 : not used
+                - x1: not used
+                - x2: not used
             1. linear_min
-                - x1 : not used
-                - x2 : not used
+                - x1: not used
+                - x2: not used
             2. linear_max
-                - x1 : not used
-                - x2 : not used
+                - x1: not used
+                - x2: not used
             3. triangle
-                - x1 : not used
-                - x2 : not used
+                - x1: not used
+                - x2: not used
             4. expon_min
-                - x1 : slope {0 = no slope -> 10 = sharp slope}
-                - x2 : not used
+                - x1: slope {0 = no slope -> 10 = sharp slope}
+                - x2: not used
             5. expon_max
-                - x1 : slope {0 = no slope -> 10 = sharp slope}
-                - x2 : not used
+                - x1: slope {0 = no slope -> 10 = sharp slope}
+                - x2: not used
             6. biexpon
-                - x1 : bandwidth {0 = huge bandwidth -> 10 = narrow bandwidth}
-                - x2 : not used
+                - x1: bandwidth {0 = huge bandwidth -> 10 = narrow bandwidth}
+                - x2: not used
             7. cauchy
-                - x1 : bandwidth {0 = narrow bandwidth -> 10 = huge bandwidth}
-                - x2 : not used
+                - x1: bandwidth {0 = narrow bandwidth -> 10 = huge bandwidth}
+                - x2: not used
             8. weibull
-                - x1 : mean location {0 -> 1}
-                - x2 : shape {0.5 = linear min, 1.5 = expon min, 3.5 = gaussian}
+                - x1: mean location {0 -> 1}
+                - x2: shape {0.5 = linear min, 1.5 = expon min, 3.5 = gaussian}
             9. gaussian
-                - x1 : mean location {0 -> 1}
-                - x2 : bandwidth {0 =  narrow bandwidth -> 10 = huge bandwidth}
+                - x1: mean location {0 -> 1}
+                - x2: bandwidth {0 =  narrow bandwidth -> 10 = huge bandwidth}
             10. poisson
-                 - x1 : gravity center {0 = low values -> 10 = high values}
-                 - x2 : compress/expand range {0.1 = full compress -> 4 full expand}
+                 - x1: gravity center {0 = low values -> 10 = high values}
+                 - x2: compress/expand range {0.1 = full compress -> 4 full expand}
             11. walker
-                 - x1 : maximum value {0.1 -> 1}
-                 - x2 : maximum step {0.1 -> 1}
+                 - x1: maximum value {0.1 -> 1}
+                 - x2: maximum step {0.1 -> 1}
             12. loopseg
-                 - x1 : maximum value {0.1 -> 1}
-                 - x2 : maximum step {0.1 -> 1}
+                 - x1: maximum value {0.1 -> 1}
+                 - x2: maximum step {0.1 -> 1}
 
     >>> s = Server().boot()
     >>> s.start()
@@ -577,6 +590,7 @@ class Xnoise(PyoObject):
 
     """
     def __init__(self, dist=0, freq=1., x1=0.5, x2=0.5, mul=1, add=0):
+        pyoArgsAssert(self, "OOOOO", freq, x1, x2, mul, add)
         PyoObject.__init__(self, mul, add)
         self._dist = dist
         self._freq = freq
@@ -584,7 +598,7 @@ class Xnoise(PyoObject):
         self._x2 = x2
         dist, freq, x1, x2, mul, add, lmax = convertArgsToLists(dist, freq, x1, x2, mul, add)
         for i, t in enumerate(dist):
-            if type(t) == StringType: dist[i] = XNOISE_DICT.get(t, 0)
+            if type(t) in [bytes_t, unicode_t]: dist[i] = XNOISE_DICT.get(t, 0)
         self._base_objs = [Xnoise_base(wrap(dist,i), wrap(freq,i), wrap(x1,i), wrap(x2,i), wrap(mul,i), wrap(add,i)) for i in range(lmax)]
 
     def setDist(self, x):
@@ -593,14 +607,14 @@ class Xnoise(PyoObject):
 
         :Args:
 
-            x : int
+            x: string or int
                 new `dist` attribute.
 
         """
         self._dist = x
         x, lmax = convertArgsToLists(x)
         for i, t in enumerate(x):
-            if type(t) == StringType: x[i] = XNOISE_DICT.get(t, 0)
+            if type(t) in [bytes_t, unicode_t]: x[i] = XNOISE_DICT.get(t, 0)
         [obj.setType(wrap(x,i)) for i, obj in enumerate(self._base_objs)]
 
     def setX1(self, x):
@@ -609,10 +623,11 @@ class Xnoise(PyoObject):
 
         :Args:
 
-            x : float or PyoObject
+            x: float or PyoObject
                 new `x1` attribute.
 
         """
+        pyoArgsAssert(self, "O", x)
         self._x1 = x
         x, lmax = convertArgsToLists(x)
         [obj.setX1(wrap(x,i)) for i, obj in enumerate(self._base_objs)]
@@ -623,10 +638,11 @@ class Xnoise(PyoObject):
 
         :Args:
 
-            x : float or PyoObject
+            x: float or PyoObject
                 new `x2` attribute.
 
         """
+        pyoArgsAssert(self, "O", x)
         self._x2 = x
         x, lmax = convertArgsToLists(x)
         [obj.setX2(wrap(x,i)) for i, obj in enumerate(self._base_objs)]
@@ -637,11 +653,12 @@ class Xnoise(PyoObject):
 
         :Args:
 
-            x : float or PyoObject
+            x: float or PyoObject
                 new `freq` attribute.
 
         """
-        self._port = x
+        pyoArgsAssert(self, "O", x)
+        self._freq = x
         x, lmax = convertArgsToLists(x)
         [obj.setFreq(wrap(x,i)) for i, obj in enumerate(self._base_objs)]
 
@@ -696,20 +713,20 @@ class XnoiseMidi(PyoObject):
 
     :Args:
 
-        dist : string or int, optional
+        dist: string or int, optional
             Distribution type. Defaults to 0.
-        freq : float or PyoObject, optional
+        freq: float or PyoObject, optional
             Polling frequency. Defaults to 1.
-        x1 : float or PyoObject, optional
+        x1: float or PyoObject, optional
             First parameter. Defaults to 0.5.
-        x2 : float or PyoObject, optional
+        x2: float or PyoObject, optional
             Second parameter. Defaults to 0.5.
-        scale : int {0, 1, 2}, optional
+        scale: int {0, 1, 2}, optional
             Output format. 0 = Midi, 1 = Hertz, 2 = transposition factor.
             In the transposition mode, the central key (the key where there
             is no transposition) is (`minrange` + `maxrange`) / 2. Defaults
             to 0.
-        mrange : tuple of int, optional
+        mrange: tuple of int, optional
             Minimum and maximum possible values, in Midi notes. Available
             only at initialization time. Defaults to (0, 127).
 
@@ -735,44 +752,44 @@ class XnoiseMidi(PyoObject):
         parameter):
 
             0. uniform
-                - x1 : not used
-                - x2 : not used
+                - x1: not used
+                - x2: not used
             1. linear_min
-                - x1 : not used
-                - x2 : not used
+                - x1: not used
+                - x2: not used
             2. linear_max
-                - x1 : not used
-                - x2 : not used
+                - x1: not used
+                - x2: not used
             3. triangle
-                - x1 : not used
-                - x2 : not used
+                - x1: not used
+                - x2: not used
             4. expon_min
-                - x1 : slope {0 = no slope -> 10 = sharp slope}
-                - x2 : not used
+                - x1: slope {0 = no slope -> 10 = sharp slope}
+                - x2: not used
             5. expon_max
-                - x1 : slope {0 = no slope -> 10 = sharp slope}
-                - x2 : not used
+                - x1: slope {0 = no slope -> 10 = sharp slope}
+                - x2: not used
             6. biexpon
-                - x1 : bandwidth {0 = huge bandwidth -> 10 = narrow bandwidth}
-                - x2 : not used
+                - x1: bandwidth {0 = huge bandwidth -> 10 = narrow bandwidth}
+                - x2: not used
             7. cauchy
-                - x1 : bandwidth {0 = narrow bandwidth -> 10 = huge bandwidth}
-                - x2 : not used
+                - x1: bandwidth {0 = narrow bandwidth -> 10 = huge bandwidth}
+                - x2: not used
             8. weibull
-                - x1 : mean location {0 -> 1}
-                - x2 : shape {0.5 = linear min, 1.5 = expon min, 3.5 = gaussian}
+                - x1: mean location {0 -> 1}
+                - x2: shape {0.5 = linear min, 1.5 = expon min, 3.5 = gaussian}
             9. gaussian
-                - x1 : mean location {0 -> 1}
-                - x2 : bandwidth {0 =  narrow bandwidth -> 10 = huge bandwidth}
+                - x1: mean location {0 -> 1}
+                - x2: bandwidth {0 =  narrow bandwidth -> 10 = huge bandwidth}
             10. poisson
-                 - x1 : gravity center {0 = low values -> 10 = high values}
-                 - x2 : compress/expand range {0.1 = full compress -> 4 full expand}
+                 - x1: gravity center {0 = low values -> 10 = high values}
+                 - x2: compress/expand range {0.1 = full compress -> 4 full expand}
             11. walker
-                 - x1 : maximum value {0.1 -> 1}
-                 - x2 : maximum step {0.1 -> 1}
+                 - x1: maximum value {0.1 -> 1}
+                 - x2: maximum step {0.1 -> 1}
             12. loopseg
-                 - x1 : maximum value {0.1 -> 1}
-                 - x2 : maximum step {0.1 -> 1}
+                 - x1: maximum value {0.1 -> 1}
+                 - x2: maximum step {0.1 -> 1}
 
     >>> s = Server().boot()
     >>> s.start()
@@ -784,6 +801,7 @@ class XnoiseMidi(PyoObject):
 
     """
     def __init__(self, dist=0, freq=1., x1=0.5, x2=0.5, scale=0, mrange=(0,127), mul=1, add=0):
+        pyoArgsAssert(self, "OOOixOO", freq, x1, x2, scale, mrange, mul, add)
         PyoObject.__init__(self, mul, add)
         self._dist = dist
         self._freq = freq
@@ -793,7 +811,7 @@ class XnoiseMidi(PyoObject):
         self._mrange = mrange
         dist, freq, x1, x2, scale, mrange, mul, add, lmax = convertArgsToLists(dist, freq, x1, x2, scale, mrange, mul, add)
         for i, t in enumerate(dist):
-            if type(t) == StringType: dist[i] = XNOISE_DICT.get(t, 0)
+            if type(t) in [bytes_t, unicode_t]: dist[i] = XNOISE_DICT.get(t, 0)
         self._base_objs = [XnoiseMidi_base(wrap(dist,i), wrap(freq,i), wrap(x1,i), wrap(x2,i), wrap(scale,i), wrap(mrange,i), wrap(mul,i), wrap(add,i)) for i in range(lmax)]
 
     def setDist(self, x):
@@ -802,14 +820,14 @@ class XnoiseMidi(PyoObject):
 
         :Args:
 
-            x : string or int
+            x: string or int
                 new `dist` attribute.
 
         """
         self._dist = x
         x, lmax = convertArgsToLists(x)
         for i, t in enumerate(x):
-            if type(t) == StringType: x[i] = XNOISE_DICT.get(t, 0)
+            if type(t) in [bytes_t, unicode_t]: x[i] = XNOISE_DICT.get(t, 0)
         [obj.setType(wrap(x,i)) for i, obj in enumerate(self._base_objs)]
 
     def setScale(self, x):
@@ -823,10 +841,11 @@ class XnoiseMidi(PyoObject):
 
         :Args:
 
-            x : int {0, 1, 2}
+            x: int {0, 1, 2}
                 new `scale` attribute.
 
         """
+        pyoArgsAssert(self, "i", x)
         self._scale = x
         x, lmax = convertArgsToLists(x)
         [obj.setScale(wrap(x,i)) for i, obj in enumerate(self._base_objs)]
@@ -837,12 +856,13 @@ class XnoiseMidi(PyoObject):
 
         :Args:
 
-            mini : int
+            mini: int
                 minimum output midi range.
-            maxi : int
+            maxi: int
                 maximum output midi range.
 
         """
+        pyoArgsAssert(self, "ii", mini, maxi)
         self._mrange = (mini, maxi)
         mini, maxi, lmax = convertArgsToLists(mini, maxi)
         [obj.setRange(wrap(mini,i), wrap(maxi,i)) for i, obj in enumerate(self._base_objs)]
@@ -853,10 +873,11 @@ class XnoiseMidi(PyoObject):
 
         :Args:
 
-            x : float or PyoObject
+            x: float or PyoObject
                 new `x1` attribute.
 
         """
+        pyoArgsAssert(self, "O", x)
         self._x1 = x
         x, lmax = convertArgsToLists(x)
         [obj.setX1(wrap(x,i)) for i, obj in enumerate(self._base_objs)]
@@ -867,11 +888,12 @@ class XnoiseMidi(PyoObject):
 
         :Args:
 
-            x : float or PyoObject
+            x: float or PyoObject
                 new `x2` attribute.
 
         """
-        self._x2= x
+        pyoArgsAssert(self, "O", x)
+        self._x2 = x
         x, lmax = convertArgsToLists(x)
         [obj.setX2(wrap(x,i)) for i, obj in enumerate(self._base_objs)]
 
@@ -881,11 +903,12 @@ class XnoiseMidi(PyoObject):
 
         :Args:
 
-            x : float or PyoObject
+            x: float or PyoObject
                 new `freq` attribute.
 
         """
-        self._port = x
+        pyoArgsAssert(self, "O", x)
+        self._freq = x
         x, lmax = convertArgsToLists(x)
         [obj.setFreq(wrap(x,i)) for i, obj in enumerate(self._base_objs)]
 
@@ -947,16 +970,16 @@ class XnoiseDur(PyoObject):
 
     :Args:
 
-        dist : string or int, optional
+        dist: string or int, optional
             Distribution type. Can be the name of the distribution as a string
             or its associated number. Defaults to 0.
-        min : float or PyoObject, optional
+        min: float or PyoObject, optional
             Minimum value for the random generation. Defaults to 0.
-        max : float or PyoObject, optional
+        max: float or PyoObject, optional
             Maximum value for the random generation. Defaults to 1.
-        x1 : float or PyoObject, optional
+        x1: float or PyoObject, optional
             First parameter. Defaults to 0.5.
-        x2 : float or PyoObject, optional
+        x2: float or PyoObject, optional
             Second parameter. Defaults to 0.5.
 
     .. note::
@@ -981,44 +1004,44 @@ class XnoiseDur(PyoObject):
         parameter):
 
             0. uniform
-                - x1 : not used
-                - x2 : not used
+                - x1: not used
+                - x2: not used
             1. linear_min
-                - x1 : not used
-                - x2 : not used
+                - x1: not used
+                - x2: not used
             2. linear_max
-                - x1 : not used
-                - x2 : not used
+                - x1: not used
+                - x2: not used
             3. triangle
-                - x1 : not used
-                - x2 : not used
+                - x1: not used
+                - x2: not used
             4. expon_min
-                - x1 : slope {0 = no slope -> 10 = sharp slope}
-                - x2 : not used
+                - x1: slope {0 = no slope -> 10 = sharp slope}
+                - x2: not used
             5. expon_max
-                - x1 : slope {0 = no slope -> 10 = sharp slope}
-                - x2 : not used
+                - x1: slope {0 = no slope -> 10 = sharp slope}
+                - x2: not used
             6. biexpon
-                - x1 : bandwidth {0 = huge bandwidth -> 10 = narrow bandwidth}
-                - x2 : not used
+                - x1: bandwidth {0 = huge bandwidth -> 10 = narrow bandwidth}
+                - x2: not used
             7. cauchy
-                - x1 : bandwidth {0 = narrow bandwidth -> 10 = huge bandwidth}
-                - x2 : not used
+                - x1: bandwidth {0 = narrow bandwidth -> 10 = huge bandwidth}
+                - x2: not used
             8. weibull
-                - x1 : mean location {0 -> 1}
-                - x2 : shape {0.5 = linear min, 1.5 = expon min, 3.5 = gaussian}
+                - x1: mean location {0 -> 1}
+                - x2: shape {0.5 = linear min, 1.5 = expon min, 3.5 = gaussian}
             9. gaussian
-                - x1 : mean location {0 -> 1}
-                - x2 : bandwidth {0 =  narrow bandwidth -> 10 = huge bandwidth}
+                - x1: mean location {0 -> 1}
+                - x2: bandwidth {0 =  narrow bandwidth -> 10 = huge bandwidth}
             10. poisson
-                 - x1 : gravity center {0 = low values -> 10 = high values}
-                 - x2 : compress/expand range {0.1 = full compress -> 4 full expand}
+                 - x1: gravity center {0 = low values -> 10 = high values}
+                 - x2: compress/expand range {0.1 = full compress -> 4 full expand}
             11. walker
-                 - x1 : maximum value {0.1 -> 1}
-                 - x2 : maximum step {0.1 -> 1}
+                 - x1: maximum value {0.1 -> 1}
+                 - x2: maximum step {0.1 -> 1}
             12. loopseg
-                 - x1 : maximum value {0.1 -> 1}
-                 - x2 : maximum step {0.1 -> 1}
+                 - x1: maximum value {0.1 -> 1}
+                 - x2: maximum step {0.1 -> 1}
 
     >>> s = Server().boot()
     >>> s.start()
@@ -1031,6 +1054,7 @@ class XnoiseDur(PyoObject):
 
     """
     def __init__(self, dist=0, min=0., max=1., x1=0.5, x2=0.5, mul=1, add=0):
+        pyoArgsAssert(self, "OOOOOO", min, max, x1, x2, mul, add)
         PyoObject.__init__(self, mul, add)
         self._dist = dist
         self._min = min
@@ -1039,7 +1063,7 @@ class XnoiseDur(PyoObject):
         self._x2 = x2
         dist, min, max, x1, x2, mul, add, lmax = convertArgsToLists(dist, min, max, x1, x2, mul, add)
         for i, t in enumerate(dist):
-            if type(t) == StringType: dist[i] = XNOISE_DICT.get(t, 0)
+            if type(t) in [bytes_t, unicode_t]: dist[i] = XNOISE_DICT.get(t, 0)
         self._base_objs = [XnoiseDur_base(wrap(dist,i), wrap(min,i), wrap(max,i), wrap(x1,i), wrap(x2,i), wrap(mul,i), wrap(add,i)) for i in range(lmax)]
 
     def setDist(self, x):
@@ -1048,14 +1072,14 @@ class XnoiseDur(PyoObject):
 
         :Args:
 
-            x : int
+            x: string or int
                 new `dist` attribute.
 
         """
         self._dist = x
         x, lmax = convertArgsToLists(x)
         for i, t in enumerate(x):
-            if type(t) == StringType: x[i] = XNOISE_DICT.get(t, 0)
+            if type(t) in [bytes_t, unicode_t]: x[i] = XNOISE_DICT.get(t, 0)
         [obj.setType(wrap(x,i)) for i, obj in enumerate(self._base_objs)]
 
     def setMin(self, x):
@@ -1064,10 +1088,11 @@ class XnoiseDur(PyoObject):
 
         :Args:
 
-            x : float or PyoObject
+            x: float or PyoObject
                 new `min` attribute.
 
         """
+        pyoArgsAssert(self, "O", x)
         self._min = x
         x, lmax = convertArgsToLists(x)
         [obj.setMin(wrap(x,i)) for i, obj in enumerate(self._base_objs)]
@@ -1078,10 +1103,11 @@ class XnoiseDur(PyoObject):
 
         :Args:
 
-            x : float or PyoObject
+            x: float or PyoObject
                 new `max` attribute.
 
         """
+        pyoArgsAssert(self, "O", x)
         self._max = x
         x, lmax = convertArgsToLists(x)
         [obj.setMax(wrap(x,i)) for i, obj in enumerate(self._base_objs)]
@@ -1092,10 +1118,11 @@ class XnoiseDur(PyoObject):
 
         :Args:
 
-            x : float or PyoObject
+            x: float or PyoObject
                 new `x1` attribute.
 
         """
+        pyoArgsAssert(self, "O", x)
         self._x1 = x
         x, lmax = convertArgsToLists(x)
         [obj.setX1(wrap(x,i)) for i, obj in enumerate(self._base_objs)]
@@ -1106,11 +1133,12 @@ class XnoiseDur(PyoObject):
 
         :Args:
 
-            x : float or PyoObject
+            x: float or PyoObject
                 new `x2` attribute.
 
         """
-        self._x2= x
+        pyoArgsAssert(self, "O", x)
+        self._x2 = x
         x, lmax = convertArgsToLists(x)
         [obj.setX2(wrap(x,i)) for i, obj in enumerate(self._base_objs)]
 
@@ -1173,14 +1201,14 @@ class Urn(PyoObject):
 
     :Args:
 
-        max : int, optional
+        max: int, optional
             Maximum value for the random generation. Defaults to 100.
-        freq : float or PyoObject, optional
+        freq: float or PyoObject, optional
             Polling frequency. Defaults to 1.
 
     .. note::
 
-        Urn will sends a trigger signal when the pool is empty.
+        Urn will send a trigger signal when the pool is empty.
         User can retreive the trigger streams by calling obj['trig'].
         Useful to synchronize other processes.
 
@@ -1194,6 +1222,7 @@ class Urn(PyoObject):
 
     """
     def __init__(self, max=100, freq=1., mul=1, add=0):
+        pyoArgsAssert(self, "iOOO", max, freq, mul, add)
         PyoObject.__init__(self, mul, add)
         self._max = max
         self._freq = freq
@@ -1210,10 +1239,11 @@ class Urn(PyoObject):
 
         :Args:
 
-            x : int
+            x: int
                 new `max` attribute.
 
         """
+        pyoArgsAssert(self, "i", x)
         self._max = x
         x, lmax = convertArgsToLists(x)
         [obj.setMax(wrap(x,i)) for i, obj in enumerate(self._base_objs)]
@@ -1224,10 +1254,11 @@ class Urn(PyoObject):
 
         :Args:
 
-            x : float or PyoObject
+            x: float or PyoObject
                 new `freq` attribute.
 
         """
+        pyoArgsAssert(self, "O", x)
         self._freq = x
         x, lmax = convertArgsToLists(x)
         [obj.setFreq(wrap(x,i)) for i, obj in enumerate(self._base_objs)]
@@ -1251,4 +1282,106 @@ class Urn(PyoObject):
         return self._freq
     @freq.setter
     def freq(self, x):
-        self.setFreq(x)
\ No newline at end of file
+        self.setFreq(x)
+
+class LogiMap(PyoObject):
+    """
+    Random generator based on the logistic map.
+
+    The logistic equation (sometimes called the Verhulst model or logistic
+    growth curve) is a model of population growth first published by Pierre
+    Verhulst (1845, 1847). The logistic map is a discrete quadratic recurrence
+    equation derived from the logistic equation that can be effectively used
+    as a number generator that exibit chaotic behavior. This object uses the
+    following equation:
+
+        x[n] = (r + 3) * x[n-1] * (1.0 - x[n-1])
+
+    where 'r' is the randomization factor between 0 and 1.
+
+    :Parent: :py:class:`PyoObject`
+
+    :Args:
+
+        chaos: float or PyoObject, optional
+            Randomization factor, 0.0 < chaos < 1.0. Defaults to 0.6.
+        freq: float or PyoObject, optional
+            Polling frequency. Defaults to 1.
+        init: float, optional
+            Initial value, 0.0 < init < 1.0. Defaults to 0.5.
+
+    .. note::
+
+        The method play() resets the internal state to the initial value.
+
+    >>> s = Server().boot()
+    >>> s.start()
+    >>> val = LogiMap([0.6,0.65], [4,8])
+    >>> mid = Round(Scale(val, 0, 1, [36,48], [72,84]))
+    >>> hz = Snap(mid, [0,2,4,5,7,9,11], 1)
+    >>> env = CosTable([(0,0), (32,1), (4064,1), (4096,0), (8192,0)])
+    >>> amp = TrigEnv(Change(val), table=env, dur=[.25,.125], mul=0.3)
+    >>> osc = RCOsc(hz, mul=amp).out()
+
+    """
+    def __init__(self, chaos=0.6, freq=1.0, init=0.5, mul=1, add=0):
+        pyoArgsAssert(self, "OOnOO", chaos, freq, init, mul, add)
+        PyoObject.__init__(self, mul, add)
+        self._chaos = chaos
+        self._freq = freq
+        chaos, freq, init, mul, add, lmax = convertArgsToLists(chaos, freq, init, mul, add)
+        self._base_objs = [LogiMap_base(wrap(chaos,i), wrap(freq,i), wrap(init,i), wrap(mul,i), wrap(add,i)) for i in range(lmax)]
+
+    def out(self, chnl=0, inc=1, dur=0, delay=0):
+        return self
+
+    def setChaos(self, x):
+        """
+        Replace the `chaos` attribute.
+
+        :Args:
+
+            x: float or PyoObject
+                new `chaos` attribute.
+
+        """
+        pyoArgsAssert(self, "O", x)
+        self._chaos = x
+        x, lmax = convertArgsToLists(x)
+        [obj.setChaos(wrap(x,i)) for i, obj in enumerate(self._base_objs)]
+
+    def setFreq(self, x):
+        """
+        Replace the `freq` attribute.
+
+        :Args:
+
+            x: float or PyoObject
+                new `freq` attribute.
+
+        """
+        pyoArgsAssert(self, "O", x)
+        self._freq = x
+        x, lmax = convertArgsToLists(x)
+        [obj.setFreq(wrap(x,i)) for i, obj in enumerate(self._base_objs)]
+
+    def ctrl(self, map_list=None, title=None, wxnoserver=False):
+        self._map_list = [SLMap(0.001, 0.999, 'lin', 'chaos', self._chaos),
+                          SLMap(0.1, 20., 'lin', 'freq', self._freq),
+                          SLMapMul(self._mul)]
+        PyoObject.ctrl(self, map_list, title, wxnoserver)
+
+    @property
+    def chaos(self):
+        """float or PyoObject. Randomization factor."""
+        return self._chaos
+    @chaos.setter
+    def chaos(self, x):
+        self.setChaos(x)
+    @property
+    def freq(self):
+        """float or PyoObject. Polling frequency."""
+        return self._freq
+    @freq.setter
+    def freq(self, x):
+        self.setFreq(x)
