@@ -1,4 +1,6 @@
 # -*- coding: utf-8 -*-
+from __future__ import print_function
+from __future__ import absolute_import
 """
 Copyright 2009-2015 Olivier Belanger
 
@@ -19,8 +21,8 @@ You should have received a copy of the GNU Lesser General Public
 License along with pyo.  If not, see <http://www.gnu.org/licenses/>.
 """
 import os, time
-from _core import *
-from _widgets import createServerGUI
+from ._core import *
+from ._widgets import createServerGUI
 
 ######################################################################
 ### Proxy of Server object
@@ -38,13 +40,13 @@ class Server(object):
 
     :Args:
 
-        sr : int, optional
+        sr: int, optional
             Sampling rate used by Portaudio and the Server to compute samples.
             Defaults to 44100.
-        nchnls : int, optional
+        nchnls: int, optional
             Number of output channels. The number of input channels will be the
             same if `ichnls` argument is not defined. Defaults to 2.
-        buffersize : int, optional
+        buffersize: int, optional
             Number of samples that Portaudio will request from the callback loop.
             Defaults to 256.
 
@@ -52,10 +54,10 @@ class Server(object):
             to compute) and on the latency of the system.
 
             Latency is `buffer size / sampling rate` in seconds.
-        duplex : int {0, 1}, optional
+        duplex: int {0, 1}, optional
             Input - output mode. 0 is output only and 1 is both ways.
             Defaults to 1.
-        audio : string {'portaudio', 'pa', 'jack', 'coreaudio', 'offline', 'offline_nb}, optional
+        audio: string {'portaudio', 'pa', 'jack', 'coreaudio', 'offline', 'offline_nb', 'embedded'}, optional
             Audio backend to use. 'pa' is equivalent to 'portaudio'. Default is 'portaudio'.
 
             'offline' save the audio output in a soundfile as fast as possible in blocking mode,
@@ -70,30 +72,37 @@ class Server(object):
 
             It is the responsibility of the user to make sure that the program doesn't exit before
             the computation is done.
-        jackname : string, optional
+
+            'embedded' should be used when pyo is embedded inside an host environment via its C api.
+
+            If 'jack' is selected but jackd is not already started when the program is executed, pyo
+            will ask jack to start in the background. Note that pyo never ask jack to close. It is
+            the user's responsability to manage the audio configuration of its system.
+        jackname: string, optional
             Name of jack client. Defaults to 'pyo'
-        ichnls : int, optional
+        ichnls: int, optional
             Number of input channels if different of output channels. If None (default), ichnls = nchnls.
 
     .. note::
 
         The following methods must be called **before** booting the server
 
-        - setInOutDevice(x) : Set both input and output devices. See `pa_list_devices()`.
-        - setInputDevice(x) : Set the audio input device number. See `pa_list_devices()`.
-        - setOutputDevice(x) : Set the audio output device number. See `pa_list_devices()`.
-        - setInputOffset(x) : Set the first physical input channel.
-        - setOutputOffset(x) : Set the first physical output channel.
-        - setInOutOffset(x) : Set the first physical input and output channels.
-        - setMidiInputDevice(x) : Set the MIDI input device number. See `pm_list_devices()`.
-        - setMidiOutputDevice(x) : Set the MIDI output device number. See `pm_list_devices()`.
-        - setSamplingRate(x) : Set the sampling rate used by the server.
-        - setBufferSize(x) : Set the buffer size used by the server.
-        - setNchnls(x) : Set the number of output (and input if `ichnls` = None) channels used by the server.
-        - setIchnls(x) : Set the number of input channels (if different of output channels) used by the server.
-        - setDuplex(x) : Set the duplex mode used by the server.
-        - setVerbosity(x) : Set the server's verbosity.
-        - reinit(sr, nchnls, buffersize, duplex, audio, jackname) : Reinit the server's settings.
+        - setInOutDevice(x): Set both input and output devices. See `pa_list_devices()`.
+        - setInputDevice(x): Set the audio input device number. See `pa_list_devices()`.
+        - setOutputDevice(x): Set the audio output device number. See `pa_list_devices()`.
+        - setInputOffset(x): Set the first physical input channel.
+        - setOutputOffset(x): Set the first physical output channel.
+        - setInOutOffset(x): Set the first physical input and output channels.
+        - setMidiInputDevice(x): Set the MIDI input device number. See `pm_list_devices()`.
+        - setMidiOutputDevice(x): Set the MIDI output device number. See `pm_list_devices()`.
+        - setSamplingRate(x): Set the sampling rate used by the server.
+        - setBufferSize(x): Set the buffer size used by the server.
+        - setNchnls(x): Set the number of output (and input if `ichnls` = None) channels used by the server.
+        - setIchnls(x): Set the number of input channels (if different of output channels) used by the server.
+        - setDuplex(x): Set the duplex mode used by the server.
+        - setVerbosity(x): Set the server's verbosity.
+        - reinit(sr, nchnls, buffersize, duplex, audio, jackname): Reinit the server's settings.
+        - deactivateMidi(): Deactivate Midi callback. 
 
     >>> # For an 8 channels server in duplex mode with
     >>> # a sampling rate of 48000 Hz and buffer size of 512
@@ -103,11 +112,11 @@ class Server(object):
     """
     def __init__(self, sr=44100, nchnls=2, buffersize=256, duplex=1,
                  audio='portaudio', jackname='pyo', ichnls=None):
-        if os.environ.has_key("PYO_SERVER_AUDIO") and "offline" not in audio and "embedded" not in audio:
+        if "PYO_SERVER_AUDIO" in os.environ and "offline" not in audio and "embedded" not in audio:
             audio = os.environ["PYO_SERVER_AUDIO"]
         self._time = time
         self._nchnls = nchnls
-        if ichnls == None:
+        if ichnls is None:
             self._ichnls = nchnls
         else:
             self._ichnls = ichnls
@@ -141,8 +150,9 @@ class Server(object):
             Same as in the __init__ method.
 
         """
+        self._gui_frame = None
         self._nchnls = nchnls
-        if ichnls == None:
+        if ichnls is None:
             self._ichnls = nchnls
         else:
             self._ichnls = ichnls
@@ -154,51 +164,76 @@ class Server(object):
         self._fileformat = 0
         self._sampletype = 0
         self._globalseed = 0
+        self._resampling = 1
         self._server.__init__(sr, nchnls, buffersize, duplex, audio, jackname, self._ichnls)
 
+    def setCallback(self, callback):
+        """
+        Register a custom process callback.
+
+        The function given as argument will be called every computation
+        block, just before the computation of the audio object tree.
+        Inside the callback, one can process the data of a table with
+        numpy calls for example.
+
+        """
+        if callable(callback):
+            self._server.setCallback(callback)
+
     def gui(self, locals=None, meter=True, timer=True, exit=True):
         """
         Show the server's user interface.
 
         :Args:
 
-            locals : locals namespace {locals(), None}, optional
+            locals: locals namespace {locals(), None}, optional
                 If locals() is given, the interface will show an interpreter extension,
                 giving a way to interact with the running script. Defaults to None.
-            meter : boolean, optinal
+            meter: boolean, optinal
                 If True, the interface will show a vumeter of the global output signal.
                 Defaults to True.
-            timer : boolean, optional
+            timer: boolean, optional
                 If True, the interface will show a clock of the current time.
                 Defaults to True.
-            exit : boolean, optional
+            exit: boolean, optional
                 If True, the python interpreter will exit when the 'Quit' button is pressed,
                 Otherwise, the GUI will be closed leaving the interpreter alive.
                 Defaults to True.
 
         """
-        f, win = createServerGUI(self._nchnls, self.start, self.stop, self.recstart, self.recstop,
-                                 self.setAmp, self.getIsStarted(), locals, self.shutdown, meter, timer, self._amp, exit)
+        self._gui_frame, win = createServerGUI(self._nchnls, self.start, self.stop,
+                                               self.recstart, self.recstop, self.setAmp,
+                                               self.getIsStarted(), locals, self.shutdown,
+                                               meter, timer, self._amp, exit)
         if meter:
-            self._server.setAmpCallable(f)
+            self._server.setAmpCallable(self._gui_frame)
         if timer:
-            self._server.setTimeCallable(f)
+            self._server.setTimeCallable(self._gui_frame)
         try:
             win.mainloop()
         except:
-            if win != None:
+            if win is not None:
                 win.MainLoop()
 
+    def closeGui(self):
+        """
+        Programmatically close the server's GUI.
+
+        """
+        if self._gui_frame is not None:
+            self._gui_frame.quit_from_code()
+            self._gui_frame = None
+
     def setTimeCallable(self, func):
         """
         Set a function callback that will receive the current time as argument.
-        
+
         The function will receive four integers in this format:
                 hours, minutes, seconds, milliseconds
 
         :Args:
-            
-            func : python callable
+
+            func: python callable
                 Python function or method to call with current time as argument.
 
         """
@@ -208,12 +243,12 @@ class Server(object):
     def setMeterCallable(self, func):
         """
         Set a function callback that will receive the current rms values as argument.
-        
+
         The function will receive a list containing the rms value for each audio channel.
 
         :Args:
-            
-            func : python callable
+
+            func: python callable
                 Python function or method to call with current rms values as argument.
 
         """
@@ -223,13 +258,13 @@ class Server(object):
     def setMeter(self, meter):
         """
         Registers a meter object to the server.
-        
+
         The object must have a method named `setRms`. This method will be called
         with the rms values of each audio channel as argument.
 
         :Args:
-            
-            meter : python object
+
+            meter: python object
                 Python object with a `setRms` method.
 
         """
@@ -241,7 +276,7 @@ class Server(object):
 
         :Args:
 
-            x : int
+            x: int
                 Number of the audio input and output devices.
 
         """
@@ -253,7 +288,7 @@ class Server(object):
 
         :Args:
 
-            x : int
+            x: int
                 Number of the audio device listed by Portaudio.
 
         """
@@ -265,7 +300,7 @@ class Server(object):
 
         :Args:
 
-            x : int
+            x: int
                 Number of the audio device listed by Portaudio.
 
         """
@@ -281,7 +316,7 @@ class Server(object):
 
         :Args:
 
-            x : int
+            x: int
                 Channel number.
 
         """
@@ -297,7 +332,7 @@ class Server(object):
 
         :Args:
 
-            x : int
+            x: int
                 Channel number.
 
         """
@@ -312,7 +347,7 @@ class Server(object):
 
         :Args:
 
-            x : int
+            x: int
                 Channel number.
 
         """
@@ -328,7 +363,7 @@ class Server(object):
 
         :Args:
 
-            x : int
+            x: int
                 Number of the Midi device listed by Portmidi.
 
         """
@@ -340,7 +375,7 @@ class Server(object):
 
         :Args:
 
-            x : int
+            x: int
                 Number of the Midi device listed by Portmidi.
 
         """
@@ -352,7 +387,7 @@ class Server(object):
 
         :Args:
 
-            x : int
+            x: int
                 New sampling rate, must be supported by the soundcard.
 
         """
@@ -364,7 +399,7 @@ class Server(object):
 
         :Args:
 
-            x : int
+            x: int
                 New buffer size.
 
         """
@@ -376,7 +411,7 @@ class Server(object):
 
         :Args:
 
-            x : int
+            x: int
                 New number of channels.
 
         """
@@ -389,7 +424,7 @@ class Server(object):
 
         :Args:
 
-            x : int
+            x: int
                 New number of input channels.
 
         """
@@ -402,7 +437,7 @@ class Server(object):
 
         :Args:
 
-            x : int {0 or 1}
+            x: int {0 or 1}
                 New mode. 0 is output only, 1 is both ways.
 
         """
@@ -414,7 +449,7 @@ class Server(object):
 
         :Args:
 
-            x : int
+            x: int
                 A sum of values to display different levels:
                     - 1 = error
                     - 2 = message
@@ -431,9 +466,9 @@ class Server(object):
 
         :Args:
 
-            xin : boolean
+            xin: boolean
                 Input Auto-connection switch. True is enabled (default) and False is disabled.
-            xout : boolean
+            xout: boolean
                 Output Auto-connection switch. True is enabled (default) and False is disabled.
 
         """
@@ -445,8 +480,11 @@ class Server(object):
 
         :Args:
 
-            ports : string or list of strings
-                Name of the Jack port(s) to auto-connect. Regular Expressions are allowed.
+            ports: list of list of strings
+                Name of the Jack ports to auto-connect to pyo input channels.
+                There must be exactly one list of port(s) for each pyo input channel.
+
+                [['ports', 'to', 'channel', '1'], ['ports', 'to', 'channel', '2'], ...]
 
         """
         ports, lmax = convertArgsToLists(ports)
@@ -458,8 +496,11 @@ class Server(object):
 
         :Args:
 
-            ports : string or list of strings
-                Name of the Jack port(s) to auto-connect. Regular Expressions are allowed.
+            ports: list of list of strings
+                Name of the Jack ports to auto-connect to pyo output channels.
+                There must be exactly one list of port(s) for each pyo output channel.
+
+                [['ports', 'to', 'channel', '1'], ['ports', 'to', 'channel', '2'], ...]
 
         """
         ports, lmax = convertArgsToLists(ports)
@@ -471,7 +512,7 @@ class Server(object):
 
         :Args:
 
-            x : int
+            x: int
                 A positive integer that will be used as the seed by random objects.
 
                 If zero, randoms will be seeded with the system clock current value.
@@ -487,7 +528,7 @@ class Server(object):
 
         :Args:
 
-            x : float
+            x: float
                 Starting time of the real-time processing.
 
         """
@@ -500,13 +541,59 @@ class Server(object):
 
         :Args:
 
-            x : float
+            x: float
                 New amplitude.
 
         """
         self._amp = x
         self._server.setAmp(x)
 
+    def beginResamplingBlock(self, x):
+        """
+        Starts a resampling block.
+
+        This factor must be a power-of-two. A positive value means
+        upsampling and a negative value means downsampling. After this
+        call, every PyoObject will be created with an internal sampling
+        rate and buffer size relative to the resampling factor. The method
+        `endResamplingBlock()` should be called at the end of the code
+        block using the resampling factor.
+
+        The `Resample` object can be used inside the resampling block to
+        perform up or down resampling of audio signal created before the
+        block.
+
+        :Args:
+
+            x: int, power-of-two
+                Resampling factor. Must be a power-of-two. A positive
+                value starts an upsampling block while a negative value
+                starts a downsampling block.
+
+        """
+        realx = x
+        x = abs(x)
+        if ((x & (x - 1)) == 0) and x != 0:
+            self._resampling = realx
+            self._server.beginResamplingBlock(realx)
+        else:
+            print("Resampling factor must be a power-of-two (positive or negative).")
+
+    def endResamplingBlock(self):
+        """
+        Ends a resampling block.
+
+        This call ends a code block using a sample rate different from
+        the current sampling rate of the system.
+
+        The `Resample` object can be used after the resampling blick to
+        perform up or down resampling of audio signal created inside the
+        block.
+
+        """
+        self._resampling = 1
+        self._server.endResamplingBlock()
+
     def shutdown(self):
         """
         Shut down and clear the server. This method will erase all objects
@@ -524,7 +611,7 @@ class Server(object):
 
         :Args:
 
-            newBuffer : bool
+            newBuffer: bool
                 Specify if the buffers need to be allocated or not. Useful to limit
                 the allocation of new buffers when the buffer size hasn't change.
 
@@ -553,20 +640,20 @@ class Server(object):
         """
         self._server.stop()
 
-    def recordOptions(self, dur=-1, filename=None, fileformat=0, sampletype=0):
+    def recordOptions(self, dur=-1, filename=None, fileformat=0, sampletype=0, quality=0.4):
         """
         Sets options for soundfile created by offline rendering or global recording.
 
         :Args:
 
-            dur : float
+            dur: float
                 Duration, in seconds, of the recorded file. Only used by
                 offline rendering. Must be positive. Defaults to -1.
-            filename : string
+            filename: string
                 Full path of the file to create. If None, a file called
                 `pyo_rec.wav` will be created in the user's home directory.
                 Defaults to None.
-            fileformat : int, optional
+            fileformat: int, optional
                 Format type of the audio file. This function will first try to
                 set the format from the filename extension.
 
@@ -579,7 +666,7 @@ class Server(object):
                     5. FLAC - FLAC lossless file format {.flac}
                     6. CAF - Core Audio File format {.caf}
                     7. OGG - Xiph OGG container {.ogg}
-            sampletype : int, optional
+            sampletype: int, optional
                 Bit depth encoding of the audio file.
 
                 SD2 and FLAC only support 16 or 24 bit int. Supported types are:
@@ -590,25 +677,29 @@ class Server(object):
                     4. 64 bits float
                     5. U-Law encoded
                     6. A-Law encoded
+            quality: float, optional
+                The encoding quality value, between 0.0 (lowest quality) and
+                1.0 (highest quality). This argument has an effect only with
+                FLAC and OGG compressed formats. Defaults to 0.4.
 
         """
 
         self._dur = dur
-        if filename == None:
+        if filename is None:
             filename = os.path.join(os.path.expanduser("~"), "pyo_rec.wav")
         self._filename = filename
         ext = filename.rsplit('.')
         if len(ext) >= 2:
             ext = ext[-1].lower()
-            if FILE_FORMATS.has_key(ext):
+            if ext in FILE_FORMATS:
                 fileformat = FILE_FORMATS[ext]
             else:
-                print 'Warning: Unknown file extension. Using fileformat value.'
+                print('Warning: Unknown file extension. Using fileformat value.')
         else:
-            print 'Warning: Filename has no extension. Using fileformat value.'
+            print('Warning: Filename has no extension. Using fileformat value.')
         self._fileformat = fileformat
         self._sampletype = sampletype
-        self._server.recordOptions(dur, filename, fileformat, sampletype)
+        self._server.recordOptions(dur, filename, fileformat, sampletype, quality)
 
     def recstart(self, filename=None):
         """
@@ -620,19 +711,19 @@ class Server(object):
 
         :Args:
 
-            filename : string, optional
+            filename: string, optional
                 Name of the file to be created. Defaults to None.
 
         """
-        if filename == None:
-            if self._filename != None:
+        if filename is None:
+            if self._filename is not None:
                 filename = self._filename
             else:
                 filename = os.path.join(os.path.expanduser("~"), "pyo_rec.wav")
         ext = filename.rsplit('.')
         if len(ext) >= 2:
             ext = ext[-1].lower()
-            if FILE_FORMATS.has_key(ext):
+            if ext in FILE_FORMATS:
                 fileformat = FILE_FORMATS[ext]
                 if fileformat != self._fileformat:
                     self._fileformat = fileformat
@@ -656,16 +747,16 @@ class Server(object):
 
         :Args:
 
-            pitch : int
+            pitch: int
                 Midi pitch, between 0 and 127.
-            velocity : int
+            velocity: int
                 Amplitude of the note, between 0 and 127. A note
                 with a velocity of 0 is equivalent to a note off.
-            channel : int, optional
+            channel: int, optional
                 The Midi channel, between 1 and 16, on which the
                 note is sent. A channel of 0 means all channels.
                 Defaults to 0.
-            timestamp : int, optional
+            timestamp: int, optional
                 The delay time, in milliseconds, before the note
                 is sent on the portmidi stream. A value of 0 means
                 to play the note now. Defaults to 0.
@@ -682,15 +773,15 @@ class Server(object):
 
         :Args:
 
-            pitch : int
+            pitch: int
                 Midi key pressed down, between 0 and 127.
-            velocity : int
+            velocity: int
                 Velocity of the pressure, between 0 and 127.
-            channel : int, optional
+            channel: int, optional
                 The Midi channel, between 1 and 16, on which the
                 note is sent. A channel of 0 means all channels.
                 Defaults to 0.
-            timestamp : int, optional
+            timestamp: int, optional
                 The delay time, in milliseconds, before the note
                 is sent on the portmidi stream. A value of 0 means
                 to play the note now. Defaults to 0.
@@ -707,15 +798,15 @@ class Server(object):
 
         :Args:
 
-            ctlnum : int
+            ctlnum: int
                 Controller number, between 0 and 127.
-            value : int
+            value: int
                 Value of the controller, between 0 and 127.
-            channel : int, optional
+            channel: int, optional
                 The Midi channel, between 1 and 16, on which the
                 message is sent. A channel of 0 means all channels.
                 Defaults to 0.
-            timestamp : int, optional
+            timestamp: int, optional
                 The delay time, in milliseconds, before the message
                 is sent on the portmidi stream. A value of 0 means
                 to play the message now. Defaults to 0.
@@ -732,13 +823,13 @@ class Server(object):
 
         :Args:
 
-            value : int
+            value: int
                 New program number, between 0 and 127.
-            channel : int, optional
+            channel: int, optional
                 The Midi channel, between 1 and 16, on which the
                 message is sent. A channel of 0 means all channels.
                 Defaults to 0.
-            timestamp : int, optional
+            timestamp: int, optional
                 The delay time, in milliseconds, before the message
                 is sent on the portmidi stream. A value of 0 means
                 to play the message now. Defaults to 0.
@@ -755,13 +846,13 @@ class Server(object):
 
         :Args:
 
-            value : int
+            value: int
                 Single greatest pressure value, between 0 and 127.
-            channel : int, optional
+            channel: int, optional
                 The Midi channel, between 1 and 16, on which the
                 message is sent. A channel of 0 means all channels.
                 Defaults to 0.
-            timestamp : int, optional
+            timestamp: int, optional
                 The delay time, in milliseconds, before the message
                 is sent on the portmidi stream. A value of 0 means
                 to play the message now. Defaults to 0.
@@ -778,14 +869,14 @@ class Server(object):
 
         :Args:
 
-            value : int
+            value: int
                 14 bits pitch bend value. 8192 is where there is no
                 bending, 0 is full down and 16383 is full up bending.
-            channel : int, optional
+            channel: int, optional
                 The Midi channel, between 1 and 16, on which the
                 message is sent. A channel of 0 means all channels.
                 Defaults to 0.
-            timestamp : int, optional
+            timestamp: int, optional
                 The delay time, in milliseconds, before the message
                 is sent on the portmidi stream. A value of 0 means
                 to play the message now. Defaults to 0.
@@ -793,6 +884,58 @@ class Server(object):
         value, channel, timestamp, lmax = convertArgsToLists(value, channel, timestamp)
         [self._server.bendout(wrap(value,i), wrap(channel,i), wrap(timestamp,i)) for i in range(lmax)]
 
+    def sysexout(self, msg, timestamp=0):
+        """
+        Send a system exclusive message to the selected midi output device.
+
+        Arguments can be list of values/messages to generate multiple events
+        in one call.
+
+        :Args:
+
+            msg: str
+                A valid system exclusive message as a string. The first byte
+                must be 0xf0 and the last one must be 0xf7.
+            timestamp: int, optional
+                The delay time, in milliseconds, before the message
+                is sent on the portmidi stream. A value of 0 means
+                to play the message now. Defaults to 0.
+        """
+        msg, timestamp, lmax = convertArgsToLists(msg, timestamp)
+        [self._server.sysexout(wrap(msg,i), wrap(timestamp,i)) for i in range(lmax)]
+
+    def addMidiEvent(self, status, data1=0, data2=0):
+        """
+        Add a MIDI event in the server processing loop.
+
+        This method can be used to  programatically simulate incomming
+        MIDI events. In an embedded framework (ie. pyo inside puredata,
+        openframeworks, etc.), this is useful to control a MIDI-driven
+        script from the host program. Arguments can be list of values to
+        generate multiple events in one call.
+
+        :Args:
+
+            status: int
+                The status byte, indicating the type of event and the
+                MIDI channel. Typical event type are:
+                    128 -> 143: Noteoff
+                    144 -> 159: Noteon
+                    176 -> 191: Control change
+                    192 -> 207: Program change
+                    209 -> 223: After touch
+                    224 -> 239: Pitch bend
+            data1: int, optional
+                The first data byte (pitch for a midi note, controller
+                number for a control change). Defaults to 0.
+            data2: int, optional
+                The second data byte (velocity for a midi note, value
+                for a control change). Defaults to 0.
+
+        """
+        status, data1, data2, lmax = convertArgsToLists(status, data1, data2)
+        [self._server.addMidiEvent(wrap(status,i), wrap(data1,i), wrap(data2,i)) for i in range(lmax)]
+
     def getStreams(self):
         """
         Return the list of streams loaded in the server.
@@ -842,6 +985,13 @@ class Server(object):
         """
         return self._server.getIsBooted()
 
+    def deactivateMidi(self):
+        """
+        Deactivate Midi callback. Must be called before the boot() method.
+
+        """
+        self._server.deactivateMidi()
+
     def getMidiActive(self):
         """
         Returns 1 if Midi callback is active, otherwise returns 0.
@@ -905,6 +1055,18 @@ class Server(object):
         """
         return self._server.getEmbedICallbackAddr()
 
+    def getCurrentTime(self):
+        """
+        Return the current time as a formatted string.
+        """
+        return self._server.getCurrentTime()
+
+    def getCurrentAmp(self):
+        """
+        Return the current amplitudes as a tuple of `nchnls` length.
+        """
+        return self._server.getCurrentAmp()
+
     @property
     def amp(self):
         """float. Overall amplitude."""
@@ -939,4 +1101,4 @@ class Server(object):
         if (type(x) == int):
             self.setGlobalSeed(x)
         else:
-            raise Exception("global seed must be an integer")
\ No newline at end of file
+            raise Exception("global seed must be an integer")
