@@ -19,6 +19,7 @@
  *************************************************************************/
 
 #include <Python.h>
+#include "py2to3.h"
 #include "structmember.h"
 #include "pyomodule.h"
 #include "streammodule.h"
@@ -34,6 +35,7 @@ typedef struct {
     MYFLT attack;
     MYFLT release;
     MYFLT duration;
+    MYFLT exp;
     double currentTime;
     MYFLT sampleToSec;
 } Fader;
@@ -50,8 +52,11 @@ static void Fader_internal_stop(Fader *self) {
 
 static void
 Fader_generate_auto(Fader *self) {
-    MYFLT val;
+    MYFLT val, iatt, irel;
     int i;
+    
+    iatt = 1.0 / self->attack;
+    irel = 1.0 / self->release;
 
     if (self->ended == 1) {
         Fader_internal_stop((Fader *)self);
@@ -60,26 +65,35 @@ Fader_generate_auto(Fader *self) {
 
     for (i=0; i<self->bufsize; i++) {
         if (self->currentTime <= self->attack)
-            val = self->currentTime / self->attack;
+            val = self->currentTime * iatt;
         else if (self->currentTime > self->duration) {
             val = 0.;
             self->ended = 1;
         }
         else if (self->currentTime >= (self->duration - self->release))
-            val = (self->duration - self->currentTime) / self->release;
+            val = (self->duration - self->currentTime) * irel;
         else
             val = 1.;
 
         self->data[i] = val;
         self->currentTime += self->sampleToSec;
     }
+
+    if (self->exp != 1.0) {
+        for (i=0; i<self->bufsize; i++) {
+            self->data[i] = MYPOW(self->data[i], self->exp);
+        }
+    }
 }
 
 static void
 Fader_generate_wait(Fader *self) {
-    MYFLT val;
+    MYFLT val, iatt, irel;
     int i;
 
+    iatt = 1.0 / self->attack;
+    irel = 1.0 / self->release;
+
     if (self->fademode == 1 && self->currentTime > self->release) {
         Fader_internal_stop((Fader *)self);
         return;
@@ -89,20 +103,26 @@ Fader_generate_wait(Fader *self) {
         if (self->fademode == 0) {
 
             if (self->currentTime <= self->attack)
-                val = self->currentTime / self->attack;
+                val = self->currentTime * iatt;
             else
                 val = 1.;
             self->topValue = val;
         }
         else {
             if (self->currentTime <= self->release)
-                val = (1. - self->currentTime / self->release) * self->topValue;
+                val = (1. - self->currentTime * irel) * self->topValue;
             else
                 val = 0.;
         }
         self->data[i] = val;
         self->currentTime += self->sampleToSec;
     }
+    
+    if (self->exp != 1.0) {
+        for (i=0; i<self->bufsize; i++) {
+            self->data[i] = MYPOW(self->data[i], self->exp);
+        }
+    }
 }
 
 static void Fader_postprocessing_ii(Fader *self) { POST_PROCESSING_II };
@@ -126,7 +146,7 @@ Fader_setProcMode(Fader *self)
     else
         self->proc_func_ptr = Fader_generate_auto;
 
-	switch (muladdmode) {
+    switch (muladdmode) {
         case 0:
             self->muladd_func_ptr = Fader_postprocessing_ii;
             break;
@@ -183,7 +203,7 @@ Fader_dealloc(Fader* self)
 {
     pyo_DEALLOC
     Fader_clear(self);
-    self->ob_type->tp_free((PyObject*)self);
+    Py_TYPE(self)->tp_free((PyObject*)self);
 }
 
 static PyObject *
@@ -202,6 +222,7 @@ Fader_new(PyTypeObject *type, PyObject *args, PyObject *kwds)
     self->attack = 0.01;
     self->release = 0.1;
     self->duration = 0.0;
+    self->exp = 1.0;
     self->currentTime = 0.0;
 
     INIT_OBJECT_COMMON
@@ -295,6 +316,16 @@ Fader_setDur(Fader *self, PyObject *arg)
     return Py_None;
 }
 
+static PyObject *
+Fader_setExp(Fader *self, PyObject *arg)
+{
+    MYFLT tmp = PyFloat_AsDouble(arg);
+    if (tmp > 0.0)
+        self->exp = tmp;
+    Py_INCREF(Py_None);
+    return Py_None;
+}
+
 static PyMemberDef Fader_members[] = {
 {"server", T_OBJECT_EX, offsetof(Fader, server), 0, "Pyo server."},
 {"stream", T_OBJECT_EX, offsetof(Fader, stream), 0, "Stream object."},
@@ -314,6 +345,7 @@ static PyMethodDef Fader_methods[] = {
 {"setFadein", (PyCFunction)Fader_setFadein, METH_O, "Sets fadein time in seconds."},
 {"setFadeout", (PyCFunction)Fader_setFadeout, METH_O, "Sets fadeout time in seconds."},
 {"setDur", (PyCFunction)Fader_setDur, METH_O, "Sets duration in seconds (0 means wait for stop method to start fadeout)."},
+{"setExp", (PyCFunction)Fader_setExp, METH_O, "Sets the exponent factor for exponential envelope."},
 {"setDiv", (PyCFunction)Fader_setDiv, METH_O, "Sets inverse mul factor."},
 {NULL}  /* Sentinel */
 };
@@ -322,7 +354,7 @@ static PyNumberMethods Fader_as_number = {
 (binaryfunc)Fader_add,                      /*nb_add*/
 (binaryfunc)Fader_sub,                 /*nb_subtract*/
 (binaryfunc)Fader_multiply,                 /*nb_multiply*/
-(binaryfunc)Fader_div,                   /*nb_divide*/
+INITIALIZE_NB_DIVIDE_ZERO               /*nb_divide*/
 0,                /*nb_remainder*/
 0,                   /*nb_divmod*/
 0,                   /*nb_power*/
@@ -336,16 +368,16 @@ static PyNumberMethods Fader_as_number = {
 0,              /*nb_and*/
 0,              /*nb_xor*/
 0,               /*nb_or*/
-0,                                          /*nb_coerce*/
+INITIALIZE_NB_COERCE_ZERO                   /*nb_coerce*/
 0,                       /*nb_int*/
 0,                      /*nb_long*/
 0,                     /*nb_float*/
-0,                       /*nb_oct*/
-0,                       /*nb_hex*/
+INITIALIZE_NB_OCT_ZERO   /*nb_oct*/
+INITIALIZE_NB_HEX_ZERO   /*nb_hex*/
 (binaryfunc)Fader_inplace_add,              /*inplace_add*/
 (binaryfunc)Fader_inplace_sub,         /*inplace_subtract*/
 (binaryfunc)Fader_inplace_multiply,         /*inplace_multiply*/
-(binaryfunc)Fader_inplace_div,           /*inplace_divide*/
+INITIALIZE_NB_IN_PLACE_DIVIDE_ZERO        /*inplace_divide*/
 0,        /*inplace_remainder*/
 0,           /*inplace_power*/
 0,       /*inplace_lshift*/
@@ -354,15 +386,14 @@ static PyNumberMethods Fader_as_number = {
 0,      /*inplace_xor*/
 0,       /*inplace_or*/
 0,             /*nb_floor_divide*/
-0,              /*nb_true_divide*/
+(binaryfunc)Fader_div,                       /*nb_true_divide*/
 0,     /*nb_inplace_floor_divide*/
-0,      /*nb_inplace_true_divide*/
+(binaryfunc)Fader_inplace_div,                       /*nb_inplace_true_divide*/
 0,                     /* nb_index */
 };
 
 PyTypeObject FaderType = {
-PyObject_HEAD_INIT(NULL)
-0,                         /*ob_size*/
+PyVarObject_HEAD_INIT(NULL, 0)
 "_pyo.Fader_base",         /*tp_name*/
 sizeof(Fader),         /*tp_basicsize*/
 0,                         /*tp_itemsize*/
@@ -370,7 +401,7 @@ sizeof(Fader),         /*tp_basicsize*/
 0,                         /*tp_print*/
 0,                         /*tp_getattr*/
 0,                         /*tp_setattr*/
-0,                         /*tp_compare*/
+0,                         /*tp_as_async (tp_compare in Python 2)*/
 0,                         /*tp_repr*/
 &Fader_as_number,             /*tp_as_number*/
 0,                         /*tp_as_sequence*/
@@ -412,6 +443,7 @@ typedef struct {
     MYFLT sustain;
     MYFLT release;
     MYFLT duration;
+    MYFLT exp;
     double currentTime;
     MYFLT sampleToSec;
 } Adsr;
@@ -431,6 +463,9 @@ Adsr_generate_auto(Adsr *self) {
     MYFLT val, invatt, invdec, invrel;
     int i;
 
+    if (self->currentTime > self->duration)
+        Adsr_internal_stop((Adsr *)self);
+
     invatt = 1.0 / self->attack;
     invdec = 1.0 / self->decay;
     invrel = 1.0 / self->release;
@@ -440,10 +475,8 @@ Adsr_generate_auto(Adsr *self) {
             val = self->currentTime * invatt;
         else if (self->currentTime <= (self->attack + self->decay))
             val = (self->decay - (self->currentTime - self->attack)) * invdec * (1. - self->sustain) + self->sustain;
-        else if (self->currentTime > self->duration) {
+        else if (self->currentTime > self->duration)
             val = 0.;
-            Adsr_internal_stop((Adsr *)self);
-        }
         else if (self->currentTime >= (self->duration - self->release))
             val = (self->duration - self->currentTime) * invrel * self->sustain;
         else
@@ -452,6 +485,12 @@ Adsr_generate_auto(Adsr *self) {
         self->data[i] = val;
         self->currentTime += self->sampleToSec;
     }
+
+    if (self->exp != 1.0) {
+        for (i=0; i<self->bufsize; i++) {
+            self->data[i] = MYPOW(self->data[i], self->exp);
+        }
+    }
 }
 
 static void
@@ -487,6 +526,12 @@ Adsr_generate_wait(Adsr *self) {
         self->data[i] = val;
         self->currentTime += self->sampleToSec;
     }
+
+    if (self->exp != 1.0) {
+        for (i=0; i<self->bufsize; i++) {
+            self->data[i] = MYPOW(self->data[i], self->exp);
+        }
+    }
 }
 
 static void Adsr_postprocessing_ii(Adsr *self) { POST_PROCESSING_II };
@@ -510,7 +555,7 @@ Adsr_setProcMode(Adsr *self)
     else
         self->proc_func_ptr = Adsr_generate_auto;
 
-	switch (muladdmode) {
+    switch (muladdmode) {
         case 0:
             self->muladd_func_ptr = Adsr_postprocessing_ii;
             break;
@@ -567,7 +612,7 @@ Adsr_dealloc(Adsr* self)
 {
     pyo_DEALLOC
     Adsr_clear(self);
-    self->ob_type->tp_free((PyObject*)self);
+    Py_TYPE(self)->tp_free((PyObject*)self);
 }
 
 static PyObject *
@@ -587,6 +632,7 @@ Adsr_new(PyTypeObject *type, PyObject *args, PyObject *kwds)
     self->sustain = 0.707;
     self->release = 0.1;
     self->duration = 0.0;
+    self->exp = 1.0;
     self->currentTime = 0.0;
 
     INIT_OBJECT_COMMON
@@ -669,9 +715,11 @@ static PyObject * Adsr_inplace_div(Adsr *self, PyObject *arg) { INPLACE_DIV };
 static PyObject *
 Adsr_setAttack(Adsr *self, PyObject *arg)
 {
-    self->attack = PyFloat_AsDouble(PyNumber_Float(arg));
-    if (self->attack < 0.000001)
-        self->attack = 0.000001;
+	if (PyNumber_Check(arg)) {
+        self->attack = PyFloat_AsDouble(arg);
+        if (self->attack < 0.000001)
+            self->attack = 0.000001;
+    }
     Py_INCREF(Py_None);
     return Py_None;
 }
@@ -679,9 +727,11 @@ Adsr_setAttack(Adsr *self, PyObject *arg)
 static PyObject *
 Adsr_setDecay(Adsr *self, PyObject *arg)
 {
-    self->decay = PyFloat_AsDouble(PyNumber_Float(arg));
-    if (self->decay < 0.000001)
-        self->decay = 0.000001;
+	if (PyNumber_Check(arg)) {
+        self->decay = PyFloat_AsDouble(arg);
+        if (self->decay < 0.000001)
+            self->decay = 0.000001;
+    }
     Py_INCREF(Py_None);
     return Py_None;
 }
@@ -689,11 +739,13 @@ Adsr_setDecay(Adsr *self, PyObject *arg)
 static PyObject *
 Adsr_setSustain(Adsr *self, PyObject *arg)
 {
-    self->sustain = PyFloat_AsDouble(PyNumber_Float(arg));
-    if (self->sustain < 0.0)
-        self->sustain = 0.0;
-    else if (self->sustain > 1.0)
-        self->sustain = 1.0;
+	if (PyNumber_Check(arg)) {
+        self->sustain = PyFloat_AsDouble(arg);
+        if (self->sustain < 0.0)
+            self->sustain = 0.0;
+        else if (self->sustain > 1.0)
+            self->sustain = 1.0;
+    }
     Py_INCREF(Py_None);
     return Py_None;
 }
@@ -701,9 +753,11 @@ Adsr_setSustain(Adsr *self, PyObject *arg)
 static PyObject *
 Adsr_setRelease(Adsr *self, PyObject *arg)
 {
-    self->release = PyFloat_AsDouble(PyNumber_Float(arg));
-    if (self->release < 0.000001)
-        self->release = 0.000001;
+	if (PyNumber_Check(arg)) {
+        self->release = PyFloat_AsDouble(arg);
+        if (self->release < 0.000001)
+            self->release = 0.000001;
+    }
     Py_INCREF(Py_None);
     return Py_None;
 }
@@ -711,7 +765,20 @@ Adsr_setRelease(Adsr *self, PyObject *arg)
 static PyObject *
 Adsr_setDur(Adsr *self, PyObject *arg)
 {
-    self->duration = PyFloat_AsDouble(PyNumber_Float(arg));
+	if (PyNumber_Check(arg)) 
+        self->duration = PyFloat_AsDouble(arg);
+    Py_INCREF(Py_None);
+    return Py_None;
+}
+
+static PyObject *
+Adsr_setExp(Adsr *self, PyObject *arg)
+{
+	if (PyNumber_Check(arg)) {
+        MYFLT tmp = PyFloat_AsDouble(arg);
+        if (tmp > 0.0)
+            self->exp = tmp;
+    }
     Py_INCREF(Py_None);
     return Py_None;
 }
@@ -737,6 +804,7 @@ static PyMethodDef Adsr_methods[] = {
 {"setSustain", (PyCFunction)Adsr_setSustain, METH_O, "Sets attack time in seconds."},
 {"setRelease", (PyCFunction)Adsr_setRelease, METH_O, "Sets release time in seconds."},
 {"setDur", (PyCFunction)Adsr_setDur, METH_O, "Sets duration in seconds (0 means wait for stop method to start fadeout)."},
+{"setExp", (PyCFunction)Adsr_setExp, METH_O, "Sets the exponent factor for exponential envelope."},
 {"setDiv", (PyCFunction)Adsr_setDiv, METH_O, "Sets inverse mul factor."},
 {NULL}  /* Sentinel */
 };
@@ -745,7 +813,7 @@ static PyNumberMethods Adsr_as_number = {
 (binaryfunc)Adsr_add,                      /*nb_add*/
 (binaryfunc)Adsr_sub,                 /*nb_subtract*/
 (binaryfunc)Adsr_multiply,                 /*nb_multiply*/
-(binaryfunc)Adsr_div,                   /*nb_divide*/
+INITIALIZE_NB_DIVIDE_ZERO               /*nb_divide*/
 0,                /*nb_remainder*/
 0,                   /*nb_divmod*/
 0,                   /*nb_power*/
@@ -759,16 +827,16 @@ static PyNumberMethods Adsr_as_number = {
 0,              /*nb_and*/
 0,              /*nb_xor*/
 0,               /*nb_or*/
-0,                                          /*nb_coerce*/
+INITIALIZE_NB_COERCE_ZERO                   /*nb_coerce*/
 0,                       /*nb_int*/
 0,                      /*nb_long*/
 0,                     /*nb_float*/
-0,                       /*nb_oct*/
-0,                       /*nb_hex*/
+INITIALIZE_NB_OCT_ZERO   /*nb_oct*/
+INITIALIZE_NB_HEX_ZERO   /*nb_hex*/
 (binaryfunc)Adsr_inplace_add,              /*inplace_add*/
 (binaryfunc)Adsr_inplace_sub,         /*inplace_subtract*/
 (binaryfunc)Adsr_inplace_multiply,         /*inplace_multiply*/
-(binaryfunc)Adsr_inplace_div,           /*inplace_divide*/
+INITIALIZE_NB_IN_PLACE_DIVIDE_ZERO        /*inplace_divide*/
 0,        /*inplace_remainder*/
 0,           /*inplace_power*/
 0,       /*inplace_lshift*/
@@ -777,15 +845,14 @@ static PyNumberMethods Adsr_as_number = {
 0,      /*inplace_xor*/
 0,       /*inplace_or*/
 0,             /*nb_floor_divide*/
-0,              /*nb_true_divide*/
+(binaryfunc)Adsr_div,                       /*nb_true_divide*/
 0,     /*nb_inplace_floor_divide*/
-0,      /*nb_inplace_true_divide*/
+(binaryfunc)Adsr_inplace_div,                       /*nb_inplace_true_divide*/
 0,                     /* nb_index */
 };
 
 PyTypeObject AdsrType = {
-PyObject_HEAD_INIT(NULL)
-0,                         /*ob_size*/
+PyVarObject_HEAD_INIT(NULL, 0)
 "_pyo.Adsr_base",         /*tp_name*/
 sizeof(Adsr),         /*tp_basicsize*/
 0,                         /*tp_itemsize*/
@@ -793,7 +860,7 @@ sizeof(Adsr),         /*tp_basicsize*/
 0,                         /*tp_print*/
 0,                         /*tp_getattr*/
 0,                         /*tp_setattr*/
-0,                         /*tp_compare*/
+0,                         /*tp_as_async (tp_compare in Python 2)*/
 0,                         /*tp_repr*/
 &Adsr_as_number,             /*tp_as_number*/
 0,                         /*tp_as_sequence*/
@@ -840,6 +907,7 @@ typedef struct {
     int newlist;
     int loop;
     int listsize;
+    int okToPause;
 } Linseg;
 
 static void
@@ -852,8 +920,8 @@ Linseg_convert_pointslist(Linseg *self) {
     self->times = (MYFLT *)realloc(self->times, self->listsize * sizeof(MYFLT));
     for (i=0; i<self->listsize; i++) {
         tup = PyList_GET_ITEM(self->pointslist, i);
-        self->times[i] = PyFloat_AsDouble(PyNumber_Float(PyTuple_GET_ITEM(tup, 0)));
-        self->targets[i] = PyFloat_AsDouble(PyNumber_Float(PyTuple_GET_ITEM(tup, 1)));
+        self->times[i] = PyFloat_AsDouble(PyTuple_GET_ITEM(tup, 0));
+        self->targets[i] = PyFloat_AsDouble(PyTuple_GET_ITEM(tup, 1));
     }
 }
 
@@ -867,6 +935,7 @@ Linseg_reinit(Linseg *self) {
     self->currentValue = self->targets[0];
     self->which = 0;
     self->flag = 1;
+    self->okToPause = 1;
 }
 
 static void
@@ -882,6 +951,7 @@ Linseg_generate(Linseg *self) {
                         Linseg_reinit((Linseg *)self);
                     else {
                         self->flag = 0;
+                        self->okToPause = 0;
                         self->currentValue = self->targets[self->which-1];
                     }
                 }
@@ -981,7 +1051,7 @@ Linseg_dealloc(Linseg* self)
     free(self->targets);
     free(self->times);
     Linseg_clear(self);
-    self->ob_type->tp_free((PyObject*)self);
+    Py_TYPE(self)->tp_free((PyObject*)self);
 }
 
 static PyObject *
@@ -994,6 +1064,7 @@ Linseg_new(PyTypeObject *type, PyObject *args, PyObject *kwds)
 
     self->loop = 0;
     self->newlist = 1;
+    self->okToPause = 0;
 	self->modebuffer[0] = 0;
 	self->modebuffer[1] = 0;
 
@@ -1049,7 +1120,20 @@ static PyObject * Linseg_play(Linseg *self, PyObject *args, PyObject *kwds)
     PLAY
 };
 
-static PyObject * Linseg_stop(Linseg *self) { STOP };
+static PyObject * Linseg_stop(Linseg *self) 
+{ 
+    self->okToPause = 0;
+    STOP 
+};
+
+static PyObject * Linseg_pause(Linseg *self) 
+{ 
+    if (self->okToPause == 1)
+        self->flag = 1 - self->flag;
+
+    Py_INCREF(Py_None);
+    return Py_None;
+};
 
 static PyObject * Linseg_multiply(Linseg *self, PyObject *arg) { MULTIPLY };
 static PyObject * Linseg_inplace_multiply(Linseg *self, PyObject *arg) { INPLACE_MULTIPLY };
@@ -1086,10 +1170,7 @@ Linseg_setList(Linseg *self, PyObject *value)
 static PyObject *
 Linseg_setLoop(Linseg *self, PyObject *arg)
 {
-	if (arg == NULL) {
-		Py_INCREF(Py_None);
-		return Py_None;
-	}
+    ASSERT_ARG_NOT_NULL
 
     self->loop = PyInt_AsLong(arg);
 
@@ -1111,6 +1192,7 @@ static PyMethodDef Linseg_methods[] = {
 {"_getStream", (PyCFunction)Linseg_getStream, METH_NOARGS, "Returns stream object."},
 {"play", (PyCFunction)Linseg_play, METH_VARARGS|METH_KEYWORDS, "Starts computing without sending sound to soundcard."},
 {"stop", (PyCFunction)Linseg_stop, METH_NOARGS, "Starts fadeout and stops computing."},
+{"pause", (PyCFunction)Linseg_pause, METH_NOARGS, "Toggles between play and stop without reset."},
 {"setList", (PyCFunction)Linseg_setList, METH_O, "Sets target points list."},
 {"setLoop", (PyCFunction)Linseg_setLoop, METH_O, "Sets looping mode."},
 {"setMul", (PyCFunction)Linseg_setMul, METH_O, "Sets Linseg mul factor."},
@@ -1124,7 +1206,7 @@ static PyNumberMethods Linseg_as_number = {
 (binaryfunc)Linseg_add,                      /*nb_add*/
 (binaryfunc)Linseg_sub,                 /*nb_subtract*/
 (binaryfunc)Linseg_multiply,                 /*nb_multiply*/
-(binaryfunc)Linseg_div,                   /*nb_divide*/
+INITIALIZE_NB_DIVIDE_ZERO               /*nb_divide*/
 0,                /*nb_remainder*/
 0,                   /*nb_divmod*/
 0,                   /*nb_power*/
@@ -1138,16 +1220,16 @@ static PyNumberMethods Linseg_as_number = {
 0,              /*nb_and*/
 0,              /*nb_xor*/
 0,               /*nb_or*/
-0,                                          /*nb_coerce*/
+INITIALIZE_NB_COERCE_ZERO                   /*nb_coerce*/
 0,                       /*nb_int*/
 0,                      /*nb_long*/
 0,                     /*nb_float*/
-0,                       /*nb_oct*/
-0,                       /*nb_hex*/
+INITIALIZE_NB_OCT_ZERO   /*nb_oct*/
+INITIALIZE_NB_HEX_ZERO   /*nb_hex*/
 (binaryfunc)Linseg_inplace_add,              /*inplace_add*/
 (binaryfunc)Linseg_inplace_sub,         /*inplace_subtract*/
 (binaryfunc)Linseg_inplace_multiply,         /*inplace_multiply*/
-(binaryfunc)Linseg_inplace_div,           /*inplace_divide*/
+INITIALIZE_NB_IN_PLACE_DIVIDE_ZERO        /*inplace_divide*/
 0,        /*inplace_remainder*/
 0,           /*inplace_power*/
 0,       /*inplace_lshift*/
@@ -1156,15 +1238,14 @@ static PyNumberMethods Linseg_as_number = {
 0,      /*inplace_xor*/
 0,       /*inplace_or*/
 0,             /*nb_floor_divide*/
-0,              /*nb_true_divide*/
+(binaryfunc)Linseg_div,                       /*nb_true_divide*/
 0,     /*nb_inplace_floor_divide*/
-0,      /*nb_inplace_true_divide*/
+(binaryfunc)Linseg_inplace_div,                       /*nb_inplace_true_divide*/
 0,                     /* nb_index */
 };
 
 PyTypeObject LinsegType = {
-PyObject_HEAD_INIT(NULL)
-0,                         /*ob_size*/
+PyVarObject_HEAD_INIT(NULL, 0)
 "_pyo.Linseg_base",         /*tp_name*/
 sizeof(Linseg),         /*tp_basicsize*/
 0,                         /*tp_itemsize*/
@@ -1172,7 +1253,7 @@ sizeof(Linseg),         /*tp_basicsize*/
 0,                         /*tp_print*/
 0,                         /*tp_getattr*/
 0,                         /*tp_setattr*/
-0,                         /*tp_compare*/
+0,                         /*tp_as_async (tp_compare in Python 2)*/
 0,                         /*tp_repr*/
 &Linseg_as_number,             /*tp_as_number*/
 0,                         /*tp_as_sequence*/
@@ -1226,6 +1307,7 @@ typedef struct {
     double exp_tmp;
     int inverse;
     int inverse_tmp;
+    int okToPause;
 } Expseg;
 
 static void
@@ -1238,8 +1320,8 @@ Expseg_convert_pointslist(Expseg *self) {
     self->times = (MYFLT *)realloc(self->times, self->listsize * sizeof(MYFLT));
     for (i=0; i<self->listsize; i++) {
         tup = PyList_GET_ITEM(self->pointslist, i);
-        self->times[i] = PyFloat_AsDouble(PyNumber_Float(PyTuple_GET_ITEM(tup, 0)));
-        self->targets[i] = PyFloat_AsDouble(PyNumber_Float(PyTuple_GET_ITEM(tup, 1)));
+        self->times[i] = PyFloat_AsDouble(PyTuple_GET_ITEM(tup, 0));
+        self->targets[i] = PyFloat_AsDouble(PyTuple_GET_ITEM(tup, 1));
     }
 }
 
@@ -1255,6 +1337,7 @@ Expseg_reinit(Expseg *self) {
     self->flag = 1;
     self->exp = self->exp_tmp;
     self->inverse = self->inverse_tmp;
+    self->okToPause = 1;
 }
 
 static void
@@ -1272,6 +1355,7 @@ Expseg_generate(Expseg *self) {
                     else {
                         self->flag = 0;
                         self->currentValue = self->targets[self->which-1];
+                        self->okToPause = 0;
                     }
                 }
                 else {
@@ -1382,7 +1466,7 @@ Expseg_dealloc(Expseg* self)
     free(self->targets);
     free(self->times);
     Expseg_clear(self);
-    self->ob_type->tp_free((PyObject*)self);
+    Py_TYPE(self)->tp_free((PyObject*)self);
 }
 
 static PyObject *
@@ -1397,6 +1481,7 @@ Expseg_new(PyTypeObject *type, PyObject *args, PyObject *kwds)
     self->newlist = 1;
     self->exp = self->exp_tmp = 10;
     self->inverse = self->inverse_tmp = 1;
+    self->okToPause = 0;
 	self->modebuffer[0] = 0;
 	self->modebuffer[1] = 0;
 
@@ -1452,7 +1537,20 @@ static PyObject * Expseg_play(Expseg *self, PyObject *args, PyObject *kwds)
     PLAY
 };
 
-static PyObject * Expseg_stop(Expseg *self) { STOP };
+static PyObject * Expseg_stop(Expseg *self)
+{ 
+    self->okToPause = 0;
+    STOP 
+};
+
+static PyObject * Expseg_pause(Expseg *self) 
+{ 
+    if (self->okToPause == 1)
+        self->flag = 1 - self->flag;
+
+    Py_INCREF(Py_None);
+    return Py_None;
+};
 
 static PyObject * Expseg_multiply(Expseg *self, PyObject *arg) { MULTIPLY };
 static PyObject * Expseg_inplace_multiply(Expseg *self, PyObject *arg) { INPLACE_MULTIPLY };
@@ -1489,10 +1587,7 @@ Expseg_setList(Expseg *self, PyObject *value)
 static PyObject *
 Expseg_setLoop(Expseg *self, PyObject *arg)
 {
-	if (arg == NULL) {
-		Py_INCREF(Py_None);
-		return Py_None;
-	}
+    ASSERT_ARG_NOT_NULL
 
     self->loop = PyInt_AsLong(arg);
 
@@ -1503,12 +1598,10 @@ Expseg_setLoop(Expseg *self, PyObject *arg)
 static PyObject *
 Expseg_setExp(Expseg *self, PyObject *arg)
 {
-	if (arg == NULL) {
-		Py_INCREF(Py_None);
-		return Py_None;
-	}
+    ASSERT_ARG_NOT_NULL
 
-    self->exp_tmp = PyFloat_AsDouble(PyNumber_Float(arg));
+	if (PyNumber_Check(arg))
+        self->exp_tmp = PyFloat_AsDouble(arg);
 
     Py_INCREF(Py_None);
     return Py_None;
@@ -1517,12 +1610,10 @@ Expseg_setExp(Expseg *self, PyObject *arg)
 static PyObject *
 Expseg_setInverse(Expseg *self, PyObject *arg)
 {
-	if (arg == NULL) {
-		Py_INCREF(Py_None);
-		return Py_None;
-	}
+    ASSERT_ARG_NOT_NULL
 
-    self->inverse_tmp = PyInt_AsLong(PyNumber_Int(arg));
+	if (PyNumber_Check(arg))
+        self->inverse_tmp = PyInt_AsLong(PyNumber_Int(arg));
 
     Py_INCREF(Py_None);
     return Py_None;
@@ -1542,6 +1633,7 @@ static PyMethodDef Expseg_methods[] = {
     {"_getStream", (PyCFunction)Expseg_getStream, METH_NOARGS, "Returns stream object."},
     {"play", (PyCFunction)Expseg_play, METH_VARARGS|METH_KEYWORDS, "Starts computing without sending sound to soundcard."},
     {"stop", (PyCFunction)Expseg_stop, METH_NOARGS, "Starts fadeout and stops computing."},
+    {"pause", (PyCFunction)Expseg_pause, METH_NOARGS, "Toggles between play and stop without reset."},
     {"setList", (PyCFunction)Expseg_setList, METH_O, "Sets target points list."},
     {"setLoop", (PyCFunction)Expseg_setLoop, METH_O, "Sets looping mode."},
     {"setExp", (PyCFunction)Expseg_setExp, METH_O, "Sets exponent factor."},
@@ -1557,7 +1649,7 @@ static PyNumberMethods Expseg_as_number = {
     (binaryfunc)Expseg_add,                      /*nb_add*/
     (binaryfunc)Expseg_sub,                 /*nb_subtract*/
     (binaryfunc)Expseg_multiply,                 /*nb_multiply*/
-    (binaryfunc)Expseg_div,                   /*nb_divide*/
+    INITIALIZE_NB_DIVIDE_ZERO               /*nb_divide*/
     0,                /*nb_remainder*/
     0,                   /*nb_divmod*/
     0,                   /*nb_power*/
@@ -1571,16 +1663,16 @@ static PyNumberMethods Expseg_as_number = {
     0,              /*nb_and*/
     0,              /*nb_xor*/
     0,               /*nb_or*/
-    0,                                          /*nb_coerce*/
+    INITIALIZE_NB_COERCE_ZERO                   /*nb_coerce*/
     0,                       /*nb_int*/
     0,                      /*nb_long*/
     0,                     /*nb_float*/
-    0,                       /*nb_oct*/
-    0,                       /*nb_hex*/
+    INITIALIZE_NB_OCT_ZERO   /*nb_oct*/
+    INITIALIZE_NB_HEX_ZERO   /*nb_hex*/
     (binaryfunc)Expseg_inplace_add,              /*inplace_add*/
     (binaryfunc)Expseg_inplace_sub,         /*inplace_subtract*/
     (binaryfunc)Expseg_inplace_multiply,         /*inplace_multiply*/
-    (binaryfunc)Expseg_inplace_div,           /*inplace_divide*/
+    INITIALIZE_NB_IN_PLACE_DIVIDE_ZERO        /*inplace_divide*/
     0,        /*inplace_remainder*/
     0,           /*inplace_power*/
     0,       /*inplace_lshift*/
@@ -1589,15 +1681,14 @@ static PyNumberMethods Expseg_as_number = {
     0,      /*inplace_xor*/
     0,       /*inplace_or*/
     0,             /*nb_floor_divide*/
-    0,              /*nb_true_divide*/
+    (binaryfunc)Expseg_div,                       /*nb_true_divide*/
     0,     /*nb_inplace_floor_divide*/
-    0,      /*nb_inplace_true_divide*/
+    (binaryfunc)Expseg_inplace_div,                       /*nb_inplace_true_divide*/
     0,                     /* nb_index */
 };
 
 PyTypeObject ExpsegType = {
-    PyObject_HEAD_INIT(NULL)
-    0,                         /*ob_size*/
+    PyVarObject_HEAD_INIT(NULL, 0)
     "_pyo.Expseg_base",         /*tp_name*/
     sizeof(Expseg),         /*tp_basicsize*/
     0,                         /*tp_itemsize*/
@@ -1605,7 +1696,7 @@ PyTypeObject ExpsegType = {
     0,                         /*tp_print*/
     0,                         /*tp_getattr*/
     0,                         /*tp_setattr*/
-    0,                         /*tp_compare*/
+    0,                         /*tp_as_async (tp_compare in Python 2)*/
     0,                         /*tp_repr*/
     &Expseg_as_number,             /*tp_as_number*/
     0,                         /*tp_as_sequence*/
@@ -1635,4 +1726,4 @@ PyTypeObject ExpsegType = {
     0,      /* tp_init */
     0,                         /* tp_alloc */
     Expseg_new,                 /* tp_new */
-};
\ No newline at end of file
+};
