@@ -19,6 +19,7 @@
  *************************************************************************/
 
 #include <Python.h>
+#include "py2to3.h"
 #include "structmember.h"
 #include <math.h>
 #include "pyomodule.h"
@@ -34,6 +35,7 @@ typedef struct {
     Stream *sharp_stream;
     int modebuffer[4]; // need at least 2 slots for mul & add
     int wavetype;
+    MYFLT oneOverSr;
     MYFLT oneOverPiOverTwo;
     MYFLT srOverFour;
     MYFLT srOverEight;
@@ -48,18 +50,22 @@ static void
 LFO_generates_ii(LFO *self) {
     MYFLT val, inc, freq, sharp, pointer, numh;
     MYFLT v1, v2, inc2, fade;
+    MYFLT sharp2 = 0.0;
     int i, maxHarms;
 
     freq = PyFloat_AS_DOUBLE(self->freq);
-    if (freq <= 0) {
-        return;
-    }
+    if (freq < 0.00001)
+        freq = 0.00001;
+    else if (freq > self->srOverFour)
+        freq = self->srOverFour;
+
     sharp = PyFloat_AS_DOUBLE(self->sharp);
     if (sharp < 0.0)
         sharp = 0.0;
     else if (sharp > 1.0)
         sharp = 1.0;
-    inc = freq / self->sr;
+
+    inc = freq * self->oneOverSr;
 
     switch (self->wavetype) {
         case 0: /* Saw up */
@@ -189,14 +195,15 @@ LFO_generates_ii(LFO *self) {
             }
             break;
         case 7: /* Sine-mod */
-            inc2 = inc * sharp;
+            inc2 = inc * sharp * 0.99;
+            sharp2 = sharp * 0.5;
             for (i=0; i<self->bufsize; i++) {
                 self->modPointerPos += inc2;
                 if (self->modPointerPos < 0)
                     self->modPointerPos += 1.0;
                 else if (self->modPointerPos >= 1)
                     self->modPointerPos -= 1.0;
-                val = (0.5 * MYCOS(TWOPI*self->modPointerPos) + 0.5) * MYSIN(TWOPI*self->pointerPos);
+                val = ((sharp2 * MYCOS(TWOPI*self->modPointerPos) + sharp2) + (1.0 - sharp)) * MYSIN(TWOPI*self->pointerPos);
                 self->data[i] = val;
                 self->pointerPos += inc;
                 if (self->pointerPos < 0)
@@ -214,12 +221,10 @@ static void
 LFO_generates_ai(LFO *self) {
     MYFLT val, inc, freq, sharp, pointer, numh;
     MYFLT v1, v2, inc2, fade;
+    MYFLT sharp2 = 0.0;
     int i, maxHarms;
 
     MYFLT *fr = Stream_getData((Stream *)self->freq_stream);
-    if (fr[0] <= 0) {
-        return;
-    }
     sharp = PyFloat_AS_DOUBLE(self->sharp);
     if (sharp < 0.0)
         sharp = 0.0;
@@ -230,7 +235,11 @@ LFO_generates_ai(LFO *self) {
         case 0: /* Saw up */
             for (i=0; i<self->bufsize; i++) {
                 freq = fr[i];
-                inc = freq / self->sr;
+                if (freq < 0.00001)
+                    freq = 0.00001;
+                else if (freq > self->srOverFour)
+                    freq = self->srOverFour;
+                inc = freq * self->oneOverSr;
                 maxHarms = (int)(self->srOverFour/freq);
                 numh = sharp * 46.0 + 4.0;
                 if (numh > maxHarms)
@@ -248,7 +257,11 @@ LFO_generates_ai(LFO *self) {
         case 1: /* Saw down */
             for (i=0; i<self->bufsize; i++) {
                 freq = fr[i];
-                inc = freq / self->sr;
+                if (freq < 0.00001)
+                    freq = 0.00001;
+                else if (freq > self->srOverFour)
+                    freq = self->srOverFour;
+                inc = freq * self->oneOverSr;
                 maxHarms = (int)(self->srOverFour/freq);
                 numh = sharp * 46.0 + 4.0;
                 if (numh > maxHarms)
@@ -266,7 +279,11 @@ LFO_generates_ai(LFO *self) {
         case 2: /* Square */
             for (i=0; i<self->bufsize; i++) {
                 freq = fr[i];
-                inc = freq / self->sr;
+                if (freq < 0.00001)
+                    freq = 0.00001;
+                else if (freq > self->srOverFour)
+                    freq = self->srOverFour;
+                inc = freq * self->oneOverSr;
                 maxHarms = (int)(self->srOverEight/freq);
                 numh = sharp * 46.0 + 4.0;
                 if (numh > maxHarms)
@@ -283,13 +300,17 @@ LFO_generates_ai(LFO *self) {
         case 3: /* Triangle */
             for (i=0; i<self->bufsize; i++) {
                 freq = fr[i];
-                inc = freq / self->sr;
+                if (freq < 0.00001)
+                    freq = 0.00001;
+                else if (freq > self->srOverFour)
+                    freq = self->srOverFour;
+                inc = freq * self->oneOverSr;
                 maxHarms = (int)(self->srOverFour/freq);
                 if ((sharp * 36.0) > maxHarms)
                     numh = (MYFLT)(maxHarms / 36.0);
                 else
                     numh = sharp;
-                v1 = MYTAN(MYSIN(TWOPI*self->pointerPos));
+                v1 = MYTAN(MYSIN(TWOPI*self->pointerPos)) * self->oneOverPiOverTwo;
                 pointer = self->pointerPos + 0.25;
                 if (pointer > 1.0)
                     pointer -= 1.0;
@@ -306,7 +327,11 @@ LFO_generates_ai(LFO *self) {
         case 4: /* Pulse */
             for (i=0; i<self->bufsize; i++) {
                 freq = fr[i];
-                inc = freq / self->sr;
+                if (freq < 0.00001)
+                    freq = 0.00001;
+                else if (freq > self->srOverFour)
+                    freq = self->srOverFour;
+                inc = freq * self->oneOverSr;
                 maxHarms = (int)(self->srOverEight/freq);
                 numh = MYFLOOR(sharp * 46.0 + 4.0);
                 if (numh > maxHarms)
@@ -325,7 +350,11 @@ LFO_generates_ai(LFO *self) {
         case 5: /* Bi-Pulse */
             for (i=0; i<self->bufsize; i++) {
                 freq = fr[i];
-                inc = freq / self->sr;
+                if (freq < 0.00001)
+                    freq = 0.00001;
+                else if (freq > self->srOverFour)
+                    freq = self->srOverFour;
+                inc = freq * self->oneOverSr;
                 maxHarms = (int)(self->srOverEight/freq);
                 numh = MYFLOOR(sharp * 46.0 + 4.0);
                 if (numh > maxHarms)
@@ -345,7 +374,11 @@ LFO_generates_ai(LFO *self) {
             numh = 1.0 - sharp;
             for (i=0; i<self->bufsize; i++) {
                 freq = fr[i];
-                inc = freq / self->sr;
+                if (freq < 0.00001)
+                    freq = 0.00001;
+                else if (freq > self->srOverFour)
+                    freq = self->srOverFour;
+                inc = freq * self->oneOverSr;
                 inc2 = 1.0 / (int)(1.0 / inc * numh);
                 self->pointerPos += inc;
                 if (self->pointerPos < 0)
@@ -368,16 +401,21 @@ LFO_generates_ai(LFO *self) {
             }
             break;
         case 7: /* Sine-mod */
+            sharp2 = sharp * 0.5;
             for (i=0; i<self->bufsize; i++) {
                 freq = fr[i];
-                inc = freq / self->sr;
-                inc2 = inc * sharp;
+                if (freq < 0.00001)
+                    freq = 0.00001;
+                else if (freq > self->srOverFour)
+                    freq = self->srOverFour;
+                inc = freq * self->oneOverSr;
+                inc2 = inc * sharp * 0.99;
                 self->modPointerPos += inc2;
                 if (self->modPointerPos < 0)
                     self->modPointerPos += 1.0;
                 else if (self->modPointerPos >= 1)
                     self->modPointerPos -= 1.0;
-                val = (0.5 * MYCOS(TWOPI*self->modPointerPos) + 0.5) * MYSIN(TWOPI*self->pointerPos);
+                val = ((sharp2 * MYCOS(TWOPI*self->modPointerPos) + sharp2) + (1.0 - sharp)) * MYSIN(TWOPI*self->pointerPos);
                 self->data[i] = val;
                 self->pointerPos += inc;
                 if (self->pointerPos < 0)
@@ -395,14 +433,17 @@ static void
 LFO_generates_ia(LFO *self) {
     MYFLT val, inc, freq, sharp, pointer, numh;
     MYFLT v1, v2, inc2, fade;
+    MYFLT sharp2 = 0.0;
     int i, maxHarms;
 
     freq = PyFloat_AS_DOUBLE(self->freq);
-    if (freq <= 0) {
-        return;
-    }
+    if (freq < 0.00001)
+        freq = 0.00001;
+    else if (freq > self->srOverFour)
+        freq = self->srOverFour;
+    inc = freq * self->oneOverSr;
+
     MYFLT *sh = Stream_getData((Stream *)self->sharp_stream);
-    inc = freq / self->sr;
 
     switch (self->wavetype) {
         case 0: /* Saw up */
@@ -479,7 +520,7 @@ LFO_generates_ia(LFO *self) {
                     numh = (MYFLT)(maxHarms / 36.0);
                 else
                     numh = sharp;
-                v1 = MYTAN(MYSIN(TWOPI*self->pointerPos));
+                v1 = MYTAN(MYSIN(TWOPI*self->pointerPos)) * self->oneOverPiOverTwo;
                 pointer = self->pointerPos + 0.25;
                 if (pointer > 1.0)
                     pointer -= 1.0;
@@ -573,13 +614,14 @@ LFO_generates_ia(LFO *self) {
                     sharp = 0.0;
                 else if (sharp > 1.0)
                     sharp = 1.0;
-                inc2 = inc * sharp;
+                inc2 = inc * sharp * 0.99;
+                sharp2 = sharp * 0.5;
                 self->modPointerPos += inc2;
                 if (self->modPointerPos < 0)
                     self->modPointerPos += 1.0;
                 else if (self->modPointerPos >= 1)
                     self->modPointerPos -= 1.0;
-                val = (0.5 * MYCOS(TWOPI*self->modPointerPos) + 0.5) * MYSIN(TWOPI*self->pointerPos);
+                val = ((sharp2 * MYCOS(TWOPI*self->modPointerPos) + sharp2) + (1.0 - sharp)) * MYSIN(TWOPI*self->pointerPos);
                 self->data[i] = val;
                 self->pointerPos += inc;
                 if (self->pointerPos < 0)
@@ -597,12 +639,10 @@ static void
 LFO_generates_aa(LFO *self) {
     MYFLT val, inc, freq, sharp, pointer, numh;
     MYFLT v1, v2, inc2, fade;
+    MYFLT sharp2 = 0.0;
     int i, maxHarms;
 
     MYFLT *fr = Stream_getData((Stream *)self->freq_stream);
-    if (fr[0] <= 0) {
-        return;
-    }
     MYFLT *sh = Stream_getData((Stream *)self->sharp_stream);
 
     switch (self->wavetype) {
@@ -614,7 +654,11 @@ LFO_generates_aa(LFO *self) {
                 else if (sharp > 1.0)
                     sharp = 1.0;
                 freq = fr[i];
-                inc = freq / self->sr;
+                if (freq < 0.00001)
+                    freq = 0.00001;
+                else if (freq > self->srOverFour)
+                    freq = self->srOverFour;
+                inc = freq * self->oneOverSr;
                 maxHarms = (int)(self->srOverFour/freq);
                 numh = sharp * 46.0 + 4.0;
                 if (numh > maxHarms)
@@ -637,7 +681,11 @@ LFO_generates_aa(LFO *self) {
                 else if (sharp > 1.0)
                     sharp = 1.0;
                 freq = fr[i];
-                inc = freq / self->sr;
+                if (freq < 0.00001)
+                    freq = 0.00001;
+                else if (freq > self->srOverFour)
+                    freq = self->srOverFour;
+                inc = freq * self->oneOverSr;
                 maxHarms = (int)(self->srOverFour/freq);
                 numh = sharp * 46.0 + 4.0;
                 if (numh > maxHarms)
@@ -660,7 +708,11 @@ LFO_generates_aa(LFO *self) {
                 else if (sharp > 1.0)
                     sharp = 1.0;
                 freq = fr[i];
-                inc = freq / self->sr;
+                if (freq < 0.00001)
+                    freq = 0.00001;
+                else if (freq > self->srOverFour)
+                    freq = self->srOverFour;
+                inc = freq * self->oneOverSr;
                 maxHarms = (int)(self->srOverEight/freq);
                 numh = sharp * 46.0 + 4.0;
                 if (numh > maxHarms)
@@ -682,13 +734,17 @@ LFO_generates_aa(LFO *self) {
                 else if (sharp > 1.0)
                     sharp = 1.0;
                 freq = fr[i];
-                inc = freq / self->sr;
+                if (freq < 0.00001)
+                    freq = 0.00001;
+                else if (freq > self->srOverFour)
+                    freq = self->srOverFour;
+                inc = freq * self->oneOverSr;
                 maxHarms = (int)(self->srOverFour/freq);
                 if ((sharp * 36.0) > maxHarms)
                     numh = (MYFLT)(maxHarms / 36.0);
                 else
                     numh = sharp;
-                v1 = MYTAN(MYSIN(TWOPI*self->pointerPos));
+                v1 = MYTAN(MYSIN(TWOPI*self->pointerPos)) * self->oneOverPiOverTwo;
                 pointer = self->pointerPos + 0.25;
                 if (pointer > 1.0)
                     pointer -= 1.0;
@@ -710,7 +766,11 @@ LFO_generates_aa(LFO *self) {
                 else if (sharp > 1.0)
                     sharp = 1.0;
                 freq = fr[i];
-                inc = freq / self->sr;
+                if (freq < 0.00001)
+                    freq = 0.00001;
+                else if (freq > self->srOverFour)
+                    freq = self->srOverFour;
+                inc = freq * self->oneOverSr;
                 maxHarms = (int)(self->srOverEight/freq);
                 numh = MYFLOOR(sharp * 46.0 + 4.0);
                 if (numh > maxHarms)
@@ -734,7 +794,11 @@ LFO_generates_aa(LFO *self) {
                 else if (sharp > 1.0)
                     sharp = 1.0;
                 freq = fr[i];
-                inc = freq / self->sr;
+                if (freq < 0.00001)
+                    freq = 0.00001;
+                else if (freq > self->srOverFour)
+                    freq = self->srOverFour;
+                inc = freq * self->oneOverSr;
                 maxHarms = (int)(self->srOverEight/freq);
                 numh = MYFLOOR(sharp * 46.0 + 4.0);
                 if (numh > maxHarms)
@@ -759,7 +823,11 @@ LFO_generates_aa(LFO *self) {
                     sharp = 1.0;
                 numh = 1.0 - sharp;
                 freq = fr[i];
-                inc = freq / self->sr;
+                if (freq < 0.00001)
+                    freq = 0.00001;
+                else if (freq > self->srOverFour)
+                    freq = self->srOverFour;
+                inc = freq * self->oneOverSr;
                 inc2 = 1.0 / (int)(1.0 / inc * numh);
                 self->pointerPos += inc;
                 if (self->pointerPos < 0)
@@ -789,14 +857,19 @@ LFO_generates_aa(LFO *self) {
                 else if (sharp > 1.0)
                     sharp = 1.0;
                 freq = fr[i];
-                inc = freq / self->sr;
-                inc2 = inc * sharp;
+                if (freq < 0.00001)
+                    freq = 0.00001;
+                else if (freq > self->srOverFour)
+                    freq = self->srOverFour;
+                inc = freq * self->oneOverSr;
+                inc2 = inc * sharp * 0.99;
+                sharp2 = sharp * 0.5;
                 self->modPointerPos += inc2;
                 if (self->modPointerPos < 0)
                     self->modPointerPos += 1.0;
                 else if (self->modPointerPos >= 1)
                     self->modPointerPos -= 1.0;
-                val = (0.5 * MYCOS(TWOPI*self->modPointerPos) + 0.5) * MYSIN(TWOPI*self->pointerPos);
+                val = ((sharp2 * MYCOS(TWOPI*self->modPointerPos) + sharp2) + (1.0 - sharp)) * MYSIN(TWOPI*self->pointerPos);
                 self->data[i] = val;
                 self->pointerPos += inc;
                 if (self->pointerPos < 0)
@@ -906,7 +979,7 @@ LFO_dealloc(LFO* self)
 {
     pyo_DEALLOC
     LFO_clear(self);
-    self->ob_type->tp_free((PyObject*)self);
+    Py_TYPE(self)->tp_free((PyObject*)self);
 }
 
 static PyObject *
@@ -931,6 +1004,7 @@ LFO_new(PyTypeObject *type, PyObject *args, PyObject *kwds)
 
     INIT_OBJECT_COMMON
 
+    self->oneOverSr = 1.0 / (MYFLT)self->sr;
     self->srOverFour = (MYFLT)self->sr * 0.25;
     self->srOverEight = (MYFLT)self->sr * 0.125;
     Stream_setFunctionPtr(self->stream, LFO_compute_next_data_frame);
@@ -1112,7 +1186,7 @@ static PyNumberMethods LFO_as_number = {
     (binaryfunc)LFO_add,                         /*nb_add*/
     (binaryfunc)LFO_sub,                         /*nb_subtract*/
     (binaryfunc)LFO_multiply,                    /*nb_multiply*/
-    (binaryfunc)LFO_div,                                              /*nb_divide*/
+    INITIALIZE_NB_DIVIDE_ZERO                       /*nb_divide*/
     0,                                              /*nb_remainder*/
     0,                                              /*nb_divmod*/
     0,                                              /*nb_power*/
@@ -1126,16 +1200,16 @@ static PyNumberMethods LFO_as_number = {
     0,                                              /*nb_and*/
     0,                                              /*nb_xor*/
     0,                                              /*nb_or*/
-    0,                                              /*nb_coerce*/
+    INITIALIZE_NB_COERCE_ZERO                       /*nb_coerce*/
     0,                                              /*nb_int*/
     0,                                              /*nb_long*/
     0,                                              /*nb_float*/
-    0,                                              /*nb_oct*/
-    0,                                              /*nb_hex*/
+    INITIALIZE_NB_OCT_ZERO                          /*nb_oct*/
+    INITIALIZE_NB_HEX_ZERO                          /*nb_hex*/
     (binaryfunc)LFO_inplace_add,                 /*inplace_add*/
     (binaryfunc)LFO_inplace_sub,                 /*inplace_subtract*/
     (binaryfunc)LFO_inplace_multiply,            /*inplace_multiply*/
-    (binaryfunc)LFO_inplace_div,                                              /*inplace_divide*/
+    INITIALIZE_NB_IN_PLACE_DIVIDE_ZERO                                           /*inplace_divide*/
     0,                                              /*inplace_remainder*/
     0,                                              /*inplace_power*/
     0,                                              /*inplace_lshift*/
@@ -1144,15 +1218,14 @@ static PyNumberMethods LFO_as_number = {
     0,                                              /*inplace_xor*/
     0,                                              /*inplace_or*/
     0,                                              /*nb_floor_divide*/
-    0,                                              /*nb_true_divide*/
+    (binaryfunc)LFO_div,                       /*nb_true_divide*/
     0,                                              /*nb_inplace_floor_divide*/
-    0,                                              /*nb_inplace_true_divide*/
+    (binaryfunc)LFO_inplace_div,                       /*nb_inplace_true_divide*/
     0,                                              /* nb_index */
 };
 
 PyTypeObject LFOType = {
-    PyObject_HEAD_INIT(NULL)
-    0,                                              /*ob_size*/
+    PyVarObject_HEAD_INIT(NULL, 0)
     "_pyo.LFO_base",                                   /*tp_name*/
     sizeof(LFO),                                 /*tp_basicsize*/
     0,                                              /*tp_itemsize*/
@@ -1160,7 +1233,7 @@ PyTypeObject LFOType = {
     0,                                              /*tp_print*/
     0,                                              /*tp_getattr*/
     0,                                              /*tp_setattr*/
-    0,                                              /*tp_compare*/
+    0,                                              /*tp_as_async (tp_compare in Python 2)*/
     0,                                              /*tp_repr*/
     &LFO_as_number,                              /*tp_as_number*/
     0,                                              /*tp_as_sequence*/
