@@ -19,6 +19,7 @@
  *************************************************************************/
 
 #include <Python.h>
+#include "py2to3.h"
 #include "structmember.h"
 #include "pyomodule.h"
 #include "servermodule.h"
@@ -35,7 +36,7 @@
 static void
 MatrixStream_dealloc(MatrixStream* self)
 {
-    self->ob_type->tp_free((PyObject*)self);
+    Py_TYPE(self)->tp_free((PyObject*)self);
 }
 
 static PyObject *
@@ -122,8 +123,7 @@ MatrixStream_setHeight(MatrixStream *self, int size)
 }
 
 PyTypeObject MatrixStreamType = {
-PyObject_HEAD_INIT(NULL)
-0, /*ob_size*/
+PyVarObject_HEAD_INIT(NULL, 0)
 "_pyo.MatrixStream", /*tp_name*/
 sizeof(MatrixStream), /*tp_basicsize*/
 0, /*tp_itemsize*/
@@ -131,7 +131,7 @@ sizeof(MatrixStream), /*tp_basicsize*/
 0, /*tp_print*/
 0, /*tp_getattr*/
 0, /*tp_setattr*/
-0, /*tp_compare*/
+0, /*tp_as_async (tp_compare in Python 2)*/
 0, /*tp_repr*/
 0, /*tp_as_number*/
 0, /*tp_as_sequence*/
@@ -220,7 +220,7 @@ NewMatrix_dealloc(NewMatrix* self)
     }
     free(self->data);
     NewMatrix_clear(self);
-    self->ob_type->tp_free((PyObject*)self);
+    Py_TYPE(self)->tp_free((PyObject*)self);
 }
 
 static PyObject *
@@ -335,6 +335,25 @@ NewMatrix_getViewData(NewMatrix *self)
     return matrix;
 };
 
+static PyObject *
+NewMatrix_getImageData(NewMatrix *self)
+{
+    int i, j, w3, index;
+    char value;
+    char matrix[self->width*self->height*3];
+
+    w3 = self->width * 3;
+    for(i=0; i<self->height; i++) {
+        for (j=0; j<self->width; j++) {
+            value = (char)(self->data[i][j]*128+128);
+            index = i * w3 + j * 3;
+            matrix[index] = matrix[index+1] = matrix[index+2] = value;
+        }
+    }
+
+    return PyByteArray_FromStringAndSize(matrix, self->width*self->height*3);
+};
+
 static PyObject *
 NewMatrix_setMatrix(NewMatrix *self, PyObject *value)
 {
@@ -361,7 +380,7 @@ NewMatrix_setMatrix(NewMatrix *self, PyObject *value)
     for(i=0; i<self->height; i++) {
         innerlist = PyList_GetItem(value, i);
         for (j=0; j<self->width; j++) {
-            self->data[i][j] = PyFloat_AS_DOUBLE(PyNumber_Float(PyList_GET_ITEM(innerlist, j)));
+            self->data[i][j] = PyFloat_AsDouble(PyList_GET_ITEM(innerlist, j));
         }
     }
 
@@ -403,6 +422,7 @@ static PyMethodDef NewMatrix_methods[] = {
 {"getServer", (PyCFunction)NewMatrix_getServer, METH_NOARGS, "Returns server object."},
 {"getData", (PyCFunction)NewMatrix_getData, METH_NOARGS, "Returns a list of matrix samples."},
 {"getViewData", (PyCFunction)NewMatrix_getViewData, METH_NOARGS, "Returns a list of matrix samples normalized between 0 and 256 ."},
+{"getImageData", (PyCFunction)NewMatrix_getImageData, METH_NOARGS, "Returns a list of matrix samples in tuple of 3 ints normalized between 0 and 256 ."},
 {"getMatrixStream", (PyCFunction)NewMatrix_getMatrixStream, METH_NOARGS, "Returns matrixstream object created by this matrix."},
 {"setMatrix", (PyCFunction)NewMatrix_setMatrix, METH_O, "Sets the matrix from a list of list of floats (must be the same size as the object size)."},
 {"setData", (PyCFunction)NewMatrix_setData, METH_O, "Sets the matrix from a list of list of floats (resizes the matrix)."},
@@ -418,8 +438,7 @@ static PyMethodDef NewMatrix_methods[] = {
 };
 
 PyTypeObject NewMatrixType = {
-PyObject_HEAD_INIT(NULL)
-0,                         /*ob_size*/
+PyVarObject_HEAD_INIT(NULL, 0)
 "_pyo.NewMatrix_base",         /*tp_name*/
 sizeof(NewMatrix),         /*tp_basicsize*/
 0,                         /*tp_itemsize*/
@@ -427,7 +446,7 @@ sizeof(NewMatrix),         /*tp_basicsize*/
 0,                         /*tp_print*/
 0,                         /*tp_getattr*/
 0,                         /*tp_setattr*/
-0,                         /*tp_compare*/
+0,                         /*tp_as_async (tp_compare in Python 2)*/
 0,                         /*tp_repr*/
 0,                         /*tp_as_number*/
 0,                         /*tp_as_sequence*/
@@ -566,7 +585,7 @@ MatrixRec_dealloc(MatrixRec* self)
     pyo_DEALLOC
     free(self->trigsBuffer);
     MatrixRec_clear(self);
-    self->ob_type->tp_free((PyObject*)self);
+    Py_TYPE(self)->tp_free((PyObject*)self);
 }
 
 static PyObject *
@@ -638,10 +657,7 @@ MatrixRec_setMatrix(MatrixRec *self, PyObject *arg)
 {
 	PyObject *tmp;
 
-	if (arg == NULL) {
-		Py_INCREF(Py_None);
-		return Py_None;
-	}
+    ASSERT_ARG_NOT_NULL
 
 	tmp = arg;
     Py_INCREF(tmp);
@@ -672,8 +688,7 @@ static PyMethodDef MatrixRec_methods[] = {
 };
 
 PyTypeObject MatrixRecType = {
-    PyObject_HEAD_INIT(NULL)
-    0,                         /*ob_size*/
+    PyVarObject_HEAD_INIT(NULL, 0)
     "_pyo.MatrixRec_base",         /*tp_name*/
     sizeof(MatrixRec),         /*tp_basicsize*/
     0,                         /*tp_itemsize*/
@@ -681,7 +696,7 @@ PyTypeObject MatrixRecType = {
     0,                         /*tp_print*/
     0,                         /*tp_getattr*/
     0,                         /*tp_setattr*/
-    0,                         /*tp_compare*/
+    0,                         /*tp_as_async (tp_compare in Python 2)*/
     0,                         /*tp_repr*/
     0,             /*tp_as_number*/
     0,                         /*tp_as_sequence*/
@@ -777,7 +792,7 @@ MatrixRecLoop_dealloc(MatrixRecLoop* self)
     pyo_DEALLOC
     free(self->trigsBuffer);
     MatrixRecLoop_clear(self);
-    self->ob_type->tp_free((PyObject*)self);
+    Py_TYPE(self)->tp_free((PyObject*)self);
 }
 
 static PyObject *
@@ -831,10 +846,7 @@ MatrixRecLoop_setMatrix(MatrixRecLoop *self, PyObject *arg)
 {
 	PyObject *tmp;
 
-	if (arg == NULL) {
-		Py_INCREF(Py_None);
-		return Py_None;
-	}
+    ASSERT_ARG_NOT_NULL
 
 	tmp = arg;
     Py_INCREF(tmp);
@@ -865,8 +877,7 @@ static PyMethodDef MatrixRecLoop_methods[] = {
 };
 
 PyTypeObject MatrixRecLoopType = {
-    PyObject_HEAD_INIT(NULL)
-    0,                         /*ob_size*/
+    PyVarObject_HEAD_INIT(NULL, 0)
     "_pyo.MatrixRecLoop_base",         /*tp_name*/
     sizeof(MatrixRecLoop),         /*tp_basicsize*/
     0,                         /*tp_itemsize*/
@@ -874,7 +885,7 @@ PyTypeObject MatrixRecLoopType = {
     0,                         /*tp_print*/
     0,                         /*tp_getattr*/
     0,                         /*tp_setattr*/
-    0,                         /*tp_compare*/
+    0,                         /*tp_as_async (tp_compare in Python 2)*/
     0,                         /*tp_repr*/
     0,             /*tp_as_number*/
     0,                         /*tp_as_sequence*/
@@ -991,7 +1002,7 @@ MatrixMorph_dealloc(MatrixMorph* self)
     pyo_DEALLOC
     free(self->buffer);
     MatrixMorph_clear(self);
-    self->ob_type->tp_free((PyObject*)self);
+    Py_TYPE(self)->tp_free((PyObject*)self);
 }
 
 static PyObject *
@@ -1043,10 +1054,7 @@ MatrixMorph_setMatrix(MatrixMorph *self, PyObject *arg)
 {
 	PyObject *tmp;
 
-	if (arg == NULL) {
-		Py_INCREF(Py_None);
-		return Py_None;
-	}
+    ASSERT_ARG_NOT_NULL
 
 	tmp = arg;
     Py_INCREF(tmp);
@@ -1060,10 +1068,7 @@ MatrixMorph_setMatrix(MatrixMorph *self, PyObject *arg)
 static PyObject *
 MatrixMorph_setSources(MatrixMorph *self, PyObject *arg)
 {
-    if (arg == NULL) {
-        PyErr_SetString(PyExc_TypeError, "Cannot delete the list attribute.");
-        return PyInt_FromLong(-1);
-    }
+    ASSERT_ARG_NOT_NULL
 
     if (! PyList_Check(arg)) {
         PyErr_SetString(PyExc_TypeError, "The amplitude list attribute value must be a list.");
@@ -1098,8 +1103,7 @@ static PyMethodDef MatrixMorph_methods[] = {
 };
 
 PyTypeObject MatrixMorphType = {
-    PyObject_HEAD_INIT(NULL)
-    0,                         /*ob_size*/
+    PyVarObject_HEAD_INIT(NULL, 0)
     "_pyo.MatrixMorph_base",         /*tp_name*/
     sizeof(MatrixMorph),         /*tp_basicsize*/
     0,                         /*tp_itemsize*/
@@ -1107,7 +1111,7 @@ PyTypeObject MatrixMorphType = {
     0,                         /*tp_print*/
     0,                         /*tp_getattr*/
     0,                         /*tp_setattr*/
-    0,                         /*tp_compare*/
+    0,                         /*tp_as_async (tp_compare in Python 2)*/
     0,                         /*tp_repr*/
     0,             /*tp_as_number*/
     0,                         /*tp_as_sequence*/
