@@ -19,6 +19,7 @@
  *************************************************************************/
 
 #include <Python.h>
+#include "py2to3.h"
 #include "structmember.h"
 #include "pyomodule.h"
 #include "streammodule.h"
@@ -30,6 +31,7 @@ typedef struct {
     PyObject *callable;
     PyObject *time;
     Stream *time_stream;
+    PyObject *arg;
     int modebuffer[1];
     MYFLT sampleToSec;
     double currentTime;
@@ -38,54 +40,75 @@ typedef struct {
 
 static void
 Pattern_generate_i(Pattern *self) {
+    int i;
     MYFLT tm;
-    int i, flag;
-    PyObject *result;
+    PyObject *tuple, *result;
 
-    flag = 0;
     tm = PyFloat_AS_DOUBLE(self->time);
 
+    if (self->init) {
+        self->init = 0;
+        self->currentTime = tm;
+    }
+
     for (i=0; i<self->bufsize; i++) {
         if (self->currentTime >= tm) {
-            flag = 1;
-            self->currentTime = 0.;
+            self->currentTime = 0.0;
+            if (self->arg == Py_None) {
+                result = PyObject_Call((PyObject *)self->callable, PyTuple_New(0), NULL);
+                if (result == NULL) {
+                    PyErr_Print();
+                    return;
+                }
+            } else {
+                tuple = PyTuple_New(1);
+                PyTuple_SET_ITEM(tuple, 0, self->arg);
+                result = PyObject_Call((PyObject *)self->callable, tuple, NULL);
+                if (result == NULL) {
+                    PyErr_Print();
+                    return;
+                }
+            }
         }
-
         self->currentTime += self->sampleToSec;
     }
-    if (flag == 1 || self->init == 1) {
-        self->init = 0;
-        result = PyObject_Call((PyObject *)self->callable, PyTuple_New(0), NULL);
-        if (result == NULL)
-            PyErr_Print();
-    }
 }
 
 static void
 Pattern_generate_a(Pattern *self) {
-    int i, flag;
-    PyObject *result;
+    int i;
+    PyObject *tuple, *result;
 
     MYFLT *tm = Stream_getData((Stream *)self->time_stream);
 
-    flag = 0;
+    if (self->init) {
+        self->init = 0;
+        self->currentTime = tm[0];
+    }
+
     for (i=0; i<self->bufsize; i++) {
         if (self->currentTime >= tm[i]) {
-            flag = 1;
-            self->currentTime = 0.;
+            self->currentTime = 0.0;
+            if (self->arg == Py_None) {
+                result = PyObject_Call((PyObject *)self->callable, PyTuple_New(0), NULL);
+                if (result == NULL) {
+                    PyErr_Print();
+                    return;
+                }
+            } else {
+                tuple = PyTuple_New(1);
+                PyTuple_SET_ITEM(tuple, 0, self->arg);
+                result = PyObject_Call((PyObject *)self->callable, tuple, NULL);
+                if (result == NULL) {
+                    PyErr_Print();
+                    return;
+                }
+            }
         }
-
         self->currentTime += self->sampleToSec;
     }
-    if (flag == 1 || self->init == 1) {
-        self->init = 0;
-        result = PyObject_Call((PyObject *)self->callable, PyTuple_New(0), NULL);
-        if (result == NULL)
-            PyErr_Print();
-    }
 }
 
-
 static void
 Pattern_setProcMode(Pattern *self)
 {
@@ -113,6 +136,7 @@ Pattern_traverse(Pattern *self, visitproc visit, void *arg)
     Py_VISIT(self->callable);
     Py_VISIT(self->time);
     Py_VISIT(self->time_stream);
+    Py_VISIT(self->arg);
     return 0;
 }
 
@@ -123,6 +147,7 @@ Pattern_clear(Pattern *self)
     Py_CLEAR(self->callable);
     Py_CLEAR(self->time);
     Py_CLEAR(self->time_stream);
+    Py_CLEAR(self->arg);
     return 0;
 }
 
@@ -131,20 +156,21 @@ Pattern_dealloc(Pattern* self)
 {
     pyo_DEALLOC
     Pattern_clear(self);
-    self->ob_type->tp_free((PyObject*)self);
+    Py_TYPE(self)->tp_free((PyObject*)self);
 }
 
 static PyObject *
 Pattern_new(PyTypeObject *type, PyObject *args, PyObject *kwds)
 {
     int i;
-    PyObject *timetmp=NULL, *calltmp=NULL;
+    PyObject *timetmp=NULL, *calltmp=NULL, *argtmp=NULL;
     Pattern *self;
     self = (Pattern *)type->tp_alloc(type, 0);
 
     self->time = PyFloat_FromDouble(1.);
 	self->modebuffer[0] = 0;
     self->init = 1;
+    self->arg = Py_None;
 
     INIT_OBJECT_COMMON
     Stream_setFunctionPtr(self->stream, Pattern_compute_next_data_frame);
@@ -155,9 +181,9 @@ Pattern_new(PyTypeObject *type, PyObject *args, PyObject *kwds)
     self->sampleToSec = 1. / self->sr;
     self->currentTime = 0.;
 
-    static char *kwlist[] = {"callable", "time", NULL};
+    static char *kwlist[] = {"callable", "time", "arg", NULL};
 
-    if (! PyArg_ParseTupleAndKeywords(args, kwds, "O|O", kwlist, &calltmp, &timetmp))
+    if (! PyArg_ParseTupleAndKeywords(args, kwds, "O|OO", kwlist, &calltmp, &timetmp, &argtmp))
         Py_RETURN_NONE;
 
     if (calltmp) {
@@ -168,6 +194,17 @@ Pattern_new(PyTypeObject *type, PyObject *args, PyObject *kwds)
         PyObject_CallMethod((PyObject *)self, "setTime", "O", timetmp);
     }
 
+    if (argtmp) {
+        if (PyTuple_Check(argtmp)) {
+            PyObject *argument = PyTuple_New(1);
+            PyTuple_SetItem(argument, 0, argtmp);
+            PyObject_CallMethod((PyObject *)self, "setArg", "O", argument);
+        }
+        else {
+            PyObject_CallMethod((PyObject *)self, "setArg", "O", argtmp);
+        }
+    }
+
     PyObject_CallMethod(self->server, "addStream", "O", self->stream);
 
     (*self->mode_func_ptr)(self);
@@ -212,10 +249,7 @@ Pattern_setTime(Pattern *self, PyObject *arg)
 {
 	PyObject *tmp, *streamtmp;
 
-	if (arg == NULL) {
-		Py_INCREF(Py_None);
-		return Py_None;
-	}
+    ASSERT_ARG_NOT_NULL
 
 	int isNumber = PyNumber_Check(arg);
 
@@ -241,6 +275,20 @@ Pattern_setTime(Pattern *self, PyObject *arg)
 	return Py_None;
 }
 
+static PyObject *
+Pattern_setArg(Pattern *self, PyObject *arg)
+{
+	PyObject *tmp;
+
+    tmp = arg;
+    Py_XDECREF(self->arg);
+    Py_INCREF(tmp);
+    self->arg = tmp;
+
+	Py_INCREF(Py_None);
+	return Py_None;
+}
+
 static PyMemberDef Pattern_members[] = {
 {"server", T_OBJECT_EX, offsetof(Pattern, server), 0, "Pyo server."},
 {"stream", T_OBJECT_EX, offsetof(Pattern, stream), 0, "Stream object."},
@@ -255,12 +303,12 @@ static PyMethodDef Pattern_methods[] = {
 {"stop", (PyCFunction)Pattern_stop, METH_NOARGS, "Stops computing."},
 {"setTime", (PyCFunction)Pattern_setTime, METH_O, "Sets time factor."},
 {"setFunction", (PyCFunction)Pattern_setFunction, METH_O, "Sets the function to be called."},
+{"setArg", (PyCFunction)Pattern_setArg, METH_O, "Sets function's argument."},
 {NULL}  /* Sentinel */
 };
 
 PyTypeObject PatternType = {
-PyObject_HEAD_INIT(NULL)
-0,                         /*ob_size*/
+PyVarObject_HEAD_INIT(NULL, 0)
 "_pyo.Pattern_base",         /*tp_name*/
 sizeof(Pattern),         /*tp_basicsize*/
 0,                         /*tp_itemsize*/
@@ -268,7 +316,7 @@ sizeof(Pattern),         /*tp_basicsize*/
 0,                         /*tp_print*/
 0,                         /*tp_getattr*/
 0,                         /*tp_setattr*/
-0,                         /*tp_compare*/
+0,                         /*tp_as_async (tp_compare in Python 2)*/
 0,                         /*tp_repr*/
 0,             /*tp_as_number*/
 0,                         /*tp_as_sequence*/
@@ -363,7 +411,7 @@ Score_dealloc(Score* self)
 {
     pyo_DEALLOC
     Score_clear(self);
-    self->ob_type->tp_free((PyObject*)self);
+    Py_TYPE(self)->tp_free((PyObject*)self);
 }
 
 static PyObject *
@@ -415,8 +463,7 @@ static PyMethodDef Score_methods[] = {
 };
 
 PyTypeObject ScoreType = {
-PyObject_HEAD_INIT(NULL)
-0,                         /*ob_size*/
+PyVarObject_HEAD_INIT(NULL, 0)
 "_pyo.Score_base",         /*tp_name*/
 sizeof(Score),         /*tp_basicsize*/
 0,                         /*tp_itemsize*/
@@ -424,7 +471,7 @@ sizeof(Score),         /*tp_basicsize*/
 0,                         /*tp_print*/
 0,                         /*tp_getattr*/
 0,                         /*tp_setattr*/
-0,                         /*tp_compare*/
+0,                         /*tp_as_async (tp_compare in Python 2)*/
 0,                         /*tp_repr*/
 0,             /*tp_as_number*/
 0,                         /*tp_as_sequence*/
@@ -526,7 +573,7 @@ CallAfter_dealloc(CallAfter* self)
 {
     pyo_DEALLOC
     CallAfter_clear(self);
-    self->ob_type->tp_free((PyObject*)self);
+    Py_TYPE(self)->tp_free((PyObject*)self);
 }
 
 static PyObject *
@@ -575,7 +622,10 @@ CallAfter_new(PyTypeObject *type, PyObject *args, PyObject *kwds)
 static PyObject * CallAfter_getServer(CallAfter* self) { GET_SERVER };
 static PyObject * CallAfter_getStream(CallAfter* self) { GET_STREAM };
 
-static PyObject * CallAfter_play(CallAfter *self, PyObject *args, PyObject *kwds) { PLAY };
+static PyObject * CallAfter_play(CallAfter *self, PyObject *args, PyObject *kwds) { 
+    self->currentTime = 0.;
+    PLAY 
+};
 static PyObject * CallAfter_stop(CallAfter *self) { STOP };
 
 static PyMemberDef CallAfter_members[] = {
@@ -593,8 +643,7 @@ static PyMethodDef CallAfter_methods[] = {
 };
 
 PyTypeObject CallAfterType = {
-PyObject_HEAD_INIT(NULL)
-0,                         /*ob_size*/
+PyVarObject_HEAD_INIT(NULL, 0)
 "_pyo.CallAfter_base",         /*tp_name*/
 sizeof(CallAfter),         /*tp_basicsize*/
 0,                         /*tp_itemsize*/
@@ -602,7 +651,7 @@ sizeof(CallAfter),         /*tp_basicsize*/
 0,                         /*tp_print*/
 0,                         /*tp_getattr*/
 0,                         /*tp_setattr*/
-0,                         /*tp_compare*/
+0,                         /*tp_as_async (tp_compare in Python 2)*/
 0,                         /*tp_repr*/
 0,             /*tp_as_number*/
 0,                         /*tp_as_sequence*/
